/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#include "data_types/sqlsync_data_binary.h"

c_data_type c_binary_elem_get_type()
{
  return c_binary_elem_type;
}

c_binary_elem_ptr c_binary_elem_empty ()
{
  c_binary_elem_ptr tmp;

  tmp = (c_binary_elem_ptr) std_malloc (sizeof (c_binary_elem));
  if (tmp == NULL)
    return NULL;

  tmp->_type = c_binary_elem_type;
  tmp->data = (uint8 *) std_malloc (1);
  if (tmp->data == NULL)
  {
    std_free (tmp);
    return NULL;
  }

  tmp->data[0] = '\0';
  tmp->length = 0;
  tmp->reserved_length = 1;
  return tmp;
}

c_binary_elem_ptr c_binary_elem_copy (c_binary_elem_ptr data)
{
  c_binary_elem_ptr tmp = c_binary_elem_empty();
  if (tmp == NULL)
    return NULL;

  tmp->data = std_realloc(tmp->data, data->length);
  if (tmp->data == NULL)
  {
    std_free(tmp);
    return NULL;
  }

  std_memcpy(tmp->data, data->data, data->length);
  tmp->_type = data->_type;
  tmp->reserved_length = data->length;
  tmp->length = data->length;

  return tmp;
}

void c_binary_elem_free (c_binary_elem_ptr data)
{
  if (data != NULL)
  {
    std_free (data->data);
    std_free (data);
    data = NULL;
  }
}

SQLSYNC_RETURN
c_binary_elem_append_reserve (c_binary_elem_ptr elem, uint32 length)
{
  if (elem == NULL || length == 0)
    return SQLSYNC_FALSE;

  void * tmp = NULL;

  uint32 new_size = MAX (elem->length + length, elem->reserved_length + length);

  if (new_size != elem->reserved_length)
  {
    tmp = std_realloc(elem->data, new_size);
  
    if (tmp != NULL)
    {
      elem->data = tmp;
      elem->reserved_length = new_size;
    }
    else
      return SQLSYNC_FALSE;
  }

  return SQLSYNC_TRUE;
}


int32
c_binary_elem_cmp (c_binary_elem_ptr src, c_binary_elem_ptr dst)
{
  int32 ret = 0;

  if (src == NULL)
  {
    if (dst == NULL)
      return 0;
    else
      return -1;
  }
  else if (dst == NULL)
    return 1;

  if (src->_type != dst->_type)
    return -3;                  /* Different kind of data */

  /*if (src->_type == c_binary_elem_type)*/
  {
    c_binary_elem *s, *d;

    uint32 min_len = 0;
    uint32 max_len = 0;

    s = ((c_binary_elem *) src);
    d = ((c_binary_elem *) dst);

    min_len = MIN (s->length, d->length);
    max_len = MAX (s->length, d->length);

    ret = std_memcmp (s->data, d->data, min_len);

    if (ret == 0 && min_len != max_len)
    {
      if (s->length < d->length)
        ret = -1;
      else
        ret = 1;
    }

  }
  return ret;
}


uint32 c_binary_elem_size(c_binary_elem_ptr elem)
{
  return elem->reserved_length;
}

void * c_binary_elem_data(c_binary_elem_ptr elem)
{
  return elem->data;
}

c_binary_elem_Vtbl * c_binary_elem_create_Vtbl(uint32 vtbl_size)
{
 c_binary_elem_Vtbl * vtbl = (c_binary_elem_Vtbl *) c_data_elem_create_Vtbl(vtbl_size);

  if (vtbl == NULL)
    return NULL;
  
  vtbl->elem_empty = c_binary_elem_empty;
  vtbl->elem_free = c_binary_elem_free;
  vtbl->elem_cmp = c_binary_elem_cmp;
  vtbl->elem_copy = c_binary_elem_copy;
  vtbl->get_type = c_binary_elem_get_type;
  vtbl->elem_size = c_binary_elem_size;
  vtbl->elem_data = c_binary_elem_data;

  return vtbl;
}

c_data_type c_bit_elem_get_type()
{
  return c_bit_elem_type;
}

c_bit_elem_ptr c_bit_elem_empty ()
{
  c_bit_elem_ptr tmp;

  tmp = (c_bit_elem_ptr) std_malloc (sizeof (c_bit_elem));
  if (tmp == NULL)
    return NULL;

  tmp->_type = c_bit_elem_type;
  tmp->data = (uint8 *) std_malloc (1);
  if (tmp->data == NULL)
  {
    std_free (tmp);
    return NULL;
  }

  tmp->data[0] = '\0';
  tmp->length = 0;
  tmp->reserved_length = 1;
  tmp->max_size = 0;
  return tmp;
}

c_bit_elem_ptr c_bit_elem_copy (c_bit_elem_ptr data)
{
  c_bit_elem_ptr tmp = c_bit_elem_empty();
  if (tmp == NULL)
    return NULL;

  tmp->data = std_realloc(tmp->data, data->length);
  if (tmp->data == NULL)
  {
    std_free(tmp);
    return NULL;
  }

  std_memcpy(tmp->data, data->data, data->length);
  tmp->_type = data->_type;
  tmp->reserved_length = data->length;
  tmp->length = data->length;
  tmp->max_size = data->max_size;
  return tmp;
}

void c_bit_elem_free (c_bit_elem_ptr data)
{
  if (data != NULL)
  {
    std_free (data->data);
    std_free (data);
    data = NULL;
  }
}


int32 c_bit_elem_cmp (c_bit_elem_ptr src, c_bit_elem_ptr dst)
{
  return c_binary_elem_cmp((c_binary_elem_ptr) src, (c_binary_elem_ptr) dst);
}



uint32 c_bit_elem_size(c_bit_elem_ptr elem)
{
  return elem->reserved_length;
}

void * c_bit_elem_data(c_bit_elem_ptr elem)
{
  return elem->data;
}

c_bit_elem_Vtbl * c_bit_elem_create_Vtbl(uint32 vtbl_size)
{
 c_bit_elem_Vtbl * vtbl = (c_bit_elem_Vtbl *) c_data_elem_create_Vtbl(vtbl_size);

  if (vtbl == NULL)
    return NULL;
  
  vtbl->elem_empty = c_bit_elem_empty;
  vtbl->elem_free = c_bit_elem_free;
  vtbl->elem_cmp = c_bit_elem_cmp;
  vtbl->elem_copy = c_bit_elem_copy;
  vtbl->get_type = c_bit_elem_get_type;
  vtbl->elem_size = c_bit_elem_size;
  vtbl->elem_data = c_bit_elem_data;


  return vtbl;
}
