/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#include "data_types/sqlsync_data_converter.h"

#include "data_types/sqlsync_data_binary.h"
#include "data_types/sqlsync_data_numeric.h"
#include "data_types/sqlsync_data_datetime.h"
#include "data_types/sqlsync_data_misc.h"

#include "server/sqlsync_sqlserver.h"

/*
 * Precondition:
 *  - buf, ret and mod pointers are not NULL
 *  - ret buffer is at least the same size as buf (it is filled up to SIZE bytes)
 *  - ret and buf are not overlapping
 */
void
divBy10Int (const void *buf, int size, void *ret, int *mod)
{
#ifdef BIGENDIAN
#error "Not implemented"
#endif

  int q = 0xa;

  const uint8 *inBuf = (const uint8 *) buf;
  uint8 *outBuf = (uint8 *) ret;

  inBuf += size - 1;
  outBuf += size - 1;
  uint8 carry = 0;

  while (size > 0)
  {
    unsigned cs = size;
    unsigned t = carry;

    if (t == 0)
      t = (t << 8) + *inBuf--, --size;

    if (size > 0)
      t = (t << 8) + *inBuf--, --size;

    if (size > 0)
      t = (t << 8) + *inBuf--, --size;

    if (size > 0)
      t = (t << 8) + *inBuf--, --size;

    unsigned d = t / q;

    for (; cs > size; --cs)
    {
      *outBuf-- = (d & 0xff);
      d >>= 8;
    }

    carry = t % q;
  }

  *mod = carry;
}


/**
 * Precondition:
 *  - buf, ret and mod pointers are not NULL
 *  - ret buffer is at least the same size as buf (it is filled up to SIZE bytes)
 *  - ret and buf are not overlapping
 */
void
divBy10Byte (const void *buf, int size, void *ret, int *mod)
{
  int q = 0xa;

  const uint8 *inBuf = (const uint8 *) buf;
  uint8 *outBuf = (uint8 *) ret;

#ifndef BIGENDIAN
  inBuf += size - 1;
  outBuf += size - 1;
#endif
  uint8 carry = 0;

#ifndef BIGENDIAN
  for (; size > 0; --size, --inBuf, --outBuf)
#else
  for (; size > 0; --size, ++inBuf, ++outBuf)
#endif
  {
    int t = (carry << 8) + *inBuf;

    *outBuf = t / q;
    carry = t % q;
  }

  *mod = carry;
}

uint8
isZero (const void *buf, int size)
{
  const uint8 *inBuf = (const uint8 *) buf;

  for (; size > 0; --size, ++inBuf)
  {
    if (*inBuf != 0)
      return 0;
  }

  return 1;
}

typedef void (*DivBy10Ptr) (const void *buf, int size, void *ret, int *mod);

/**
 * Precondition:
 *  - buf, and out pointers are not NULL
 * Return:
 *  - pointer to start of decoded number, inside OUT buffer (buffer is filled out from tail)
 *    or NULL if error, output buffer have to be at least as big as number to decode + 1 (for '\0')
 */
char *
num2str (const void *buf, int size, char *out, int len, DivBy10Ptr divF)
{
  out += len - 1;

  uint8 *actual_buffer = NULL;

  if (isZero (buf, size) == 1)
    return NULL;

  uint8 *buf1 = (uint8 *) std_malloc (size);

  if (buf1 == NULL)
    return NULL;

  uint8 *buf2 = (uint8 *) std_malloc (size);

  if (buf2 == NULL)
  {
    std_free (buf1);
    return NULL;
  }

  std_memcpy (buf1, buf, size);

  actual_buffer = buf1;

  *out-- = '\0';
  --len;

  int direction = 0;

  for (; len > 0 && !isZero (actual_buffer, size); --out, --len)
  {
    int mod;

    if (direction)
    {
      divF (buf2, size, buf1, &mod);
      actual_buffer = buf1;
    }
    else
    {

      divF (buf1, size, buf2, &mod);
      actual_buffer = buf2;
    }

    *out = mod + '0';
    direction = !direction;
  }

  std_free (buf1);
  std_free (buf2);

  return out + 1;
}

c_string_elem_ptr c_binary_data_to_hex(uint8 *table, uint32 size)
{
  char hex[]="0123456789ABCDEF";
  uint32 i = 0;
  c_string_elem_ptr out = c_string_elem_reserve(5);

  for (i = 0; i< size; i++)
  {
    c_string_elem_append(out, &hex[table[i] >> 4], 1);
    c_string_elem_append(out, &hex[table[i] & 0x0f], 1);
  }
  return out;
}

c_string_elem_ptr c_binary_data_to_octet(uint8 *table, uint32 size)
{
  char oct[]="01234567";
  uint32 i = 0;
  uint8 v[4];
  uint8 val = 0;
  uint8 j = 0;

  c_string_elem_ptr out = c_string_elem_reserve(5);

  for (i = 0; i< size; i++)
  {
    j = 2;
    uint8 k = 0;
    val = table[i];
    v[0] = 0;
    v[1] = 0;
    v[2] = 0;

    do
    {
      v[j] = val % 8;
      val /= 8;
      j--;
    } while(val);

    c_string_elem_append(out, "\\", 1);
    for (k = 0; k < 3; k++)
      c_string_elem_append(out, &oct[v[k]], 1);

  }
  return out;
}


c_string_elem_ptr c_bit_to_string(c_bit_elem_ptr elem)
{
  c_string_elem_ptr out = c_string_elem_reserve(5);
  uint32 i = 0;
  /* Ugly workaround but some drivers return 0 
  as a max size for bit type */
  if (elem->max_size == 0)
        elem->max_size = elem->length * 8;
  for (i = 0; i < elem->length; i++)
  {
    int8 j;
    if (i == 0)
    {
        //uint8 mod = elem->max_size % 8;
        //j = mod == 0 ? 8 : mod - 1;
        j = 7;
        for(; j >= 0; j--)
        {
          if ((elem->data[i] & (1<<j)) != 0)
            break;

          if (j == 0)
            break;
        }
        

    }
    else
      j = 7;
    for (; j >= 0; j--)
    {
      if ((elem->data[i] & (1<<j)) != 0)
        c_string_elem_append(out, "1", 1);
      else
        c_string_elem_append(out, "0", 1);
    }
  }
  return out;
}

c_string_elem_ptr
c_numeric_to_string (c_numeric_elem_ptr elem)
{
  const uint8 conv_size = 48;
  char converted[conv_size];
  char *tmp = NULL;
  uint8 tmp_size = 0;

  c_string_elem_ptr out = c_string_elem_empty ();

  tmp = num2str (elem->numeric.val, odbc_SQL_MAX_NUMERIC_LEN, converted, conv_size, divBy10Byte);

  if (tmp == NULL)
  {
    std_snprintf (converted, conv_size, "%0*d.%0*d", elem->numeric.precision, 0, elem->numeric.scale,
              0);
    c_string_elem_append_cstring (out, converted);
  }
  else
  {
    tmp_size = std_strlen (tmp);
    uint32 before_point = MAX(tmp_size - elem->numeric.scale, 0);


    if (elem->numeric.sign == 0)
      c_string_elem_append (out, "-", 1);

    if (elem->numeric.precision == 0 || before_point == 0)
      c_string_elem_append (out, "0", 1);
    else
      c_string_elem_append (out, tmp, before_point);

    c_string_elem_append (out, ".", 1);

    c_string_elem_append (out, tmp + before_point, elem->numeric.scale);
  }
  return out;
}

c_string_elem_ptr c_data_convert_escapes(c_string_elem_ptr elem, char escape, char * chars_to_escape)
{
    c_string_elem_ptr string_elem = elem;
    c_string_elem_ptr escaped_string = c_string_elem_empty ();
    uint32 i = 0;
    uint32 last_pos = 0;
    
    for (i = 0; i < string_elem->length; i++)
    {
      if (string_elem->string[i] !=0 && std_strchr(chars_to_escape, string_elem->string[i])!= NULL)
      {
        c_string_elem_append (escaped_string, (char *) (string_elem->string + last_pos),
                               i - last_pos);
        c_string_elem_append (escaped_string, &escape, 1);
        last_pos = i;
      }
      else if (string_elem->string[i] == '\r' && i + 1 < string_elem->length
               && string_elem->string[i + 1] == '\n')
      {
        c_string_elem_append (escaped_string, (char *) (string_elem->string + last_pos),
                               i - last_pos);
        last_pos = i + 1;
      }
    }
    c_string_elem_append (escaped_string, string_elem->string + last_pos, i - last_pos);

    return escaped_string;
}

#define c_nhtol(val) ((val >> 24) | ((val >> 16) & 0xff) << 8 | ((val >> 8) & 0xff) <<16 | (val & 0xff) << 24)
#define c_nhtos(val) (((val >> 8) & 0xff) | ((val & 0xff) << 8))

c_string_elem_ptr
c_data_convert (sqlsync_sqlserver * server, c_data_elem_ptr elem, SQLSYNC_RETURN with_escapes)
{
  c_string_elem_ptr out = NULL;

  if (elem == NULL)
    return NULL;

  switch (elem->_type)
  {
    case c_string_elem_type:
    {
      if (with_escapes == SQLSYNC_TRUE)
      {
        return sqlsync_sqlserver_data_to_string(server, elem, with_escapes);
      }
      else
        return c_string_elem_copy ((c_string_elem_ptr) elem);
      break;
    }

    case c_guid_elem_type:
    {
      c_string_elem_ptr tmp = NULL;
      c_guid_elem_ptr guid_elem = (c_guid_elem_ptr) elem;
      out = c_string_elem_reserve(40);

      guid_elem->guid.Data1 = c_nhtol(guid_elem->guid.Data1);
      tmp = c_binary_data_to_hex((uint8 *)&guid_elem->guid.Data1, sizeof(guid_elem->guid.Data1));
      c_string_elem_append_string(out, tmp);
      c_string_elem_append_cstring(out, "-");
      c_string_elem_free(tmp);

      guid_elem->guid.Data2 = c_nhtos(guid_elem->guid.Data2);
      tmp = c_binary_data_to_hex((uint8 *)&guid_elem->guid.Data2, sizeof(guid_elem->guid.Data2));
      c_string_elem_append_string(out, tmp);
      c_string_elem_append_cstring(out, "-");
      c_string_elem_free(tmp);

      guid_elem->guid.Data3 = c_nhtos(guid_elem->guid.Data3);
      tmp = c_binary_data_to_hex((uint8 *)&guid_elem->guid.Data3, sizeof(guid_elem->guid.Data3));
      c_string_elem_append_string(out, tmp);
      c_string_elem_append_cstring(out, "-");
      c_string_elem_free(tmp);

      tmp = c_binary_data_to_hex(guid_elem->guid.Data4, 2);
      c_string_elem_append_string(out, tmp);
      c_string_elem_append_cstring(out, "-");
      c_string_elem_free(tmp);


      tmp = c_binary_data_to_hex(guid_elem->guid.Data4 + 2, sizeof(guid_elem->guid.Data4) - 2);
      c_string_elem_append_string(out, tmp);
      c_string_elem_free(tmp);


      break;

    }

    case c_bool_elem_type:
    {
      c_uint8_elem_ptr bool_elem = (c_uint8_elem_ptr) elem;
      out = c_string_elem_reserve (6);
      if (bool_elem->value == 0)
				c_string_elem_append(out, "false", 5);
      else
				c_string_elem_append(out, "true", 4);

      break;
    }
    case c_bit_elem_type:
    {
      c_bit_elem_ptr bit_elem = (c_bit_elem_ptr) elem;
      out = c_bit_to_string(bit_elem);
      break;
    }
    case c_binary_elem_type:
    {
      c_binary_elem_ptr binary_elem = (c_binary_elem_ptr) elem;
      if (binary_elem->length == 0)
        out = c_string_elem_empty();
      else
        out = c_binary_data_to_hex(binary_elem->data, binary_elem->length);
      break;
    }
    
    case c_uint8_elem_type:
    {
      c_uint8_elem_ptr data = (c_uint8_elem_ptr) elem;

      out = c_string_elem_reserve (4);
      out->length = std_snprintf (out->string, 4, "%d", data->value);
      break;
    }
    case c_int8_elem_type:
    {
      c_int8_elem_ptr data = (c_int8_elem_ptr) elem;

      out = c_string_elem_reserve (5);
      out->length = std_snprintf (out->string, 5, "%d", data->value);
      break;
    }
    case c_uint16_elem_type:
    {
      c_uint16_elem_ptr data = (c_uint16_elem_ptr) elem;

      out = c_string_elem_reserve (6);
      out->length = std_snprintf (out->string, 6, "%d", data->value);
      break;
    }
    case c_int16_elem_type:
    {
      c_int16_elem_ptr data = (c_int16_elem_ptr) elem;

      out = c_string_elem_reserve (7);
      out->length = std_snprintf (out->string, 7, "%d", data->value);
      break;
    }
    case c_uint32_elem_type:
    {
      c_uint32_elem_ptr data = (c_uint32_elem_ptr) elem;

      out = c_string_elem_reserve (11);
      out->length = std_snprintf (out->string, 11, "%d", data->value);
      break;
    }
    case c_int32_elem_type:
    {
      c_int32_elem_ptr data = (c_int32_elem_ptr) elem;

      out = c_string_elem_reserve (12);
      out->length = std_snprintf (out->string, 12, "%d", data->value);
      break;
    }

    case c_bigint_elem_type:
    {
      c_bigint_elem_ptr data = (c_bigint_elem_ptr) elem;

      out = c_string_elem_reserve (21);
#if (defined(_WINDOWS) || defined(__MINGW32__))
      out->length = std_snprintf (out->string, 21, "%I64d", data->bigint);
#else
      out->length = std_snprintf (out->string, 21, "%lld", data->bigint);
#endif
      break;
    }

    case c_float_elem_type:
    {
      c_float_elem_ptr data = (c_float_elem_ptr) elem;

      out = c_string_elem_reserve (26);
      out->length = std_snprintf (out->string, 26, "%.6f", data->value);
      break;

    }
    case c_double_elem_type:
    {
      c_double_elem_ptr data = (c_double_elem_ptr) elem;

      out = c_string_elem_reserve (54);
      out->length = std_snprintf (out->string, 54, "%.15f", data->value);
      break;
    }
    case c_date_elem_type:
    {
      out = c_string_elem_reserve (11);
      c_date_elem_ptr date_elem = (c_date_elem_ptr) elem;

      out->length =
        std_snprintf (out->string, 11, "%.4d-%.2d-%.2d", date_elem->date.year, date_elem->date.month,
                  date_elem->date.day);
      break;
    }

    case c_time_elem_type:
    {
      out = c_string_elem_reserve (9);
      c_time_elem_ptr time_elem = (c_time_elem_ptr) elem;

      out->length =
        std_snprintf (out->string, 9, "%.2d:%.2d:%.2d", time_elem->time.hour, time_elem->time.minute,
                  time_elem->time.second);
      break;
    }

    case c_timestamp_elem_type:
    {
      out = c_string_elem_reserve (24);
      c_timestamp_elem_ptr timestamp_elem = (c_timestamp_elem_ptr) elem;

      out->length =
        std_snprintf (out->string, 24, "%.4d-%.2d-%.2d %.2d:%.2d:%.2d.%.03d",
                  timestamp_elem->timestamp.year, timestamp_elem->timestamp.month,
                  timestamp_elem->timestamp.day, timestamp_elem->timestamp.hour,
                  timestamp_elem->timestamp.minute, timestamp_elem->timestamp.second,
                  timestamp_elem->timestamp.fraction);
      out->length = 23;
      break;
    }

    case c_numeric_elem_type:
    {
      out = c_numeric_to_string ((c_numeric_elem_ptr) elem);
      break;
    }


    default:
      return NULL;
  }

  return out;
}
