/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#include "data_types/sqlsync_data_datetime.h"

c_data_type c_date_elem_get_type()
{
  return c_date_elem_type;
}

c_date_elem_ptr c_date_elem_empty()
{
  c_date_elem_ptr tmp = std_malloc (sizeof (c_date_elem));

  if (tmp == NULL)
    return NULL;

  std_memset (tmp, 0, sizeof (c_date_elem));
  tmp->_type = c_date_elem_type;

  return tmp;
}

void c_date_elem_free(c_date_elem_ptr elem)
{
  std_free(elem);
}

c_date_elem_ptr c_date_elem_copy(c_date_elem_ptr elem)
{
  c_date_elem_ptr out = c_date_elem_empty();
  if (out == NULL)
    return NULL;

  out->_type = elem->_type;
  out->date = elem->date;
  return out;
}

int32 c_date_elem_cmp(c_date_elem_ptr src, c_date_elem_ptr dst)
{
  int32 ret = 0;

  if (src == NULL)
  {
    if (dst == NULL)
      return 0;
    else
      return -1;
  }
  else if (dst == NULL)
    return 1;

  ret = CMP_RESULT(src->date.year, dst->date.year);
  if (ret != 0)
    return ret;

  ret = CMP_RESULT(src->date.month, dst->date.month);
  if (ret != 0)
    return ret;

  return CMP_RESULT(src->date.day, dst->date.day);
}


uint32 c_date_elem_size(c_date_elem_ptr elem)
{
  return sizeof(odbc_date_struct);
}

void * c_date_elem_data(c_date_elem_ptr elem)
{
  return &elem->date;
}

c_date_elem_Vtbl * c_date_elem_create_Vtbl(uint32 vtbl_size)
{
  c_date_elem_Vtbl * vtbl = (c_date_elem_Vtbl *) c_data_elem_create_Vtbl(vtbl_size);

  if (vtbl == NULL)
    return NULL;
  
  vtbl->elem_empty = c_date_elem_empty;
  vtbl->elem_free = c_date_elem_free;
  vtbl->elem_cmp = c_date_elem_cmp;
  vtbl->elem_copy = c_date_elem_copy;
  vtbl->get_type = c_date_elem_get_type;
  vtbl->elem_size = c_date_elem_size;
  vtbl->elem_data = c_date_elem_data;


  return vtbl;
}

c_data_type c_time_elem_get_type()
{
  return c_time_elem_type;
}

c_time_elem_ptr c_time_elem_empty()
{
  c_time_elem_ptr tmp = std_malloc (sizeof (c_time_elem));

  if (tmp == NULL)
    return NULL;

  std_memset (tmp, 0, sizeof (c_time_elem));

  tmp->_type = c_time_elem_type;
  return tmp;
}


int32 c_time_elem_cmp(c_time_elem_ptr src, c_time_elem_ptr dst)
{
  int32 ret = 0;

  if (src == NULL)
  {
    if (dst == NULL)
      return 0;
    else
      return -1;
  }
  else if (dst == NULL)
    return 1;

  ret = CMP_RESULT(src->time.hour, dst->time.hour);
  if (ret != 0)
    return ret;

  ret = CMP_RESULT(src->time.minute, dst->time.minute);
  if (ret != 0)
    return ret;

  return CMP_RESULT(src->time.second, dst->time.second);
}

void c_time_elem_free(c_time_elem_ptr data)
{
  if (data != NULL)
    std_free (data);
}

c_time_elem_ptr c_time_elem_copy(c_time_elem_ptr data)
{
  c_time_elem_ptr elem = c_time_elem_empty ();

  if (elem == NULL || data == NULL)
  {
    c_time_elem_free (elem);
    return NULL;
  }

  elem->_type = data->_type;
  std_memcpy (elem, data, sizeof (c_time_elem));
  return elem;
}


uint32 c_time_elem_size(c_time_elem_ptr elem)
{
  return sizeof(odbc_time_struct);
}

void * c_time_elem_data(c_time_elem_ptr elem)
{
  return &elem->time;
}

c_time_elem_Vtbl * c_time_elem_create_Vtbl(uint32 vtbl_size)
{
  c_time_elem_Vtbl * vtbl = (c_time_elem_Vtbl *) c_data_elem_create_Vtbl(vtbl_size);

  if (vtbl == NULL)
    return NULL;
  
  vtbl->elem_empty = c_time_elem_empty;
  vtbl->elem_free = c_time_elem_free;
  vtbl->elem_cmp = c_time_elem_cmp;
  vtbl->elem_copy = c_time_elem_copy;
  vtbl->get_type = c_time_elem_get_type;
  vtbl->elem_size = c_time_elem_size;
  vtbl->elem_data = c_time_elem_data;

  return vtbl;
}


c_data_type c_timestamp_elem_get_type()
{
  return c_timestamp_elem_type;
}

c_timestamp_elem_ptr c_timestamp_elem_empty()
{
  c_timestamp_elem_ptr tmp = std_malloc (sizeof (c_timestamp_elem));

  if (tmp == NULL)
    return NULL;

  std_memset (tmp, 0, sizeof (c_timestamp_elem));
  tmp->_type = c_timestamp_elem_type;
  return tmp;

}

int32 c_timestamp_elem_cmp(c_timestamp_elem_ptr src, c_timestamp_elem_ptr dst)
{
	int32 ret = 0;
    if (src == NULL)
    {
      if (dst == NULL)
        return 0;
      else
        return -1;
    }
    else if (dst == NULL)
      return 1;

  ret = CMP_RESULT(src->timestamp.year, dst->timestamp.year);
  if (ret != 0)
    return ret;

  ret = CMP_RESULT(src->timestamp.month, dst->timestamp.month);
  if (ret != 0)
    return ret;

  ret = CMP_RESULT(src->timestamp.day, dst->timestamp.day);
  if (ret != 0)
    return ret;

  ret = CMP_RESULT(src->timestamp.hour, dst->timestamp.hour);
  if (ret != 0)
    return ret;

  ret = CMP_RESULT(src->timestamp.minute, dst->timestamp.minute);
  if (ret != 0)
    return ret;

  ret = CMP_RESULT(src->timestamp.second, dst->timestamp.second);
  if (ret != 0)
    return ret;

  return CMP_RESULT(src->timestamp.fraction, dst->timestamp.fraction);

}

void c_timestamp_elem_free (c_timestamp_elem_ptr data)
{
  if (data != NULL)
    std_free (data);
}


c_timestamp_elem_ptr c_timestamp_elem_copy(c_timestamp_elem_ptr data)
{
  c_timestamp_elem_ptr elem = c_timestamp_elem_empty ();

  if (elem == NULL || data == NULL)
  {
    c_timestamp_elem_free (elem);
    return NULL;
  }

  elem->_type = data->_type;
  std_memcpy (elem, data, sizeof (c_timestamp_elem));
  return elem;
}


uint32 c_timestamp_elem_size(c_timestamp_elem_ptr elem)
{
  return sizeof(odbc_timestamp_struct);
}

void * c_timestamp_elem_data(c_timestamp_elem_ptr elem)
{
  return &elem->timestamp;
}

c_timestamp_elem_Vtbl * c_timestamp_elem_create_Vtbl(uint32 vtbl_size)
{
  c_timestamp_elem_Vtbl * vtbl = (c_timestamp_elem_Vtbl *) c_data_elem_create_Vtbl(vtbl_size);

  if (vtbl == NULL)
    return NULL;
  
  vtbl->elem_empty = c_timestamp_elem_empty;
  vtbl->elem_free = c_timestamp_elem_free;
  vtbl->elem_cmp = c_timestamp_elem_cmp;
  vtbl->elem_copy = c_timestamp_elem_copy;
  vtbl->get_type = c_timestamp_elem_get_type;
  vtbl->elem_size = c_timestamp_elem_size;
  vtbl->elem_data = c_timestamp_elem_data;

  return vtbl;
}
