/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#ifndef SQLSYNC_DATA_DATETIME_H
#define SQLSYNC_DATA_DATETIME_H

#include "data_types/sqlsync_data_type.h"

#define INHERIT_date_elem(type) \
          INHERIT_data_elem(type)

typedef struct
{
  int16 year;
  uint16 month;
  uint16 day;
} odbc_date_struct;

#define INHERIT_OBJECT_date_elem(type) \
          INHERIT_OBJECT_data_elem(type); \
          odbc_date_struct date;

#define INHERIT_FUNCTIONS_date_elem(type)

DEFINE_INTERFACE_STRUCT(date_elem);
DEFINE_INTERFACE(date_elem);


c_date_elem_ptr c_date_elem_empty();

void c_date_elem_free(c_date_elem_ptr elem);

c_date_elem_ptr c_date_elem_copy(c_date_elem_ptr elem);

int32 c_date_elem_cmp(c_date_elem_ptr src, c_date_elem_ptr dst);

c_date_elem_Vtbl * c_date_elem_create_Vtbl(uint32 vtbl_size);


#define INHERIT_time_elem(type) \
          INHERIT_data_elem(type)


typedef struct
{
  uint16 hour;
  uint16 minute;
  uint16 second;
} odbc_time_struct;

#define INHERIT_OBJECT_time_elem(type) \
          INHERIT_OBJECT_data_elem(type); \
          odbc_time_struct time;

#define INHERIT_FUNCTIONS_time_elem(type)

DEFINE_INTERFACE_STRUCT(time_elem);
DEFINE_INTERFACE(time_elem);

c_time_elem_ptr c_time_elem_empty();

int32 c_time_elem_cmp(c_time_elem_ptr src, c_time_elem_ptr dst);

void c_time_elem_free(c_time_elem_ptr data);

c_time_elem_ptr c_time_elem_copy(c_time_elem_ptr data);

c_time_elem_Vtbl * c_time_elem_create_Vtbl(uint32 vtbl_size);



typedef struct
{
  int16 year;
  uint16 month;
  uint16 day;
  uint16 hour;
  uint16 minute;
  uint16 second;
  uint32 fraction;
} odbc_timestamp_struct;

#define INHERIT_timestamp_elem(type) \
          INHERIT_data_elem(type)

#define INHERIT_OBJECT_timestamp_elem(type) \
          INHERIT_OBJECT_data_elem(type); \
          odbc_timestamp_struct timestamp;

#define INHERIT_FUNCTIONS_timestamp_elem(type)

DEFINE_INTERFACE_STRUCT(timestamp_elem);
DEFINE_INTERFACE(timestamp_elem);


c_timestamp_elem_ptr c_timestamp_elem_empty();

int32 c_timestamp_elem_cmp(c_timestamp_elem_ptr src, c_timestamp_elem_ptr dst);

void c_timestamp_elem_free (c_timestamp_elem_ptr data);

c_timestamp_elem_ptr c_timestamp_elem_copy(c_timestamp_elem_ptr data);

c_timestamp_elem_Vtbl * c_timestamp_elem_create_Vtbl(uint32 vtbl_size);


#if 0 /* Not implemented */

typedef enum
{
  odbc_SQL_IS_YEAR = 1,
  odbc_SQL_IS_MONTH = 2,
  odbc_SQL_IS_DAY = 3,
  odbc_SQL_IS_HOUR = 4,
  odbc_SQL_IS_MINUTE = 5,
  odbc_SQL_IS_SECOND = 6,
  odbc_SQL_IS_YEAR_TO_MONTH = 7,
  odbc_SQL_IS_DAY_TO_HOUR = 8,
  odbc_SQL_IS_DAY_TO_MINUTE = 9,
  odbc_SQL_IS_DAY_TO_SECOND = 10,
  odbc_SQL_IS_HOUR_TO_MINUTE = 11,
  odbc_SQL_IS_HOUR_TO_SECOND = 12,
  odbc_SQL_IS_MINUTE_TO_SECOND = 13
} odbc_interval;


typedef struct
{
  uint32 year;
  uint32 month;
} odbc_year_month_struct;

typedef struct
{
  uint32 day;
  uint32 hour;
  uint32 minute;
  uint32 second;
  uint32 fraction;
} odbc_day_second_struct;

typedef struct
{
  odbc_interval interval_type;
  int16 interval_sign;
  union
  {
    odbc_year_month_struct year_month;
    odbc_day_second_struct day_second;
  } intval;

} odbc_interval_struct;

#define INHERIT_interval_elem(type) \
          INHERIT_data_elem(type) \


#define INHERIT_OBJECT_interval_elem(type) \
          INHERIT_OBJECT_data_elem(type); \
          odbc_interval_struct interval;

#define INHERIT_FUNCTIONS_interval_elem(type)

DEFINE_INTERFACE_STRUCT(interval_elem);
DEFINE_INTERFACE(interval_elem);

#endif /* 0 */

#endif /* SQLSYNC_DATA_DATETIME_H */

