/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#include "sqlsync_debug.h"

#include "data_types/sqlsync_data_grid.h"
#include "data_types/sqlsync_data_numeric.h"

c_vector_int32_ptr sort_positions = NULL;


int
c_grid_data_sort_cmp_elems (const void *_src, const void *_dst)
{
  c_vector_data_ptr src = *(c_vector_data_ptr *) _src;
  c_vector_data_ptr dst = *(c_vector_data_ptr *) _dst;

  int size = c_vector_int32_size (sort_positions);
  int i = 0;
  int ret = 0;

  for (i = 0; i < size; i++)
  {
    int32 pos = (c_vector_int32_pos (sort_positions, i))->value;
    c_data_elem_ptr s = c_vector_data_pos (src, pos);
    c_data_elem_ptr d = c_vector_data_pos (dst, pos);

    ret = c_data_elem_cmp (s, d);

#ifdef DEBUG
    sqlsync_debug_details ("Result of comparing two elements %d\n", ret);
#endif

    if (ret != 0)
      return ret;
  }
  return ret;
}

SQLSYNC_RETURN
c_grid_data_sort_by_primary_keys_pos (c_grid_data_ptr grid, c_vector_int32_ptr primary_keys_pos)
{
  if (grid == NULL)
    return SQLSYNC_FAILED_NULL;

#ifdef DEBUG
  sqlsync_debug ("Primary key positions list used to sort table %p\n", primary_keys_pos);
#endif

  (void) c_grid_data_sort (grid, primary_keys_pos);

  return SQLSYNC_SUCCESS;
}


SQLSYNC_RETURN
c_grid_data_sort (c_grid_data_ptr grid, c_vector_int32_ptr positions)
{
  if (grid == NULL)
    return SQLSYNC_FAILED_NULL;

  sort_positions = positions;
  std_qsort (grid->array, c_grid_data_size (grid), sizeof (c_data_elem_ptr *),
         c_grid_data_sort_cmp_elems);

  sort_positions = NULL;

  return SQLSYNC_SUCCESS;
}

SQLSYNC_RETURN
c_grid_data_sort_by_all (c_grid_data_ptr grid)
{
  uint32 i = 0;
  uint32 max_cols = 0;
  c_vector_int32_ptr positions = NULL;

  max_cols = c_vector_data_size (c_grid_data_pos (grid, 0));

  if (max_cols == 0)
    return SQLSYNC_FAILED;

  positions = c_vector_int32_init (max_cols);

  for (i = 0; i < max_cols; i++)
  {
    c_int32_elem val;

    val._type = c_int32_elem_type;
    val.value = i;
    c_vector_int32_add_last_copy (positions, &val);
  }

  (void) c_grid_data_sort (grid, positions);

  c_vector_int32_destroy (positions);
  return SQLSYNC_SUCCESS;
}
