/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#include "data_types/sqlsync_data_misc.h"


c_data_type c_bool_elem_get_type()
{
  return c_bool_elem_type;
}

c_bool_elem_ptr c_bool_elem_empty ()
{
  c_bool_elem_ptr tmp = std_malloc(sizeof(c_bool_elem));
  if (tmp == NULL)
    return NULL;
  
  tmp->value = 0;
  tmp->_type = c_bool_elem_type;

  return tmp;
}

void c_bool_elem_free (c_bool_elem_ptr data)
{
  if (data != NULL)
    std_free(data);
}

c_bool_elem_ptr c_bool_elem_copy (c_bool_elem_ptr data)
{
  c_bool_elem_ptr copy = c_bool_elem_empty();
  if (data == NULL)
  {
    c_bool_elem_free(copy);
    return NULL;
  }
  *copy = *data;
  return copy;
}

int32 c_bool_elem_cmp (c_bool_elem_ptr src, c_bool_elem_ptr dst)
{
  if (src == NULL)
  {
    if (dst == NULL)
      return 0;
    else
      return -1;
  }
  else if (dst == NULL)
    return 1;

  return CMP_RESULT(src->value, dst->value);
}

uint32 c_bool_elem_size(c_bool_elem_ptr elem)
{
  return sizeof(uint16);
}

void * c_bool_elem_data(c_bool_elem_ptr elem)
{
  return &elem->value;
}

c_bool_elem_Vtbl * c_bool_elem_create_Vtbl(uint32 vtbl_size)
{
  c_bool_elem_Vtbl * vtbl = (c_bool_elem_Vtbl *) c_data_elem_create_Vtbl(vtbl_size);

  if (vtbl == NULL)
    return NULL;
  
  vtbl->elem_empty = c_bool_elem_empty;
  vtbl->elem_free = c_bool_elem_free;
  vtbl->elem_cmp = c_bool_elem_cmp;
  vtbl->elem_copy = c_bool_elem_copy;
  vtbl->get_type = c_bool_elem_get_type;
  vtbl->elem_size = c_bool_elem_size;
  vtbl->elem_data = c_bool_elem_data;

  return vtbl;
}


c_guid_elem_ptr c_guid_elem_empty()
{
  c_guid_elem_ptr tmp = std_malloc(sizeof(c_guid_elem));
  if (tmp == NULL)
    return NULL;
  
  std_memset(&tmp->guid, 0, sizeof(odbc_guid_struct));
  tmp->_type = c_guid_elem_type;

  return tmp;
}

void c_guid_elem_free(c_guid_elem_ptr elem)
{
  std_free(elem);
}

c_guid_elem_ptr c_guid_elem_copy(c_guid_elem_ptr elem)
{
  c_guid_elem_ptr copy = c_guid_elem_empty();
  if (elem == NULL)
  {
    c_guid_elem_free(copy);
    return NULL;
  }
  *copy = *elem;
  return copy;
}

int32 c_guid_elem_cmp(c_guid_elem_ptr src, c_guid_elem_ptr dst)
{
  int32 ret = 0;
  if (src == NULL)
  {
    if (dst == NULL)
      return 0;
    else
      return -1;
  }
  else if (dst == NULL)
    return 1;

  ret = CMP_RESULT(src->guid.Data1,dst->guid.Data1);
  if (ret != 0)
    return ret;

  ret = CMP_RESULT(src->guid.Data2, dst->guid.Data2);
  if (ret != 0)
    return ret;

  ret = CMP_RESULT(src->guid.Data3,dst ->guid.Data3);
  if (ret != 0)
    return ret;


  return std_memcmp(src->guid.Data4, dst->guid.Data4, 8);
}

c_data_type c_guid_elem_get_type()
{
  return c_guid_elem_type;
}

void * c_guid_elem_data(c_guid_elem_ptr elem)
{
  return &elem->guid;
}

uint32 c_guid_elem_size(c_guid_elem_ptr elem)
{
  return sizeof(odbc_guid_struct);
}

c_guid_elem_Vtbl * c_guid_elem_create_Vtbl(uint32 vtbl_size)
{
  c_guid_elem_Vtbl * vtbl = (c_guid_elem_Vtbl *) c_data_elem_create_Vtbl(vtbl_size);

  if (vtbl == NULL)
    return NULL;
  vtbl->elem_empty = c_guid_elem_empty;
  vtbl->elem_free = c_guid_elem_free;
  vtbl->elem_cmp = c_guid_elem_cmp;
  vtbl->elem_copy = c_guid_elem_copy;
  vtbl->get_type = c_guid_elem_get_type;
  vtbl->elem_size = c_guid_elem_size;
  vtbl->elem_data = c_guid_elem_data;
  return vtbl;
}


c_column_elem_ptr c_column_elem_copy(c_column_elem_ptr data)
{
  c_column_elem_ptr elem = NULL;

  if (data == NULL || data->column == NULL)
    return NULL;

  elem = (c_column_elem_ptr) std_malloc (sizeof (c_column_elem));
  if (elem == NULL)
    return NULL;

  elem->column = c_string_elem_empty ();

  if (elem->column == NULL)
  {
    std_free (elem);
    return NULL;
  }

  elem->_type = c_column_elem_type;
  elem->max_length = data->max_length;
  elem->data_type = data->data_type;
  elem->use_operator = data->use_operator;
  c_string_elem_append_string (elem->column, data->column);

  return elem;
}

void c_column_elem_free(c_column_elem_ptr data)
{
  if (data != NULL)

  {
    if (data->column != NULL)
    {
      if (data->column->string != NULL)
        std_free (data->column->string);

      std_free (data->column);
    }
    std_free (data);
  }
}

int32 c_column_elem_cmp (c_column_elem_ptr src, c_column_elem_ptr dst)
{
  if (src == 0)
  {
    if (dst == 0)
      return 0;
    else
      return -1;
  }
  else if (dst == 0)
    return 1;

  return c_string_elem_cmp (src->column, dst->column);
}

c_column_elem_ptr
c_column_elem_empty ()
{
  c_column_elem_ptr elem = (c_column_elem_ptr) std_malloc (sizeof (c_column_elem));

  if (elem == NULL)
    return NULL;

  elem->_type = c_column_elem_type;
  elem->column = c_string_elem_empty ();

  if (elem->column == NULL)
  {
    c_column_elem_free (elem);
    return NULL;
  }

  elem->max_length = 0;
  elem->data_type = 0;
  return elem;
}

c_data_type c_column_elem_get_type()
{
  return c_column_elem_type;
}


c_column_elem_ptr
c_column_elem_create (c_string_elem_ptr name, uint32 max_size, uint32 data_type, int8 use_operator)
{

  c_column_elem_ptr elem = c_column_elem_empty ();

  c_string_elem_append_string (elem->column, name);
  elem->max_length = max_size;
  elem->data_type = data_type;
  elem->_type = c_column_elem_type;
  elem->use_operator = use_operator;
  return elem;
}


uint32 c_column_elem_size(c_column_elem_ptr elem)
{
  return 0;
}

void * c_column_elem_data(c_column_elem_ptr elem)
{
  return elem->column->string;
}

c_column_elem_Vtbl * c_column_elem_create_Vtbl(uint32 vtbl_size)
{
 c_column_elem_Vtbl * vtbl = (c_column_elem_Vtbl *) c_data_elem_create_Vtbl(vtbl_size);

  if (vtbl == NULL)
    return NULL;
  
  vtbl->elem_empty = c_column_elem_empty;
  vtbl->elem_free = c_column_elem_free;
  vtbl->elem_cmp = c_column_elem_cmp;
  vtbl->elem_copy = c_column_elem_copy;
  vtbl->get_type = c_column_elem_get_type;
  vtbl->elem_size = c_column_elem_size;
  vtbl->elem_data = c_column_elem_data;

  return vtbl;
}

