/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#include "sqlsync_debug.h"

#include "sqlsync_compare_results.h"

#include "sqlsync_data_converter.h"

#include "sqlsync_user_internal.h"

#include "server/sqlsync_sqlserver.h"


const uint8 max_print_column_size = 255;

void
c_data_elem_print_in_row (c_data_elem_ptr elem, uint32 i, void *user)
{
  char pattern[] = "%*s|";
  c_column_elem_ptr column = NULL;
  //sqlsync_sqlserver * server = sqlsync_sqlserver_create_dummy(sqlsync_server_unknown);
  c_string_elem_ptr converted = c_data_convert (NULL, elem, SQLSYNC_FALSE);
  uint32 size;

  if (user != NULL)
    column = c_vector_columns_pos ((c_vector_columns_ptr) (user), i);
  else
    column = NULL;

  if (converted != NULL)
  {
    if (column == NULL)
      size = converted->length;
    else
    {
      size = MAX (column->column->length, column->max_length);
      size = MIN (size, max_print_column_size);
    }

    sqlsync_printf (SQLSYNC_PRINT, pattern, size, converted->string, 0);

  }
  else if (column != NULL)
  {
    size = MAX (column->column->length, column->max_length);
    size = MIN (size, max_print_column_size);

    sqlsync_printf (SQLSYNC_PRINT, pattern, size, "", 0);
  }
  else
    sqlsync_printf (SQLSYNC_PRINT, "|", 0);

  c_string_elem_free (converted);
  //sqlsync_sqlserver_destroy(server);

}

void
c_string_elem_print_in_row (c_string_elem_ptr elem, uint32 i, c_vector_columns_ptr columns)
{
  char pattern[] = "%*s|";
  c_column_elem_ptr column = NULL;

  uint32 size;

  if (columns != NULL)
    column = c_vector_columns_pos ((c_vector_columns_ptr) columns, i);
  else
    column = NULL;

  if (elem != NULL)
  {
    if (column == NULL)
      size = elem->length;
    else
    {
      size = MAX (column->column->length, column->max_length);
      size = MIN (size, max_print_column_size);
    }

    sqlsync_printf (SQLSYNC_PRINT, pattern, size, elem->string, 0);

  }
  else if (column != NULL)
  {
    size = MAX (column->column->length, column->max_length);
    size = MIN (size, max_print_column_size);
    sqlsync_printf (SQLSYNC_PRINT, pattern, size, "", 0);
  }
  else
    sqlsync_printf (SQLSYNC_PRINT, "|", 0);

}


void
c_grid_data_elem_print (c_vector_data_ptr container, uint32 pos, void *user)
{
  c_vector_columns_ptr columns = user;

  pos = 0;                      /* Dummy code - compiler warning */
  if (container)
  {
    c_vector_data_all_elems (container, c_data_elem_print_in_row, columns);
    sqlsync_printf (SQLSYNC_PRINT, "\n", 0);
  }
}
#ifdef SQLSYNC_DEPRECATED
void
c_grid_data_all_different_elems (c_grid_data_ptr grid_src,
                                 c_grid_data_ptr grid_dst,
                                 c_vector_columns_ptr columns, c_grid_data_one_elem_t f)
{
  uint32 i = 0;
  c_strings_ptr elem_src;
  c_strings_ptr elem_dst;

  if (grid_src == NULL || grid_dst == NULL)
    return;

  for (i = 0; i < c_grid_data_size (grid_src); i++)
  {
    elem_src = c_grid_data_pos (grid_src, i);
    elem_dst = c_grid_data_pos (grid_dst, i);
    f (elem_src, i, columns);
    f (elem_dst, i, columns);
  }
}


#endif

void
c_string_elem_print (c_string_elem_ptr elem, uint32 i, void *user)
{
  i = 0;                        /* Dummy code - compiler warning */
  user = NULL;                  /* Dummy code - compiler warning */

  sqlsync_write (SQLSYNC_PRINT, elem->string, elem->length);
  sqlsync_printf (SQLSYNC_PRINT, "\n", 0);
}



void
c_column_elem_print (c_column_elem_ptr elem, uint32 i, void /*@unused@ */ *user)
{
  char pattern[] = "%*s";
  uint32 len = 0;

  i = 0;                        /* Dummy code - compiler warning */
  user = NULL;                  /* Dummy code - compiler warning */

  if (elem == NULL)
    return;

  if (elem->column == NULL)
  {
    sqlsync_printf (SQLSYNC_PRINT, "|", 0);
    return;
  }

  len = MAX ((int32) (MIN (max_print_column_size, elem->max_length) - elem->column->length), 0);

  sqlsync_printf (SQLSYNC_PRINT, pattern, len, "");
  sqlsync_prints (SQLSYNC_PRINT, elem->column);
  sqlsync_printf (SQLSYNC_PRINT, "|", 0);
}



void
sqlsync_user_send_elem_in_row (c_data_elem_ptr elem, uint32 i, void *user)
{

  sqlsync_user_data row_data;
  //sqlsync_sqlserver * server = sqlsync_sqlserver_create_dummy(sqlsync_server_unknown);
  c_string_elem_ptr converted = c_data_convert (NULL, elem, SQLSYNC_FALSE);
  if (converted == NULL)
  {
    converted = c_string_elem_empty();
    c_string_elem_append (converted, "", 1);
  }
  else
    c_string_elem_append (converted, "", 1);

  row_data.user_data.row_elem.elem = converted;
  row_data.user_data.row_elem.position = i;

  sqlsync_send_user_notification (*(sqlsync_user_message *) user, &row_data);

  c_string_elem_free (converted);
  //sqlsync_sqlserver_destroy(server);
}


void
sqlsync_send_user_grid_elem_print (c_vector_data_ptr container, uint32 pos, void *user)
{
  pos = 0;                      /* Dummy code - compiler warning */
  if (container)
  {
    c_vector_data_all_elems (container, sqlsync_user_send_elem_in_row, user);
  }
}


void
sqlsync_send_user_all_different_elems (c_grid_data_ptr grid_src, c_grid_data_ptr grid_dst)
{
  uint32 i = 0;
  c_vector_data_ptr elem_src;
  c_vector_data_ptr elem_dst;

  if (grid_src == NULL || grid_dst == NULL)
    return;

  for (i = 0; i < c_grid_data_size (grid_src); i++)
  {
    sqlsync_user_message msg;

    elem_src = c_grid_data_pos (grid_src, i);
    elem_dst = c_grid_data_pos (grid_dst, i);

    msg = SQLSYNC_USER_DIFF_SRC_ROW_ELEM;
    sqlsync_send_user_grid_elem_print (elem_src, i, &msg);

    msg = SQLSYNC_USER_DIFF_DST_ROW_ELEM;
    sqlsync_send_user_grid_elem_print (elem_dst, i, &msg);
  }
}



void
c_cmp_print_all_elems (c_cmp_ptr container)
{
  if (container == NULL)
    return;

  sqlsync_user_message msg;
  sqlsync_user_data summary_data;

  summary_data.user_data.summary.table = container->table_name;
  summary_data.user_data.summary.nr_columns = c_vector_columns_size (container->columns);
  summary_data.user_data.summary.columns = container->columns;
  summary_data.user_data.summary.nr_equal_rows = container->equals;
  summary_data.user_data.summary.nr_diff_rows = c_grid_data_size (container->row_diff_src);
  summary_data.user_data.summary.nr_miss_rows = c_grid_data_size (container->row_miss);
  summary_data.user_data.summary.nr_add_rows = c_grid_data_size (container->row_add);

  sqlsync_send_user_notification (SQLSYNC_USER_SUMMARY, &summary_data);

  sqlsync_send_user_notification (SQLSYNC_USER_SUMMARY_DIFF, NULL);

  sqlsync_send_user_all_different_elems (container->row_diff_src, container->row_diff_dst);

  sqlsync_send_user_notification (SQLSYNC_USER_SUMMARY_MISS, NULL);
  msg = SQLSYNC_USER_MISS_ROW_ELEM;
  c_grid_data_all_elems (container->row_miss, sqlsync_send_user_grid_elem_print, &msg);

  sqlsync_send_user_notification (SQLSYNC_USER_SUMMARY_ADD, NULL);
  msg = SQLSYNC_USER_ADD_ROW_ELEM;
  c_grid_data_all_elems (container->row_add, sqlsync_send_user_grid_elem_print, &msg);

  sqlsync_send_user_notification (SQLSYNC_USER_SUMMARY_END, NULL);
}
