/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#include "data_types/sqlsync_data_binary.h"
#include "data_types/sqlsync_data_string.h"

c_data_type c_string_elem_get_type()
{
  return c_string_elem_type;
}


c_string_elem_ptr c_string_elem_empty ()
{
  c_string_elem_ptr tmp;

  tmp = (c_string_elem_ptr) std_malloc (sizeof (c_string_elem));
  if (tmp == NULL)
    return NULL;

  tmp->_type = c_string_elem_type;
  tmp->string = (char *) std_malloc (1);
  if (tmp->string == NULL)
  {
    std_free (tmp);
    return NULL;
  }

  tmp->string[0] = '\0';
  tmp->length = 0;
  tmp->reserved_length = 0;
  return tmp;
}

void c_string_elem_free (c_string_elem_ptr data)
{
  if (data != NULL)
  {
    std_free (data->string);
    std_free (data);
    data = NULL;
  }
}


SQLSYNC_RETURN
c_string_elem_append_cstring (c_string_elem_ptr elem, char *data)
{
  if (elem == NULL)
    return SQLSYNC_FAILED_NULL;
  c_string_elem_append (elem, data, std_strlen (data));
  return SQLSYNC_SUCCESS;
}



SQLSYNC_RETURN
c_string_elem_append_string (c_string_elem_ptr elem, c_string_elem_ptr append)
{
  if (append == NULL)
    return SQLSYNC_FAILED_NULL;
  c_string_elem_append (elem, append->string, append->length);
  return SQLSYNC_SUCCESS;
}


c_string_elem_ptr c_string_elem_copy (c_string_elem_ptr data)
{
  c_string_elem_ptr elem = NULL;

  if (data == NULL)
    return NULL;

  elem = c_string_elem_empty ();
  if (elem == NULL)
    return NULL;

  elem->_type = c_string_elem_type;
  if (c_string_elem_append_string (elem, data) != SQLSYNC_SUCCESS)
  {
    c_string_elem_free (elem);
    elem = NULL;
  }
  return elem;
}

void
c_string_elem_make_upper (c_string_elem_ptr elem)
{
  uint32 i = 0;

  if (elem == NULL || elem->string == NULL)
    return;

  for (i = 0; i < elem->length; i++)
    elem->string[i] = std_toupper (elem->string[i]);
}

int8
c_string_elem_icmp (c_string_elem_ptr src, c_string_elem_ptr dst)
{

  if (src == NULL)
  {
    if (dst == NULL)
      return 0;
    else
      return 1;
  }
  else if (dst == NULL)
    return -1;

  uint32 min = MIN (src->length, dst->length);
  uint32 max = MAX (src->length, dst->length);

  uint32 i = 0;

  for (i = 0; i < min; i++)
  {
    int s = std_toupper (src->string[i]);
    int d = std_toupper (dst->string[i]);

    if (s != d)
    {
      if (s < d)
        return -1;
      else
        return 1;
    }
  }
  if (max != min)
  {
    if (max == src->length)
      return -1;
    else
      return 1;
  }
  return 0;

}

int8
c_string_elem_ncmp (c_string_elem_ptr src, c_string_elem_ptr dst, uint32 max_pos)
{

  if (src == NULL)
  {
    if (dst == NULL)
      return 0;
    else
      return 1;
  }
  else if (dst == NULL)
    return -1;

  uint32 min = MIN (src->length, dst->length);
  min = MIN(min, max_pos);
  uint32 max = max_pos;

  uint32 i = 0;

  for (i = 0; i < min; i++)
  {
    int s = src->string[i];
    int d = dst->string[i];

    if (s != d)
    {
      if (s < d)
        return -1;
      else
        return 1;
    }
  }
  if (max != min)
  {
    if (max == src->length)
      return -1;
    else
      return 1;
  }
  return 0;

}

int32 c_string_elem_strip_lspaces(c_string_elem_ptr src)
{
  if (src == NULL)
    return 0;
  int32 l = src->length;
  int32 i = l;

  while(i)
  {

    if (src->string[i-1] != ' ')
      break;

    src->length--;
    i--;
  }

  return l - i;
}

int32 c_string_elem_cmp (c_string_elem_ptr src, c_string_elem_ptr dst)
{
  c_binary_elem bin_src;
  c_binary_elem bin_dst;

  if (src == NULL)
  {
    if (dst == NULL)
      return 0;
    else
      return -1;
  }
  else if (dst == NULL)
    return 1;

  bin_src._type = c_binary_elem_type;
  bin_src.data = (uint8 *) src->string;
  bin_src.length = src->length;

  bin_dst._type = c_binary_elem_type;
  bin_dst.data = (uint8 *) dst->string;
  bin_dst.length = dst->length;

  return c_binary_elem_cmp ((c_binary_elem_ptr) & bin_src, (c_binary_elem_ptr) & bin_dst);
}



SQLSYNC_RETURN
c_string_elem_append (c_string_elem_ptr elem, char *data, uint32 length)
{
  char *tmp;
  uint32 new_size = 0;

  if (elem == NULL || length == 0)
    return SQLSYNC_FAILED;

  if (elem->string == NULL)
    return SQLSYNC_FAILED_NULL;


  new_size = MAX (elem->length + length, elem->reserved_length + length);

  if (new_size != elem->reserved_length)
  {
    tmp = (char *) std_realloc (elem->string, new_size);

    if (tmp != NULL)
      elem->string = tmp;
    else
      return SQLSYNC_FAILED;
  }

  if (data != NULL)
  {
    std_memcpy (elem->string + elem->length, data, length);
    elem->length += length;
  }

  if (new_size != elem->reserved_length)
    elem->reserved_length = new_size;

  return SQLSYNC_SUCCESS;
}


c_string_elem_ptr
c_string_elem_reserve (uint32 length)
{
  c_string_elem_ptr elem = NULL;

  elem = c_string_elem_empty ();
  if (elem == NULL)
    return NULL;

  if (c_string_elem_append (elem, NULL, length) != SQLSYNC_SUCCESS)
    return NULL;
  else
    return elem;
}

SQLSYNC_RETURN
c_string_elem_append_reserve (c_string_elem_ptr elem, uint32 length)
{
  return c_string_elem_append (elem, NULL, length);
}

SQLSYNC_RETURN
c_vector_strings_append_data (c_vector_strings_ptr container, uint32 pos, char *data, uint32 length)
{
  c_string_elem_ptr elem = c_vector_strings_pos (container, pos);

  if (elem == NULL)
  {
    c_string_elem_ptr tmp = c_string_elem_empty ();

    elem = c_vector_strings_add (container, tmp, pos);
  }

  return c_string_elem_append (elem, data, length);
}


uint32 c_string_elem_size(c_string_elem_ptr elem)
{
  return elem->reserved_length;
}

void * c_string_elem_data(c_string_elem_ptr elem)
{
  return elem->string;
}

c_string_elem_Vtbl * c_string_elem_create_Vtbl(uint32 vtbl_size)
{
 c_string_elem_Vtbl * vtbl = (c_string_elem_Vtbl *) c_data_elem_create_Vtbl(vtbl_size);

  if (vtbl == NULL)
    return NULL;
  
  vtbl->elem_empty = c_string_elem_empty;
  vtbl->elem_free = c_string_elem_free;
  vtbl->elem_cmp = c_string_elem_cmp;
  vtbl->elem_copy = c_string_elem_copy;
  vtbl->get_type = c_string_elem_get_type;
  vtbl->elem_size = c_string_elem_size;
  vtbl->elem_data = c_string_elem_data;

  return vtbl;
}


