/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#include "data_types/sqlsync_data_binary.h"
#include "data_types/sqlsync_data_datetime.h"
#include "data_types/sqlsync_data_grid.h"
#include "data_types/sqlsync_data_misc.h"
#include "data_types/sqlsync_data_numeric.h"
#include "data_types/sqlsync_data_string.h"
#include "data_types/sqlsync_data_vector.h"

#include "data_types/sqlsync_data_type.h"

c_data_elem_Vtbl * c_data_elem_create_Vtbl(uint32 vtbl_size)
{
  c_data_elem_Vtbl * vtbl = std_malloc(vtbl_size);
  
  if (vtbl == NULL)
    return NULL;

  vtbl->elem_free = NULL;
  vtbl->elem_copy = NULL;
  vtbl->get_type = NULL;

  return vtbl;
}

void c_data_elem_free_Vtbl(c_data_elem_Vtbl * vtbl)
{
  std_free(vtbl);
}


int32 c_data_elem_cmp(c_data_elem_ptr elem, c_data_elem_ptr dst)
{
  if (elem == NULL)
  {
    if (dst == NULL)
      return 0;
    else
      return -1;
  }
  else if (dst == NULL)
    return 1;


  if (elem->_type == dst->_type)
    return v_tbl(elem)->elem_cmp(elem, dst);
  else
  {
    int8 ret = 0;
    c_string_elem_ptr e_src = NULL;
    c_string_elem_ptr e_dst = NULL;
    if (elem->_type != c_string_elem_type)
      e_src = c_data_convert(NULL, elem, SQLSYNC_FALSE);
    else
      e_src = (c_string_elem_ptr) elem;

    if (dst->_type != c_string_elem_type)
      e_dst = c_data_convert(NULL, dst, SQLSYNC_FALSE);
    else
      e_dst = (c_string_elem_ptr) dst;

    ret = c_string_elem_cmp(e_src, e_dst);

    if (e_src != (c_string_elem_ptr) elem)
      c_string_elem_free(e_src);
    if (e_dst != (c_string_elem_ptr) dst)
      c_string_elem_free(e_dst);

    return ret;
  }
}


typedef struct _c_vtbls
{
  void * c_data_elem_vtbl;
  void * c_vector_data_vtbl;
  
  void * c_date_elem_vtbl;
  void * c_time_elem_vtbl;
  void * c_timestamp_elem_vtbl;
  
  void * c_uint8_elem_vtbl;
  void * c_int8_elem_vtbl;
  void * c_uint16_elem_vtbl;
  void * c_int16_elem_vtbl;
  void * c_uint32_elem_vtbl;

  void * c_int32_elem_vtbl;

  void * c_ubigint_elem_vtbl;
  void * c_bigint_elem_vtbl;
  void * c_numeric_elem_vtbl;
  void * c_float_elem_vtbl;
  void * c_double_elem_vtbl;
  
  void * c_binary_elem_vtbl;
  void * c_bit_elem_vtbl;
  
  void * c_column_elem_vtbl;
  
  void * c_string_elem_vtbl;

  void * c_bool_elem_vtbl;
  void * c_guid_elem_vtbl;
  
  void * c_notification_elem_vtbl;
} c_vtbls;

c_vtbls * c_vtbls_ptr = NULL;

void * c_create_vtbls()
{
  c_vtbls_ptr =  std_malloc(sizeof(c_vtbls));

  c_vtbls_ptr->c_data_elem_vtbl = NULL;
  c_vtbls_ptr->c_bigint_elem_vtbl = c_bigint_elem_create_Vtbl(sizeof(c_bigint_elem_Vtbl));
  c_vtbls_ptr->c_ubigint_elem_vtbl = c_ubigint_elem_create_Vtbl(sizeof(c_ubigint_elem_Vtbl));
  c_vtbls_ptr->c_binary_elem_vtbl = c_binary_elem_create_Vtbl(sizeof(c_binary_elem_Vtbl));
  c_vtbls_ptr->c_bit_elem_vtbl = c_bit_elem_create_Vtbl(sizeof(c_bit_elem_Vtbl));
  c_vtbls_ptr->c_column_elem_vtbl = c_column_elem_create_Vtbl(sizeof(c_column_elem_Vtbl));
  c_vtbls_ptr->c_date_elem_vtbl = c_date_elem_create_Vtbl(sizeof(c_date_elem_Vtbl));
  c_vtbls_ptr->c_double_elem_vtbl = c_double_elem_create_Vtbl(sizeof(c_double_elem_Vtbl));
  c_vtbls_ptr->c_float_elem_vtbl = c_float_elem_create_Vtbl(sizeof(c_float_elem_Vtbl));
  c_vtbls_ptr->c_int16_elem_vtbl = c_int16_elem_create_Vtbl(sizeof(c_int16_elem_Vtbl));
  c_vtbls_ptr->c_int32_elem_vtbl = c_int32_elem_create_Vtbl(sizeof(c_int32_elem_Vtbl));
  c_vtbls_ptr->c_int8_elem_vtbl = c_int8_elem_create_Vtbl(sizeof(c_int8_elem_Vtbl));
  c_vtbls_ptr->c_numeric_elem_vtbl = c_numeric_elem_create_Vtbl(sizeof(c_numeric_elem_Vtbl));
  c_vtbls_ptr->c_string_elem_vtbl = c_string_elem_create_Vtbl(sizeof(c_string_elem_Vtbl));
  c_vtbls_ptr->c_time_elem_vtbl = c_time_elem_create_Vtbl(sizeof(c_time_elem_Vtbl));
  c_vtbls_ptr->c_timestamp_elem_vtbl = c_timestamp_elem_create_Vtbl(sizeof(c_timestamp_elem_Vtbl));

  c_vtbls_ptr->c_uint32_elem_vtbl = c_uint32_elem_create_Vtbl(sizeof(c_uint32_elem_Vtbl));
  c_vtbls_ptr->c_uint16_elem_vtbl = c_uint16_elem_create_Vtbl(sizeof(c_uint16_elem_Vtbl));
  c_vtbls_ptr->c_uint8_elem_vtbl = c_uint8_elem_create_Vtbl(sizeof(c_uint8_elem_Vtbl));
  c_vtbls_ptr->c_vector_data_vtbl = c_vector_data_create_Vtbl(sizeof(c_vector_data_Vtbl));

  c_vtbls_ptr->c_bool_elem_vtbl = c_bool_elem_create_Vtbl(sizeof(c_bool_elem_Vtbl));
  c_vtbls_ptr->c_guid_elem_vtbl = c_guid_elem_create_Vtbl(sizeof(c_guid_elem_Vtbl));

  c_vtbls_ptr->c_notification_elem_vtbl = c_notification_elem_create_Vtbl(sizeof(c_notification_elem_Vtbl));

  return c_vtbls_ptr;
}

void c_destroy_vtbls()
{
  if (c_vtbls_ptr == NULL)
     return;

  std_free(c_vtbls_ptr->c_bool_elem_vtbl);

  std_free(c_vtbls_ptr->c_bigint_elem_vtbl);
  std_free(c_vtbls_ptr->c_ubigint_elem_vtbl);
  std_free(c_vtbls_ptr->c_binary_elem_vtbl);
  std_free(c_vtbls_ptr->c_bit_elem_vtbl);
  std_free(c_vtbls_ptr->c_column_elem_vtbl);
  /*std_free(c_vtbls_ptr->c_data_elem_vtbl);*/
  std_free(c_vtbls_ptr->c_date_elem_vtbl);
  std_free(c_vtbls_ptr->c_double_elem_vtbl);
  std_free(c_vtbls_ptr->c_float_elem_vtbl);
  std_free(c_vtbls_ptr->c_int16_elem_vtbl);
  std_free(c_vtbls_ptr->c_int32_elem_vtbl);
  std_free(c_vtbls_ptr->c_int8_elem_vtbl);
  std_free(c_vtbls_ptr->c_numeric_elem_vtbl);
  std_free(c_vtbls_ptr->c_string_elem_vtbl);
  std_free(c_vtbls_ptr->c_time_elem_vtbl);
  std_free(c_vtbls_ptr->c_timestamp_elem_vtbl);
  std_free(c_vtbls_ptr->c_uint32_elem_vtbl);
  std_free(c_vtbls_ptr->c_uint16_elem_vtbl);
  std_free(c_vtbls_ptr->c_uint8_elem_vtbl);
  std_free(c_vtbls_ptr->c_vector_data_vtbl); 
  std_free(c_vtbls_ptr->c_guid_elem_vtbl);
  std_free(c_vtbls_ptr->c_notification_elem_vtbl);

  std_free(c_vtbls_ptr);
  c_vtbls_ptr = NULL;
}


c_data_elem_Vtbl * c_data_get_vtbl(c_data_type type)
{
  switch(type) 
  {
    case c_bool_elem_type:
      return c_vtbls_ptr->c_bool_elem_vtbl;

    case c_guid_elem_type:
      return c_vtbls_ptr->c_guid_elem_vtbl;

    case c_bigint_elem_type:
      return c_vtbls_ptr->c_bigint_elem_vtbl;
    
    case c_ubigint_elem_type:
      return c_vtbls_ptr->c_ubigint_elem_vtbl;
    
    case c_binary_elem_type:
      return c_vtbls_ptr->c_binary_elem_vtbl;
    
    case c_bit_elem_type:
      return c_vtbls_ptr->c_bit_elem_vtbl;
    
    case c_column_elem_type:
      return c_vtbls_ptr->c_column_elem_vtbl;
    
    case c_data_elem_type:
      return c_vtbls_ptr->c_data_elem_vtbl;
    
    case c_date_elem_type:
      return c_vtbls_ptr->c_date_elem_vtbl;
    
    case c_double_elem_type:
      return c_vtbls_ptr->c_double_elem_vtbl;
    
    case c_float_elem_type:
      return c_vtbls_ptr->c_float_elem_vtbl;
    
    case c_int16_elem_type:
      return c_vtbls_ptr->c_int16_elem_vtbl;
    
    case c_int32_elem_type:
      return c_vtbls_ptr->c_int32_elem_vtbl;
    
    case c_int8_elem_type:
      return c_vtbls_ptr->c_int8_elem_vtbl;
    
    case c_numeric_elem_type:
      return c_vtbls_ptr->c_numeric_elem_vtbl;
    
    case c_string_elem_type:
      return c_vtbls_ptr->c_string_elem_vtbl;
    
    case c_time_elem_type:
      return c_vtbls_ptr->c_time_elem_vtbl;
    
    case c_timestamp_elem_type:
      return c_vtbls_ptr->c_timestamp_elem_vtbl;
    
    case c_uint16_elem_type:
      return c_vtbls_ptr->c_uint16_elem_vtbl;
    
    case c_uint8_elem_type:
      return c_vtbls_ptr->c_uint8_elem_vtbl;

    case c_uint32_elem_type:
      return c_vtbls_ptr->c_uint32_elem_vtbl;


    case c_grid_data_type:
    case c_grid_strings_type:
    case c_grid_columns_type:
    case c_vector_strings_type:
    case c_vector_data_type:
    case c_vector_columns_type:
    case c_vector_int32_type:
      return c_vtbls_ptr->c_vector_data_vtbl; 
    
    case c_notification_elem_type:
      return c_vtbls_ptr->c_notification_elem_vtbl;

    default:
        return NULL;
  }
}
