/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#ifndef SQLSYNC_DATA_TYPE_H
#define SQLSYNC_DATA_TYPE_H

#include "sqlsync_def.h"

#define packed_attr __attribute__((__packed__))

#define INHERIT_BASE(type) \
          c_data_type (*get_type)(); \
          void (*elem_free)(c_##type##_ptr elem); \
          c_##type##_ptr (*elem_copy)(c_##type##_ptr elem); 

#define INHERIT_OBJECT_BASE(type) \
          uint8 _type;

#define DEFINE_INTERFACE(type) \
          typedef struct _c##type##_Vtbl \
          {\
            INHERIT_##type(type)\
          } c_##type##_Vtbl; \
          INHERIT_FUNCTIONS_##type(type) \
          c_##type##_Vtbl * c_##type##_create_Vtbl(uint32 vtbl_size)

#define DEFINE_INTERFACE_STRUCT(type) \
          typedef struct _c_##type \
          {\
            INHERIT_OBJECT_##type(type)\
          } packed_attr c_##type; \
          typedef c_##type * c_##type##_ptr


typedef enum _c_data_type
{
  c_data_unknown_type = 0,
  c_data_elem_type,
  c_vector_data_type,

  c_grid_data_type,

  c_bool_elem_type,
  c_guid_elem_type,

  c_date_elem_type,
  c_time_elem_type,
  c_timestamp_elem_type,
  
  c_uint8_elem_type,
  c_int8_elem_type,
  c_uint16_elem_type,
  c_int16_elem_type,
  c_uint32_elem_type,

  c_int32_elem_type,
  c_vector_int32_type,

  c_ubigint_elem_type,
  c_bigint_elem_type,
  c_numeric_elem_type,
  c_float_elem_type,
  c_double_elem_type,
  
  c_binary_elem_type,
  c_bit_elem_type,
  
  c_column_elem_type,
  c_vector_columns_type,
  c_grid_columns_type,
  
  c_string_elem_type,
  c_vector_strings_type,
  c_grid_strings_type,
  
  c_notification_elem_type
} c_data_type;



#define INHERIT_data_elem(type) \
          INHERIT_BASE(type) \
          c_##type##_ptr (*elem_empty)(); \
          uint32 (*elem_size)(c_##type##_ptr elem); \
          int32 (*elem_cmp)(c_##type##_ptr src, c_##type##_ptr dst); \
          void * (*elem_data)(c_##type##_ptr elem); 


#define INHERIT_OBJECT_data_elem(type) \
          INHERIT_OBJECT_BASE(type)

#define INHERIT_FUNCTIONS_data_elem(type)

DEFINE_INTERFACE_STRUCT(data_elem);
DEFINE_INTERFACE(data_elem);


void c_data_elem_free_Vtbl(c_data_elem_Vtbl * vtbl);
c_data_elem_Vtbl * c_data_get_vtbl(c_data_type type);

SQLSYNC_RETURN c_data_is_vector(c_data_elem_ptr elem);

int32 c_data_elem_cmp(c_data_elem_ptr elem, c_data_elem_ptr dst);

#define v_tbl(elem) c_data_get_vtbl((elem)->_type)

#define c_data_get_type(elem) v_tbl(elem)->get_type()
#define c_data_copy(elem) (elem != NULL ? v_tbl(elem)->elem_copy(elem) : NULL)
#define c_data_free(elem) v_tbl(elem)->elem_free(elem)

#define c_data_elem_empty(type) c_data_get_vtbl(type)->elem_empty()
#define c_data_elem_size(elem) (elem != NULL ? v_tbl(elem)->elem_size(elem) : 0)
#define c_data_elem_data(elem) (elem != NULL ? v_tbl(elem)->elem_data(elem) : NULL)

void * c_create_vtbls();
void c_destroy_vtbls();

#endif /* SQLSYNC_DATA_TYPE_H */
