/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#ifdef DEBUG
#include "sqlsync_debug.h"
#endif

#include "data_types/sqlsync_data_type.h"
#include "data_types/sqlsync_data_vector.h"

c_vector_data_ptr
c_vector_data_init (uint32 size)
{
  c_vector_data_ptr q = NULL;

  if (size == 0)
    return NULL;

  q = (c_vector_data_ptr) std_malloc (sizeof (c_vector_data));
  q->array = (c_data_elem_ptr *) std_malloc (sizeof (c_data_elem_ptr) * size);
  q->_type = c_vector_data_type;
  std_memset (q->array, 0, sizeof (c_data_elem_ptr) * size);
  q->size = size;
  q->max_add_pos = 0;
  q->is_empty = 1;
  q->refCount = 0;
  return q;
}

void
c_vector_data_inc_reference (c_vector_data_ptr container)
{
  if (container != NULL)
  {
#ifdef DEBUG
    sqlsync_debug_details ("Increment refCount %d\n", container->refCount);
#endif
    container->refCount++;
  }
}

void
c_vector_data_dec_reference (c_vector_data_ptr container)
{
  if (container != NULL && container->refCount > 0)
  {
#ifdef DEBUG
    sqlsync_debug_details ("Decrement refCount %d\n", container->refCount);
#endif
    container->refCount--;
  }
}


c_data_elem_ptr
c_vector_data_add_last (c_vector_data_ptr container, c_data_elem_ptr elem)
{
  uint32 pos = 0;

  if (container == 0)
    return NULL;

  if (container->is_empty == 1)
    pos = 0;
  else
    pos = container->max_add_pos + 1;

  return c_vector_data_add (container, elem, pos);
}

c_data_elem_ptr
c_vector_data_add_last_copy (c_vector_data_ptr container, c_data_elem_ptr elem)
{
  uint32 pos = 0;

  if (container == 0)
    return NULL;

  if (container->is_empty == 1)
    pos = 0;
  else
    pos = container->max_add_pos + 1;

  return c_vector_data_add_copy (container, elem, pos);
}

c_data_elem_ptr
c_vector_data_add (c_vector_data_ptr container, c_data_elem_ptr elem, uint32 pos)
{
  uint32 new_size;
  c_data_elem_ptr *tmp;

  if (container == NULL)
    return NULL;

  if (c_data_is_vector (elem) == SQLSYNC_TRUE)
    c_vector_data_inc_reference ((c_vector_data_ptr) elem);


  if (pos >= container->size)   /* Resize container if necessary */
  {
    new_size = MAX (container->size + DEFAULT_DATA_SIZE, pos + 1);
#ifdef DEBUG
    sqlsync_debug_details ("Resizing container" "from %ld to %ld\n", container->size, new_size);
#endif
    tmp = (c_data_elem_ptr *) std_malloc (sizeof (c_data_elem_ptr) * new_size);

    /* copy old data array */
    std_memcpy (tmp, container->array, sizeof (c_data_elem_ptr) * container->size);
    /* and free .. */
    std_free (container->array);

    container->array = tmp;

    /* clear all new entries */
    std_memset (container->array + container->size, 0,
            (new_size - container->size) * sizeof (c_data_elem_ptr));

    container->size = new_size;
  }

  (void) c_vector_data_delete (container, pos);        /* Delete old entry */

  if (pos > container->max_add_pos)
    container->max_add_pos = pos;

  container->array[pos] = elem;
  container->is_empty = 0;

  return container->array[pos];
}


c_data_elem_ptr
c_vector_data_add_copy (c_vector_data_ptr container, c_data_elem_ptr elem, uint32 pos)
{
  uint32 new_size;
  c_data_elem_ptr *tmp;

  if (container == NULL)
    return NULL;

  if (pos >= container->size)   /* Resize container if necessary */
  {
    new_size = MAX (container->size + DEFAULT_DATA_SIZE, pos + 1);
#ifdef DEBUG
    sqlsync_debug_details ("Resizing container" "from %ld to %ld\n", container->size, new_size);
#endif
    tmp = (c_data_elem_ptr *) std_malloc (sizeof (c_data_elem_ptr) * new_size);

    /* copy old data array */
    std_memcpy (tmp, container->array, sizeof (c_data_elem_ptr) * container->size);
    /* and free .. */
    std_free (container->array);

    container->array = tmp;

    /* clear all new entries */
    std_memset (container->array + container->size, 0,
            (new_size - container->size) * sizeof (c_data_elem_ptr));

    container->size = new_size;
  }

  (void) c_vector_data_delete (container, pos);        /* Delete old entry */

  container->is_empty = 0;

  if (pos > container->max_add_pos)
    container->max_add_pos = pos;

  container->array[pos] = c_data_copy(elem);

  return container->array[pos];
}


c_data_elem_ptr
c_vector_data_add_reserve (c_vector_data_ptr container, uint8 data_type, uint32 pos)
{
  return c_vector_data_add (container, NULL, pos);
}


c_data_elem_ptr
c_vector_data_pos (c_vector_data_ptr container, uint32 pos)
{
  if (container == NULL || container->array == NULL)
    return NULL;

  if (container->size > pos)
    return container->array[pos];
  else
    return NULL;
}


uint8
c_vector_data_pos_type (c_vector_data_ptr container, uint32 pos)
{
  c_data_elem_ptr data = c_vector_data_pos (container, pos);

  if (data != NULL)
    return data->_type;
  else
    return c_data_unknown_type;

}

uint32
c_vector_data_find (c_vector_data_ptr container, c_data_elem_ptr data, SQLSYNC_RETURN * error)
{
  uint32 i = 0;
  int8 ret;
  c_data_elem_ptr elem;

  uint32 size = c_vector_data_size (container);

  if (data == NULL)
  {
    if (error)
      *error = SQLSYNC_FAILED;
    return 0;
  }

  for (i = 0; i < size; i++)
  {
    elem = c_vector_data_pos (container, i);
    if (elem == NULL)
      continue;

      ret = c_data_elem_cmp (elem, data);

    if (ret == 0)
    {
      if (error)
        *error = SQLSYNC_SUCCESS;

      return i;
    }
  }

  if (error)
    *error = SQLSYNC_FAILED;

  return 0;
}


uint32
c_vector_data_find_wf (c_vector_data_ptr container, c_data_elem_ptr data, c_data_elem_cmp_t f, SQLSYNC_RETURN * error)
{
  uint32 i = 0;
  int8 ret;
  c_data_elem_ptr elem;

  uint32 size = c_vector_data_size (container);

  if (data == NULL)
  {
    if (error)
      *error = SQLSYNC_FAILED;
    return 0;
  }

  for (i = 0; i < size; i++)
  {
    elem = c_vector_data_pos (container, i);
    if (elem == NULL)
      continue;

      ret = f (elem, data);

    if (ret == 0)
    {
      if (error)
        *error = SQLSYNC_SUCCESS;

      return i;
    }
  }

  if (error)
    *error = SQLSYNC_FAILED;

  return 0;
}


uint32
c_vector_data_real_size (c_vector_data_ptr container)
{
  if (container == NULL)
    return 0;
  else
    return container->size;
}

uint32
c_vector_data_size (c_vector_data_ptr container)
{
  if (container == NULL)
    return 0;
  else if (container->max_add_pos != 0 || container->is_empty == 0)
    return container->max_add_pos + 1;
  else
    return 0;
}

uint32
c_vector_data_find_free (c_vector_data_ptr container, SQLSYNC_RETURN * error)
{

  if (container != NULL)
  {
    uint32 i = 0;
    uint32 size = c_vector_data_size (container);

    for (i = 0; i < size; i++)
    {
      if (c_vector_data_pos (container, i) == NULL)
      {
        if (error)
          *error = SQLSYNC_SUCCESS;

        return i;
      }
    }
  }

  if (error)
    *error = SQLSYNC_FAILED;

  return 0;
}

SQLSYNC_RETURN
c_vector_data_delete (c_vector_data_ptr container, uint32 pos)
{
  c_data_elem_ptr elem = c_vector_data_pos (container, pos);

  if (elem != NULL)
  {
    if (c_data_is_vector (elem) == SQLSYNC_TRUE)
    {
      c_vector_data_ptr tmp = (c_vector_data_ptr) elem;

      c_vector_data_dec_reference (tmp);
      if (tmp->refCount == 0 && elem != NULL)
        c_data_free(elem);
    }
    else
    {
        if (elem != NULL)
        c_data_free(elem);
    }

    container->array[pos] = NULL;
  }
  else
    return SQLSYNC_FAILED_NULL;

  return SQLSYNC_SUCCESS;
}

SQLSYNC_RETURN
c_vector_data_clear (c_vector_data_ptr container)
{
  uint32 i = 0;
  uint32 size = 0;

  if (container == NULL)
    return SQLSYNC_FAILED_NULL;

  size = c_vector_data_size (container);

  for (i = 0; i < size; i++)
    (void) c_vector_data_delete (container, i);

  container->max_add_pos = 0;
  container->is_empty = 1;
  return SQLSYNC_SUCCESS;
}

void
c_vector_data_all_elems (c_vector_data_ptr container, c_vector_data_one_elem f, void *user)
{
  c_data_elem_ptr elem = NULL;
  uint32 i = 0;

  if (container == NULL || f == NULL)
    return;

  for (i = 0; i < c_vector_data_size (container); i++)
  {
    elem = c_vector_data_pos (container, i);
/*    if (elem != NULL) */
    f (elem, i, user);
  }
}

c_vector_data_ptr
c_vector_data_copy (c_vector_data_ptr container)
{
  uint32 i = 0;
  uint32 size = 0;
  c_vector_data_ptr tmp = 0;

  if (container == NULL)
    return NULL;

  size = c_vector_data_real_size (container);
#ifdef DEBUG
  sqlsync_debug_details ("Number of elements to copy %ld\n", size);
#endif

  tmp = c_vector_data_init (size);


  for (i = 0; i < size; i++)
  {
    (void) c_vector_data_add_copy (tmp, c_vector_data_pos (container, i), i);
  }

  tmp->_type = container->_type;
  tmp->is_empty = container->is_empty;
  tmp->max_add_pos = container->max_add_pos;
  return tmp;
}


void c_vector_data_destroy (c_vector_data_ptr container)
{
#ifdef DEBUG
  sqlsync_debug("c_vector_data_destroy\n");
#endif
  if (container != NULL)
  {
    (void) c_vector_data_clear (container);
    if (container->array)
      std_free (container->array);

    std_free (container);
  }
}

c_vector_data_ptr c_vector_data_empty()
{
  return NULL;
}

c_data_type c_vector_data_get_type()
{
  return c_vector_data_type;
}

c_vector_data_Vtbl * c_vector_data_create_Vtbl(uint32 vtbl_size)
{
  c_vector_data_Vtbl * vtbl = (c_vector_data_Vtbl *) c_data_elem_create_Vtbl(vtbl_size);

  if (vtbl == NULL)
    return NULL;
  
  vtbl->elem_free = c_vector_data_destroy;
  vtbl->elem_copy = c_vector_data_copy;
  vtbl->get_type = c_vector_data_get_type;

  return vtbl;
}



SQLSYNC_RETURN
c_data_is_vector (c_data_elem_ptr elem)
{
  if (elem == NULL)
    return SQLSYNC_FAILED_NULL;

  switch (elem->_type)
  {
    case c_vector_data_type:
    case c_vector_strings_type:
    case c_vector_columns_type:
    case c_grid_columns_type:
    case c_grid_strings_type:
    case c_grid_data_type:
    case c_vector_int32_type:
      return SQLSYNC_TRUE;

    default:
      return SQLSYNC_FALSE;
  }
  return SQLSYNC_TRUE;
}
