/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#ifndef SQLSYNC_VECTOR_DATA_H
#define SQLSYNC_VECTOR_DATA_H

#include "data_types/sqlsync_data_type.h"

#define INHERIT_vector_data(type) \
          INHERIT_BASE(type)


#define INHERIT_OBJECT_vector_data(type) \
          INHERIT_OBJECT_BASE(type) \
          uint16 refCount; \
          c_data_elem_ptr *array;       /*!<Array */ \
          uint8 is_empty; \
          uint32 max_add_pos;           /*!<Maximum from all added posistions */ \
          uint32 size;                  /*!<Size of array */ \

#define INHERIT_FUNCTIONS_vector_data(type) \
          c_##type##_ptr c_##type##_init(uint32 size);

DEFINE_INTERFACE_STRUCT(vector_data);
DEFINE_INTERFACE(vector_data);

c_vector_data_Vtbl * c_vector_data_get_vtbl(c_data_type type);

#define vv_tbl c_vector_data_get_vtbl((elem)->_type)

#define c_vector_data_vcopy(elem) vv_tbl->elem_copy(elem)
#define c_vector_data_vfree(elem) vv_tbl->elem_free(elem)


/**
 * Default size of elemens in data array
 */
#ifndef DEFAULT_DATA_SIZE
#define DEFAULT_DATA_SIZE 10
#endif

/**
 * Function initalizes data structure
 * @param size How many elements array should have on the
 *             begining
 * @return c_data Returns pointer to allocated structure
 */

c_vector_data_ptr c_vector_data_init (uint32 size);

/**
 * Function copies data to the internal structure at specified
 * position
 * @param container Pointer to the container
 * @param elem Pointer to data that should be stored
 * @param pos Position in array where data should be stored
 * @return c_data_elem_ptr Returns pointer to element in array
 */
c_data_elem_ptr c_vector_data_add (c_vector_data_ptr container, c_data_elem_ptr elem, uint32 pos);

/**
* Function copies data to the internal structure at the end of
* array
 * @param container Pointer to the container
 * @param elem Pointer to data that should be stored
 * @return c_data_elem_ptr Returns pointer to element in array
 */
c_data_elem_ptr c_vector_data_add_last (c_vector_data_ptr container, c_data_elem_ptr elem);


/**
 * Function copies data to the internal structure at specified
 * position
 * @param container Pointer to the container
 * @param elem Pointer to data that should be stored
 * @param pos Position in array where data should be stored
 * @return c_data_elem_ptr Returns pointer to element in array
 */
c_data_elem_ptr c_vector_data_add_copy (c_vector_data_ptr container,
                                            c_data_elem_ptr elem, uint32 pos);

/**
* Function copies data to the internal structure at the end of
* array
 * @param container Pointer to the container
 * @param elem Pointer to data that should be stored
 * @return c_data_elem_ptr Returns pointer to element in array
 */
c_data_elem_ptr c_vector_data_add_last_copy (c_vector_data_ptr container, c_data_elem_ptr elem);


/**
 * Function reserves memory in array
 * @param container Pointer to the container
 * @param size Size of data that should allocated
 * @param pos Position in array where data should be allocated
 * 
 * @return c_data_elem_ptr Returns pointer to an empty element in
 *         array
 */
c_data_elem_ptr c_vector_data_add_reserve (c_vector_data_ptr container, uint8 data_type, uint32 pos);


/**
 * Function returns c_data_elem_ptr from specified position
 * @param container Pointer to the container
 * @param pos Position in array
 * 
 * @return c_data_elem_ptr Element from array
 */
c_data_elem_ptr c_vector_data_pos (c_vector_data_ptr container, uint32 pos);

/*
 * Function returns data type from specified position
 * @param container Pointer to the container
 * @param pos Position in array
 * 
 * @return uint32 data size 
 */
uint8 c_vector_data_pos_type (c_vector_data_ptr container, uint32 pos);

c_vector_data_ptr c_vector_data_copy (c_vector_data_ptr container);

uint32 c_vector_data_real_size (c_vector_data_ptr container);

/**
 * Function returns size of array
 * @param container Pointer to the container
 * 
 * @return uint32 Returns size of array
 */
uint32 c_vector_data_size (c_vector_data_ptr container);

/**
 * Function returns first free position in array
 * @param container Pointer to the container
 * 
 * @return uint32 Returns position in array, "error" variable contains return info
 */
uint32 c_vector_data_find_free(c_vector_data_ptr container, SQLSYNC_RETURN * error);


typedef int32 c_data_elem_cmp_t(c_data_elem_ptr src, c_data_elem_ptr dst);
/**
 * Function finds position in array where searched element is
 * stored
 * @param container Pointer to the container
 * @param data Data that is searched
 * @param f Pointer to comparizing function
 * 
 * @return uint32 Returns position in array, "error" variable contains return info
 */
uint32 c_vector_data_find_wf (c_vector_data_ptr container,
                            c_data_elem_ptr data, c_data_elem_cmp_t f, SQLSYNC_RETURN * error);


/**
 * Function finds position in array where searched element is
 * stored
 * @param container Pointer to the container
 * @param data Data that is searched 
 * 
 * @return uint32 Returns position in array, "error" variable contains return info
 */
uint32 c_vector_data_find (c_vector_data_ptr container,
                            c_data_elem_ptr data, SQLSYNC_RETURN * error);


/* pos informs about elem number in c_data_ptr*/
typedef void c_vector_data_one_elem (c_data_elem_ptr elem, uint32 pos, void *user);

/**
 * Function goes over all elements
 * @param container Pointer to the container
 * @param f Function that receives column and data from
 *          container
 * @param user User pointer passed to f function
 */
void c_vector_data_all_elems (c_vector_data_ptr container,
                       c_vector_data_one_elem f, void *user);

/**
 * Function deletes data from specified position in array
 * @param container Pointer to the container
 * @param pos Position
 * 
 * @return SQLSYNC_RETURN Returns SQLSYNC_FAILED_NULL if
 *         something went wrong otherwise SQLSYNC_SUCCESS is
 *         returned
 */
SQLSYNC_RETURN c_vector_data_delete (c_vector_data_ptr container, uint32 pos);

/**
 * Function clears whole internal structure
 * @param container Pointer to the container
 * 
 * @return SQLSYNC_RETURN Returns SQLSYNC_FAILED_NULL if
 *         something went wrong otherwise SQLSYNC_SUCCESS is
 *         returned
 */
SQLSYNC_RETURN c_vector_data_clear (c_vector_data_ptr container);

/**
 * Function destroyes container
 * @param container Pointer to the container
 * 
 */
void c_vector_data_destroy (c_vector_data_ptr container);


SQLSYNC_RETURN c_data_is_vector (c_data_elem_ptr elem);

#endif /* SQLSYNC_VECTOR_DATA_H */
