/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#include "server/sqlsync_sqlserver.h"
#include "server/sqlsync_sqlserver_priv.h"

#include "sqlsync_debug.h"

#include "server/sqlsync_default_impl.h"
#include "data_types/sqlsync_data_converter.h"
#include "data_types/sqlsync_data_numeric.h"

#include "server/firebird/sqlsync_sqlserver_firebird.h"

c_string_elem_ptr sqlsync_sqlserver_firebird_data_to_string(c_data_elem_ptr elem, SQLSYNC_RETURN with_escapes)
{
  c_string_elem_ptr out = c_string_elem_reserve(2);
  c_string_elem_ptr tmp = NULL;
  if (elem == NULL)
  {
     c_string_elem_append(out, "\'", 1);
     c_string_elem_append_cstring(out, "NULL");
     c_string_elem_free(tmp);
     c_string_elem_append(out, "\'", 1);
  
  }
  switch (elem->_type)
  {
    case c_string_elem_type:
    case c_binary_elem_type:
    {
      if (with_escapes == SQLSYNC_TRUE)
      {
        tmp =  c_data_convert_escapes((c_string_elem_ptr)elem, '\'', "\'");
        break;
      }
    }
    default:
      tmp = c_data_convert(NULL,elem,with_escapes);
  }
  if (c_data_is_numeric(elem) == SQLSYNC_TRUE)
  {
     c_string_elem_free(out);
     return tmp;
  }
  else
  {
    c_string_elem_append(out, "\'", 1);
    c_string_elem_append_string(out, tmp);
    c_string_elem_free(tmp);
    c_string_elem_append(out, "\'", 1);
  }
  return out;
}

c_string_elem_ptr sqlsync_sqlserver_firebird_column_identifier(c_string_elem_ptr elem)
{
  c_string_elem_ptr out = c_string_elem_reserve(2);
  c_string_elem_append(out, "\"", 1);
  c_string_elem_append_string(out, elem);
  c_string_elem_append(out, "\"", 1);
  return out;
}

SQLSYNC_RETURN sqlsync_sqlserver_firebird_does_table_exist(sqlsync_sqlserver * server, c_string_elem_ptr table)
{
  return sqlsync_sqlserver_default_check_table(server, table);
}

c_vector_columns_ptr
sqlsync_old_get_columns (sqlsync_sqlserver * server, c_string_elem_ptr table)
{
  SQLRETURN ret;
  c_string_elem_ptr name = NULL;
  SQLINTEGER len;
  SQLINTEGER max_size;
  SQLSMALLINT data_type;
  c_vector_columns_ptr columns;
  c_column_elem_ptr column_data = NULL;
  c_string_elem_ptr tmp_table = c_string_elem_copy(table);

  name = c_string_elem_reserve(256);

  sqlsync_create_stmt (server);

  //c_string_elem_make_upper(tmp_table);

  ret = SQLColumns (server->hstmt, NULL, 0, NULL, 0, (SQLCHAR *) tmp_table->string, 
                    (SQLSMALLINT) tmp_table->length, NULL, 0);

  c_string_elem_free(tmp_table);

  ret |= SQLBindCol (server->hstmt, SQLColumns_COLUMN_NAME, SQL_C_CHAR,
                     name->string, (SQLSMALLINT) 256, (SQLLEN*)(&name->length));

  ret |= SQLBindCol (server->hstmt, SQLColumns_COLUMN_SIZE, SQL_C_LONG,
                     &max_size, (SQLINTEGER) sizeof (max_size), &len);
  ret |= SQLBindCol (server->hstmt, SQLColumns_DATA_TYPE, SQL_C_SHORT,
                     &data_type, (SQLINTEGER) sizeof (data_type), &len);


  if (SQLSYNC_IS_ERROR (ret))
  {
#ifdef DEBUG
    sqlsync_debug ("sqlsync_get_columns failed %d\n", ret);
#endif

    server->last_bad_handle = SQL_HANDLE_STMT;
    c_string_elem_free(name);
    return NULL;
  }


  columns = c_vector_columns_init (DEFAULT_DATA_SIZE);

#ifdef DEBUG
  sqlsync_debug ("Retrieving columns %s - %s\n", table->string, name->string);
#endif

  while (SQL_SUCCEEDED (ret = SQLFetch (server->hstmt)))
  {
    int8 use_operator = 0;
    column_data = c_column_elem_create (name, max_size, sqlsync_sqlserver_recognize_data_type_odbc(server,data_type, &use_operator), use_operator);

    if (column_data == NULL)
      continue;

    /*    sqlsync_printf(SQLSYNC_PRINT, "data type %ld\n", data_type); */
#ifdef DEBUG
    sqlsync_debug ("Adding column %s\n", name->string);
#endif

    (void) c_vector_columns_add_last (columns, column_data);
  }

  c_string_elem_free(name);
  return columns;
}



c_vector_columns_ptr sqlsync_sqlserver_firebird_get_columns(sqlsync_sqlserver * server, c_string_elem_ptr table)
{
  return sqlsync_old_get_columns(server, table);
}

SQLSYNC_RETURN sqlsync_sqlserver_firebird_is_system_table(c_string_elem_ptr table)
{
  return SQLSYNC_FALSE;
}

sqlsync_sqlserver_data_type sqlsync_sqlserver_firebird_recognize_data_type_odbc(int16 data_type, int8 * use_operator)
{

    switch(data_type) 
    {
      case SQL_DOUBLE:
      case SQL_REAL:
        return SQLSYNC_SQLSERVER_DATA_CHAR;

      case SQL_DECIMAL:
      case SQL_NUMERIC:
        return SQLSYNC_SQLSERVER_DATA_NUMERIC;

      case SQL_INTEGER:
         return SQLSYNC_SQLSERVER_DATA_INTEGER;

      case SQL_SMALLINT:
        return SQLSYNC_SQLSERVER_DATA_SMALLINT;

      case SQL_DATE:
        return SQLSYNC_SQLSERVER_DATA_TIMESTAMP;

      case SQL_TINYINT:
        return SQLSYNC_SQLSERVER_DATA_UTINYINT;

      case SQL_BIT:
        return SQLSYNC_SQLSERVER_DATA_UTINYINT;

      case SQL_BIGINT: 
        return SQLSYNC_SQLSERVER_DATA_BIGINT;

      case SQL_BINARY:
      case SQL_VARBINARY:
      case SQL_LONGVARBINARY:
        return SQLSYNC_SQLSERVER_DATA_BINARY;

      case SQL_WVARCHAR:
      case SQL_WCHAR :

      case SQL_VARCHAR:
      case SQL_CHAR:
        return SQLSYNC_SQLSERVER_DATA_CHAR;

      case SQL_TYPE_DATE:
        return SQLSYNC_SQLSERVER_DATA_DATE;

      case SQL_TYPE_TIME:
        return SQLSYNC_SQLSERVER_DATA_TIME;

      case SQL_TYPE_TIMESTAMP:
        return SQLSYNC_SQLSERVER_DATA_TIMESTAMP;

      default:
        return SQLSYNC_SQLSERVER_DATA_CHAR;
    }
}

sqlsync_sqlserver_data_type sqlsync_sqlserver_firebird_recognize_data_type(c_string_elem_ptr data_type_name, int8 *use_operator)
{
 return SQLSYNC_SQLSERVER_DATA_UNKNOWN;
}

c_vector_columns_ptr sqlsync_sqlserver_firebird_get_primary_keys(sqlsync_sqlserver * server, c_string_elem_ptr table)
{
    return sqlsync_sqlserver_default_old_get_primary_keys(server, table);
}

sqlsync_sqlserver_impl * sqlsync_sqlserver_firebird()
{
   sqlsync_sqlserver_impl * tmp = (sqlsync_sqlserver_impl *) std_malloc(sizeof(sqlsync_sqlserver_impl));
   if (tmp == NULL)
     return NULL;

   tmp->data_to_string = sqlsync_sqlserver_firebird_data_to_string;
   tmp->column_identifier = sqlsync_sqlserver_firebird_column_identifier;
   tmp->is_system_table = sqlsync_sqlserver_firebird_is_system_table;
   tmp->does_table_exist = sqlsync_sqlserver_firebird_does_table_exist;
   tmp->get_columns = sqlsync_sqlserver_firebird_get_columns;
   tmp->get_primary_keys = sqlsync_sqlserver_firebird_get_primary_keys;
   tmp->recognize_data_type = sqlsync_sqlserver_firebird_recognize_data_type;
   tmp->recognize_data_type_odbc = sqlsync_sqlserver_firebird_recognize_data_type_odbc;
   tmp->get_tables = sqlsync_sqlserver_default_get_tables;
   return tmp;
}


