/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#include "server/sqlsync_sqlserver.h"
#include "server/sqlsync_sqlserver_priv.h"

#include "server/sqlsync_default_impl.h"
#include "data_types/sqlsync_data_converter.h"
#include "data_types/sqlsync_data_numeric.h"
#include "data_types/sqlsync_data_binary.h"
#include "server/mssql/sqlsync_sqlserver_mssql.h"



c_string_elem_ptr sqlsync_sqlserver_mssql_data_to_string(c_data_elem_ptr elem, SQLSYNC_RETURN with_escapes)
{
  c_string_elem_ptr out = c_string_elem_reserve(2);
  c_string_elem_ptr tmp = NULL;
  switch (elem->_type)
  {
    case c_string_elem_type:
    {
      if (with_escapes == SQLSYNC_TRUE)
      {
        tmp =  c_data_convert_escapes((c_string_elem_ptr)elem, '\'', "\'");
        break;
      }
    }
    case c_binary_elem_type:
    {
      c_binary_elem_ptr binary_elem = (c_binary_elem_ptr) elem;
 
      tmp = c_binary_data_to_hex(binary_elem->data, binary_elem->length);
      break;
    }
    default:
    {
      tmp = c_data_convert(NULL,elem,with_escapes);
    }
  }
  if (c_data_is_numeric(elem) == SQLSYNC_TRUE)
  {
     c_string_elem_free(out);
     return tmp;
  }
  else if (elem->_type == c_binary_elem_type)
  {
    c_string_elem_append(out, "0x", 2);
    c_string_elem_append_string(out, tmp);
    c_string_elem_free(tmp);     
  }
  else
  {
    c_string_elem_append(out, "\'", 1);
    c_string_elem_append_string(out, tmp);
    c_string_elem_free(tmp);
    c_string_elem_append(out, "\'", 1);
  }
  return out;
}

c_string_elem_ptr sqlsync_sqlserver_mssql_column_identifier(c_string_elem_ptr elem)
{
  c_string_elem_ptr out = c_string_elem_reserve(2);
  c_string_elem_append(out, "[", 1);
  c_string_elem_append_string(out, elem);
  c_string_elem_append(out, "]", 1);
  return out;
}


SQLSYNC_RETURN sqlsync_sqlserver_mssql_is_system_table(c_string_elem_ptr table)
{
#define MSSQL_SIZE 2
  c_string_elem tables_mssql[MSSQL_SIZE] = {
    {c_string_elem_type, "spt_", 5, 5},
    {c_string_elem_type, "MSreplication_options", 22, 22}
  };


  uint8 i = 0;
  SQLSYNC_RETURN ret = SQLSYNC_FALSE;
  c_string_elem_ptr elem = NULL;

  if (table == NULL || table->string == NULL)
    return SQLSYNC_FAILED_NULL;

    ret = SQLSYNC_FALSE;

    for (i = 0; i < MSSQL_SIZE; i++)
    {
      elem = &tables_mssql[i];
      if (c_string_elem_ncmp (table, elem, elem->length - 1) == 0)
      {
        ret = SQLSYNC_TRUE;
        break;
      }
    }

  return ret;
#undef MSSQL_SIZE
}

sqlsync_sqlserver_data_type sqlsync_sqlserver_mssql_recognize_data_type_odbc(int16 data_type, int8 * use_operator)
{
    switch(data_type) 
    {

      case -150: //variant
        return SQLSYNC_SQLSERVER_DATA_CHAR; //??

      case -152://xml
        return SQLSYNC_SQLSERVER_DATA_XML;

      case SQL_REAL:
        return SQLSYNC_SQLSERVER_DATA_FLOAT;

      case SQL_FLOAT:
        return SQLSYNC_SQLSERVER_DATA_CHAR;

      case SQL_DECIMAL:
      case SQL_NUMERIC:
        return SQLSYNC_SQLSERVER_DATA_CHAR;

      case SQL_GUID:
        return SQLSYNC_SQLSERVER_DATA_GUID;

      case SQL_INTEGER:
         return SQLSYNC_SQLSERVER_DATA_INTEGER;
      case SQL_SMALLINT:
        return SQLSYNC_SQLSERVER_DATA_SMALLINT;
      case SQL_DATE:
        return SQLSYNC_SQLSERVER_DATA_TIMESTAMP;
      case SQL_TINYINT:
        return SQLSYNC_SQLSERVER_DATA_UTINYINT;
      case SQL_BIT:
        return SQLSYNC_SQLSERVER_DATA_UTINYINT;
      case SQL_BIGINT: 
        return SQLSYNC_SQLSERVER_DATA_BIGINT;

      case SQL_BINARY:
      case SQL_VARBINARY:
      case SQL_LONGVARBINARY:
        return SQLSYNC_SQLSERVER_DATA_BINARY;

      case SQL_WLONGVARCHAR:
        if (use_operator)
          *use_operator = 1;

      case SQL_WVARCHAR:
      case SQL_WCHAR :
        return SQLSYNC_SQLSERVER_DATA_CHAR;

      case SQL_LONGVARCHAR:
        if (use_operator)
          *use_operator = 1;

      case SQL_VARCHAR:
      

      case SQL_CHAR:
        return SQLSYNC_SQLSERVER_DATA_CHAR;

      default:
        return SQLSYNC_SQLSERVER_DATA_CHAR;
    }
}

sqlsync_sqlserver_data_type sqlsync_sqlserver_mssql_recognize_data_type(c_string_elem_ptr data_type_name, int8 * use_operator)
{
 return SQLSYNC_SQLSERVER_DATA_UNKNOWN;
}


sqlsync_sqlserver_impl * sqlsync_sqlserver_mssql()
{
   sqlsync_sqlserver_impl * tmp = (sqlsync_sqlserver_impl *) std_malloc(sizeof(sqlsync_sqlserver_impl));
   if (tmp == NULL)
     return NULL;

   tmp->data_to_string = sqlsync_sqlserver_mssql_data_to_string;
   tmp->column_identifier = sqlsync_sqlserver_mssql_column_identifier;
   tmp->is_system_table = sqlsync_sqlserver_mssql_is_system_table;
   tmp->does_table_exist = sqlsync_sqlserver_default_check_table;
   tmp->get_columns = sqlsync_sqlserver_default_get_columns;
   tmp->get_primary_keys = sqlsync_sqlserver_default_get_primary_keys;
   tmp->recognize_data_type = sqlsync_sqlserver_mssql_recognize_data_type;
   tmp->recognize_data_type_odbc = sqlsync_sqlserver_mssql_recognize_data_type_odbc;
   tmp->get_tables = sqlsync_sqlserver_default_get_tables;
   return tmp;
}
