/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#include "server/sqlsync_sqlserver.h"
#include "server/sqlsync_sqlserver_priv.h"

#include "server/sqlsync_default_impl.h"

#include "data_types/sqlsync_data_converter.h"
#include "data_types/sqlsync_data_binary.h"
#include "data_types/sqlsync_data_numeric.h"

#include "server/mysql/sqlsync_sqlserver_mysql.h"

c_string_elem_ptr sqlsync_sqlserver_mysql_data_to_string(c_data_elem_ptr elem, SQLSYNC_RETURN with_escapes)
{
  c_string_elem_ptr out = c_string_elem_reserve(2);
  c_string_elem_ptr tmp = NULL;
	
  switch (elem->_type)
  {
    case c_string_elem_type:
    {
      if (with_escapes == SQLSYNC_TRUE)
      {
        tmp =  c_data_convert_escapes((c_string_elem_ptr)elem, '\\', "\\\'");
        break;
      }
    }
    case c_binary_elem_type:
    {
      c_binary_elem_ptr binary_elem = (c_binary_elem_ptr) elem;
 
      tmp = c_binary_data_to_hex(binary_elem->data, binary_elem->length);
			
      break;
    }

    default:
      tmp = c_data_convert(NULL,elem,with_escapes);
  }

  if (c_data_is_numeric(elem) == SQLSYNC_TRUE)
  {
     c_string_elem_free(out);
     return tmp;
  }
  else
  {
    if (elem->_type == c_bit_elem_type)
      c_string_elem_append(out, "B", 1);

		if (elem->_type == c_binary_elem_type)
		  c_string_elem_append_cstring(out, "UNHEX(");
		
	  c_string_elem_append(out, "\'", 1);    
		c_string_elem_append_string(out, tmp);
    c_string_elem_free(tmp);
			  
		c_string_elem_append(out, "\'", 1);
		
		if (elem->_type == c_binary_elem_type)
		  c_string_elem_append_cstring(out, ")");

  }
  return out;
}

c_string_elem_ptr sqlsync_sqlserver_mysql_column_identifier(c_string_elem_ptr elem)
{
  c_string_elem_ptr out = c_string_elem_reserve(2);
  c_string_elem_append(out, "`", 1);
  c_string_elem_append_string(out, elem);
  c_string_elem_append(out, "`", 1);
  return out;
}

SQLSYNC_RETURN sqlsync_sqlserver_mysql_is_system_table(c_string_elem_ptr table)
{
  return SQLSYNC_FALSE;
}


sqlsync_sqlserver_data_type sqlsync_sqlserver_mysql_recognize_data_type(c_string_elem_ptr data_type_name, int8 * use_operator)
{
  struct type
  {
    c_string_elem type;
    int8 like;
  };
  struct type types[] =
 {
     {{SQLSYNC_SQLSERVER_DATA_UTINYINT, "tinyint unsigned", 16, 16}, 0},
     {{SQLSYNC_SQLSERVER_DATA_TINYINT, "tinyint", 7, 7}, 0},
     {{SQLSYNC_SQLSERVER_DATA_USMALLINT, "smallint unsigned", 17, 17}, 0},
     {{SQLSYNC_SQLSERVER_DATA_SMALLINT, "smallint", 8, 8}, 0},
     {{SQLSYNC_SQLSERVER_DATA_UMEDIUMINT, "mediumint unsigned", 18, 18}, 0},
     {{SQLSYNC_SQLSERVER_DATA_MEDIUMINT, "mediumint", 9, 9}, 0},
     {{SQLSYNC_SQLSERVER_DATA_UINTEGER, "integer unsigned", 17, 17}, 0},
     {{SQLSYNC_SQLSERVER_DATA_INTEGER, "integer", 7, 7}, 0},
     {{SQLSYNC_SQLSERVER_DATA_UBIGINT, "bigint unsienged", 15, 15}, 0},
     {{SQLSYNC_SQLSERVER_DATA_BIGINT, "bigint", 6, 6}, 0},
     {{SQLSYNC_SQLSERVER_DATA_CHAR, "float unsigned", 14, 14}, 1},
     {{SQLSYNC_SQLSERVER_DATA_CHAR, "float", 5, 5}, 1},
     {{SQLSYNC_SQLSERVER_DATA_CHAR, "double unsigned", 15, 15}, 1},
     {{SQLSYNC_SQLSERVER_DATA_CHAR, "double", 6, 6}, 1},
     {{SQLSYNC_SQLSERVER_DATA_BINARY, "binary", 6, 6}, 0},
     {{SQLSYNC_SQLSERVER_DATA_BIT, "bit", 3, 3}, 0},
     {{SQLSYNC_SQLSERVER_DATA_CHAR, "char", 4, 4}, 0},
     {{SQLSYNC_SQLSERVER_DATA_CHAR, "varchar", 7, 7}, 0},
     {{SQLSYNC_SQLSERVER_DATA_BLOB, "blob", 4, 4}, 0},
     {{SQLSYNC_SQLSERVER_DATA_CHAR, "text", 4, 4}, 0},
     {{SQLSYNC_SQLSERVER_DATA_TIMESTAMP, "timestamp", 9, 9}, 0},
     {{SQLSYNC_SQLSERVER_DATA_TIMESTAMP, "datetime", 8, 8}, 0},
     {{SQLSYNC_SQLSERVER_DATA_DATE, "date", 4, 4}, 0},
     {{SQLSYNC_SQLSERVER_DATA_TIME, "time", 4, 4}, 0},
     {{SQLSYNC_SQLSERVER_DATA_SMALLINT, "year", 4, 4}, 0},
     {{SQLSYNC_SQLSERVER_DATA_CHAR, "decimal", 7, 7}, 0},
 };

  uint8 i = 0;
  for (i = 0; i< sizeof(types)/sizeof(types[0]); i++) 
  {
      if (c_string_elem_ncmp(data_type_name, &types[i].type, types[i].type.length) == 0)
      {
        if (use_operator)
          *use_operator = types[i].like;

        return types[i].type._type;
      }
  }

 return SQLSYNC_SQLSERVER_DATA_CHAR;
}

sqlsync_sqlserver_impl * sqlsync_sqlserver_mysql()
{
   sqlsync_sqlserver_impl * tmp = (sqlsync_sqlserver_impl *) std_malloc(sizeof(sqlsync_sqlserver_impl));
   if (tmp == NULL)
     return NULL;

   tmp->data_to_string = sqlsync_sqlserver_mysql_data_to_string;
   tmp->column_identifier = sqlsync_sqlserver_mysql_column_identifier;
   tmp->is_system_table = sqlsync_sqlserver_mysql_is_system_table;
   tmp->does_table_exist = sqlsync_sqlserver_default_check_table;
   tmp->get_columns = sqlsync_sqlserver_default_get_columns;
   tmp->get_primary_keys = sqlsync_sqlserver_default_get_primary_keys;
   tmp->recognize_data_type = sqlsync_sqlserver_mysql_recognize_data_type;
   tmp->get_tables = sqlsync_sqlserver_default_get_tables;
   return tmp;
}


