/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#include "server/sqlsync_sqlserver.h"
#include "server/sqlsync_sqlserver_priv.h"

#include "server/sqlsync_default_impl.h"
#include "data_types/sqlsync_data_converter.h"
#include "data_types/sqlsync_data_binary.h"
#include "data_types/sqlsync_data_numeric.h"

#include "server/oracle/sqlsync_sqlserver_oracle.h"

#ifdef DEBUG
#include "sqlsync_debug.h"
#endif

c_string_elem_ptr sqlsync_sqlserver_oracle_data_to_string(c_data_elem_ptr elem, SQLSYNC_RETURN with_escapes)
{
  c_string_elem_ptr out = c_string_elem_reserve(2);
  c_string_elem_ptr tmp = NULL;

  if (elem->_type == c_binary_elem_type) 
  {
     c_binary_elem_ptr binary = (c_binary_elem_ptr) elem;
     char hex_table[]="0123456789ABCDEF";
     char hex[2]="AA";
     uint32 i = 0;
     tmp = c_string_elem_empty();
     for (i = 0; i < binary->length; i++) 
     {
         hex[0] = hex_table[binary->data[i]>>4];
         hex[1] = hex_table[binary->data[i] & 0x0f];
         c_string_elem_append(tmp, hex, 2);
     }

  }
  else
  switch (elem->_type)
  {
    case c_string_elem_type:
    {
      if (with_escapes == SQLSYNC_TRUE)
      {
        tmp =  c_data_convert_escapes((c_string_elem_ptr)elem, '\'', "\'");
        break;
      }
    }
    default:
      tmp = c_data_convert(NULL,elem,with_escapes);
  }
  if (c_data_is_numeric(elem) == SQLSYNC_TRUE)
  {
     c_string_elem_free(out);
     return tmp;
  }
  else
  {
    if (elem->_type == c_date_elem_type)
    {
      c_string_elem_append(out, "date", 4);
    }
    else if(elem->_type == c_timestamp_elem_type)
    {
      c_string_elem_append(out, "timestamp", 9);
    }
    c_string_elem_append(out, "\'", 1);
    c_string_elem_append_string(out, tmp);
    c_string_elem_free(tmp);
    c_string_elem_append(out, "\'", 1);
  }
  return out;
}

c_string_elem_ptr sqlsync_sqlserver_oracle_column_identifier(c_string_elem_ptr elem)
{
  c_string_elem_ptr out = c_string_elem_reserve(2);

  c_string_elem_append(out, "\"", 1);
  c_string_elem_append_string(out, elem);
  c_string_elem_append(out, "\"", 1);


  return out;
}

c_vector_columns_ptr sqlsync_sqlserver_oracle_get_columns(sqlsync_sqlserver * server, c_string_elem_ptr table)
{
    SQLRETURN ret;
    c_string_elem_ptr name = NULL;
    c_vector_columns_ptr columns;
    c_column_elem_ptr column_data = NULL;

   sqlsync_create_stmt (server);

    ret =
      SQLColumns (server->hstmt, NULL, 0, NULL, 0, (SQLCHAR *) table->string, (SQLSMALLINT) table->length,
                  NULL, 0);

    if (SQLSYNC_IS_ERROR (ret))
    {
  #ifdef DEBUG
      sqlsync_debug ("sqlsync_get_columns failed %d\n", ret);
  #endif

      server->last_bad_handle = SQL_HANDLE_STMT;
      return NULL;
    }


    columns = c_vector_columns_init (DEFAULT_DATA_SIZE);

  #ifdef DEBUG
    sqlsync_debug ("Retrieving columns\n");
  #endif

    while (SQL_SUCCEEDED (ret = SQLFetch (server->hstmt)))
    {
        column_data = NULL;
        c_int32_elem_ptr max_size = NULL;
        c_string_elem_ptr data_type = NULL;
        c_data_elem_ptr tmp = NULL;
        int8 use_operator = 0;

        sqlsync_get_data (server->hstmt, SQLColumns_COLUMN_NAME, SQLSYNC_SQLSERVER_DATA_CHAR, 128, (c_data_elem_ptr *) &tmp);
        name = (c_string_elem_ptr) tmp;

        sqlsync_get_data (server->hstmt, SQLColumns_COLUMN_SIZE, SQLSYNC_SQLSERVER_DATA_INTEGER, 4, (c_data_elem_ptr *) &tmp);
        max_size = (c_int32_elem_ptr) tmp;

        sqlsync_get_data (server->hstmt, SQLColumns_TYPE_NAME, SQLSYNC_SQLSERVER_DATA_CHAR, 128, (c_data_elem_ptr *) &tmp);
        data_type = (c_string_elem_ptr) tmp;

        
        column_data = c_column_elem_create (name, max_size->value, sqlsync_sqlserver_recognize_data_type(server,data_type, &use_operator), use_operator);

        switch(column_data->data_type)
        {
          case SQLSYNC_SQLSERVER_DATA_NUMERIC:
          //case SQLSYNC_SQLSERVER_DATA_FLOAT:
          //case SQLSYNC_SQLSERVER_DATA_DOUBLE:
          {
            column_data->max_length +=2;
            column_data->data_type = SQLSYNC_SQLSERVER_DATA_CHAR;
            break;
          }
        }


        /* For some reason driver returns twice the same columns */
        SQLSYNC_RETURN ret;
        uint32 pos = 0;
        pos = c_vector_columns_find(columns, column_data, &ret);
        if (ret == SQLSYNC_TRUE)
        {
            c_column_elem_free(column_data);
            column_data = NULL;
        }

        c_string_elem_free (name);
        c_int32_elem_free (max_size);
        c_string_elem_free (data_type);

        if (column_data == NULL)
          continue;

  #ifdef DEBUG
      sqlsync_debug ("Adding column\n");
  #endif

      (void) c_vector_columns_add_last (columns, column_data);
    }

    return columns;
}

SQLSYNC_RETURN sqlsync_sqlserver_oracle_is_system_table(c_string_elem_ptr table)
{
#define ORACLE_SIZE 20
  c_string_elem tables_oracle[ORACLE_SIZE] = {
    {c_string_elem_type, "DR$", 4, 4},
    {c_string_elem_type, "WWV_", 5, 5},
    {c_string_elem_type, "OGIS", 5, 5},
    {c_string_elem_type, "SDO_", 5, 5},
    {c_string_elem_type, "BIN$", 5, 5},
    {c_string_elem_type, "XDB$", 5, 5},
    {c_string_elem_type, "SYS_", 5, 5},
    {c_string_elem_type, "AQ$", 4, 4},
    {c_string_elem_type, "DEF$", 5, 5},
    {c_string_elem_type, "HELP", 5, 5},
    {c_string_elem_type, "LOGMNR_", 8, 8},
    {c_string_elem_type, "LOGSTDBY$", 10, 10},
    {c_string_elem_type, "MVIEW$_", 8, 8},
    {c_string_elem_type, "OL$", 4, 4},
    {c_string_elem_type, "REPCAT$_", 9, 9},
    {c_string_elem_type, "SQLPLUS_PRODUCT_PROFILE", 24, 24},
    {c_string_elem_type, "MGMT_", 6, 6},
    {c_string_elem_type, "HTMLDB_", 8, 8},
    {c_string_elem_type, "SRS$", 5, 5},
    {c_string_elem_type, "MD$", 4, 4},
    
  };
  uint8 i = 0;
  SQLSYNC_RETURN ret = SQLSYNC_FALSE;
  c_string_elem_ptr elem = NULL;

  if (table == NULL || table->string == NULL)
    return SQLSYNC_FAILED_NULL;

    ret = SQLSYNC_FALSE;

    for (i = 0; i < ORACLE_SIZE; i++)
    {
      elem = &tables_oracle[i];
      if (c_string_elem_ncmp(table,elem, elem->length-1) == 0)
      {
          ret = SQLSYNC_TRUE;
          break;
      }
    }

  return ret;
#undef ORACLE_SIZE
}


sqlsync_sqlserver_data_type sqlsync_sqlserver_oracle_recognize_data_type(c_string_elem_ptr data_type_name, int8 * use_operator)
{
 c_string_elem types[] =
 {
     {SQLSYNC_SQLSERVER_DATA_CHAR, "CHAR", 4, 4},
     {SQLSYNC_SQLSERVER_DATA_CHAR, "NCHAR", 5, 5},
     {SQLSYNC_SQLSERVER_DATA_BLOB, "BLOB", 4, 4},
     {SQLSYNC_SQLSERVER_DATA_CBLOB, "CLOB", 4, 4},
     {SQLSYNC_SQLSERVER_DATA_CHAR, "NVARCHAR2", 9, 9},
     {SQLSYNC_SQLSERVER_DATA_CHAR, "VARCHAR2", 8, 8},
     {SQLSYNC_SQLSERVER_DATA_TIMESTAMP, "TIMESTAMP", 9, 9},
     {SQLSYNC_SQLSERVER_DATA_DOUBLE, "DOUBLE PRECISION", 16, 16},
     {SQLSYNC_SQLSERVER_DATA_FLOAT, "BINARY_FLOAT", 12, 12},
     {SQLSYNC_SQLSERVER_DATA_NUMERIC, "DECIMAL", 7, 7},
     {SQLSYNC_SQLSERVER_DATA_DATE, "DATE", 4, 4},
     {SQLSYNC_SQLSERVER_DATA_CHAR, "XMLTYPE", 7, 7},
 };

  uint8 i = 0;
  for (i = 0; i< sizeof(types)/sizeof(types[0]); i++) 
  {
      if (c_string_elem_ncmp(data_type_name, &types[i], types[i].length) == 0)
      return types[i]._type;
  }

 return SQLSYNC_SQLSERVER_DATA_CHAR;
}

extern c_vector_columns_ptr
sqlsync_old_get_primary_keys (sqlsync_sqlserver * server, c_string_elem_ptr table);

sqlsync_sqlserver_impl * sqlsync_sqlserver_oracle()
{
   sqlsync_sqlserver_impl * tmp = (sqlsync_sqlserver_impl *) std_malloc(sizeof(sqlsync_sqlserver_impl));
   if (tmp == NULL)
     return NULL;

   tmp->data_to_string = sqlsync_sqlserver_oracle_data_to_string;
   tmp->column_identifier = sqlsync_sqlserver_oracle_column_identifier;
   tmp->is_system_table = sqlsync_sqlserver_oracle_is_system_table;
   tmp->does_table_exist = /*sqlsync_sqlserver_oracle_does_table_exist;*/ sqlsync_sqlserver_default_check_table;
   tmp->get_columns = sqlsync_sqlserver_oracle_get_columns;
   tmp->get_primary_keys = sqlsync_sqlserver_default_old_get_primary_keys;
   tmp->recognize_data_type = sqlsync_sqlserver_oracle_recognize_data_type;
   tmp->get_tables = sqlsync_sqlserver_default_get_tables;
   return tmp;
}


