/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/


#include "server/sqlsync_sqlserver.h"
#include "server/sqlsync_sqlserver_priv.h"

#include "server/sqlsync_default_impl.h"
#include "data_types/sqlsync_data_converter.h"
#include "data_types/sqlsync_data_numeric.h"
#include "data_types/sqlsync_data_binary.h"

#include "server/postgresql/sqlsync_sqlserver_postgresql.h"

c_string_elem_ptr sqlsync_sqlserver_postgresql_data_to_string(c_data_elem_ptr elem, SQLSYNC_RETURN with_escapes)
{
  c_string_elem_ptr out = c_string_elem_reserve(2);
  c_string_elem_ptr tmp = NULL;
  switch (elem->_type)
  {
  case c_binary_elem_type:
    {
      c_string_elem_ptr str = c_string_elem_empty();
      c_binary_elem_ptr binary_elem = (c_binary_elem_ptr) elem;

      c_string_elem_append(str, (char *)binary_elem->data, binary_elem->length);
      tmp = c_data_convert_escapes(str, '\\', "\\");
      c_string_elem_free(str);
      tmp = c_binary_data_to_octet((void *)tmp->string, tmp->length);
      break;
    }
    case c_string_elem_type:
    {
      if (with_escapes == SQLSYNC_TRUE)
      {
        tmp =  c_data_convert_escapes((c_string_elem_ptr)elem, '\\', "\\\'");
        break;
      }
    }
    default:
      tmp = c_data_convert(NULL,elem,with_escapes);
  }
  if (c_data_is_numeric(elem) == SQLSYNC_TRUE && 
      elem->_type != c_float_elem_type && 
      elem->_type != c_double_elem_type)
  {
     c_string_elem_free(out);
     return tmp;
  }
  else
  {
    c_string_elem_append(out, "\'", 1);
    c_string_elem_append_string(out, tmp);
    c_string_elem_free(tmp);
    c_string_elem_append(out, "\'", 1);
  }
  return out;
}

c_string_elem_ptr sqlsync_sqlserver_postgresql_column_identifier(c_string_elem_ptr elem)
{
  c_string_elem_ptr out = c_string_elem_reserve(2);
  c_string_elem_append(out, "\"", 1);
  c_string_elem_append_string(out, elem);
  c_string_elem_append(out, "\"", 1);
  return out;
}


SQLSYNC_RETURN sqlsync_sqlserver_postgresql_does_table_exist(sqlsync_sqlserver * server, c_string_elem_ptr table)
{
  return sqlsync_sqlserver_default_check_table(server, table);
}

c_vector_columns_ptr sqlsync_sqlserver_postgresql_get_columns(sqlsync_sqlserver * server, c_string_elem_ptr table)
{
  return sqlsync_sqlserver_default_get_columns(server, table);
}

SQLSYNC_RETURN sqlsync_sqlserver_postgresql_is_system_table(c_string_elem_ptr table)
{

#define POSTGRESQL_SIZE 6
  c_string_elem tables_postgresql[POSTGRESQL_SIZE] = {
    {c_string_elem_type, "sql_features", 12, 12},
    {c_string_elem_type, "sql_implementation_info", 23, 23},
    {c_string_elem_type, "sql_languages", 13, 13},
    {c_string_elem_type, "sql_packages", 12, 12},
    {c_string_elem_type, "sql_sizing", 10, 10},
    {c_string_elem_type, "sql_sizing_profiles", 19, 19}
  };

  uint8 i = 0;
  SQLSYNC_RETURN ret = SQLSYNC_FALSE;
  c_string_elem_ptr elem = NULL;

  if (table == NULL || table->string == NULL)
    return SQLSYNC_FAILED_NULL;

    ret = SQLSYNC_FALSE;

    for (i = 0; i < POSTGRESQL_SIZE; i++)
    {
      elem = &tables_postgresql[i];
      if (c_string_elem_cmp (table, elem) == 0)
      {
        ret = SQLSYNC_TRUE;
        break;
      }
    }

  return ret;
#undef POSTGRESQL_SIZE
}


sqlsync_sqlserver_data_type sqlsync_sqlserver_postgresql_recognize_data_type(c_string_elem_ptr data_type_name, int8 * use_operator)
{
  struct type
  {
    c_string_elem type;
    int8 use_operator;
  };

  struct type types[] =
  {
     {{SQLSYNC_SQLSERVER_DATA_SMALLINT, "int2", 4, 4},0},
     {{SQLSYNC_SQLSERVER_DATA_INTEGER, "int4", 4, 4},0},
     {{SQLSYNC_SQLSERVER_DATA_CHAR, "int8", 4, 4},0},
/*  broken on linux   {{SQLSYNC_SQLSERVER_DATA_BIGINT, "int8", 4, 4},0},*/
     {{SQLSYNC_SQLSERVER_DATA_NUMERIC, "numeric", 7, 7},0},
     {{SQLSYNC_SQLSERVER_DATA_FLOAT, "float4", 6, 6},0},
     {{SQLSYNC_SQLSERVER_DATA_DOUBLE, "float8", 6, 6},0},
     /*{{"bit", 3, 3}, SQLSYNC_SQLSERVER_DATA_BIT},*/
     {{SQLSYNC_SQLSERVER_DATA_CHAR, "bit", 3, 3},0},
     {{SQLSYNC_SQLSERVER_DATA_BINARY, "bytea", 5, 5},0},
     /*{{"varbit", 6, 6}, SQLSYNC_SQLSERVER_DATA_BIT},*/
     {{SQLSYNC_SQLSERVER_DATA_CHAR, "varbit", 6, 6},0},
     {{SQLSYNC_SQLSERVER_DATA_CHAR, "bpchar", 6, 6},0},
     {{SQLSYNC_SQLSERVER_DATA_CHAR, "varchar", 7, 7},0},
     {{SQLSYNC_SQLSERVER_DATA_CHAR, "text", 4, 4},0},
     {{SQLSYNC_SQLSERVER_DATA_TIMESTAMP, "timestamptz", 11, 11},0},
     {{SQLSYNC_SQLSERVER_DATA_TIMESTAMP, "timestamp", 9, 9},0},
     /*{{"timestamptz", 9, 9}, SQLSYNC_SQLSERVER_DATA_TIMESTAMP_TZ},*/
     /*{{"interval", 8, 8}, SQLSYNC_SQLSERVER_DATA_INTERVAL}*/
     {{SQLSYNC_SQLSERVER_DATA_CHAR, "interval", 8, 8},0},
     {{SQLSYNC_SQLSERVER_DATA_DATE, "date", 4, 4},0},
     {{SQLSYNC_SQLSERVER_DATA_CHAR, "timetz", 6, 6},0},
     {{SQLSYNC_SQLSERVER_DATA_TIME, "time", 4, 4},0},
     /*{{"timetz", 4, 4}, SQLSYNC_SQLSERVER_DATA_TIME_TZ}*/
     {{SQLSYNC_SQLSERVER_DATA_BOOL, "bool", 4, 4},0},
     {{SQLSYNC_SQLSERVER_DATA_CHAR, "circle", 6, 6},2},
     {{SQLSYNC_SQLSERVER_DATA_CHAR, "path", 4, 4},3},
     {{SQLSYNC_SQLSERVER_DATA_CHAR, "point", 5, 5},2},
     {{SQLSYNC_SQLSERVER_DATA_CHAR, "polygon", 6, 6},2},

  };

  uint8 i = 0;
  for (i = 0; i< sizeof(types)/sizeof(types[0]); i++) 
  {
      if (c_string_elem_ncmp(data_type_name, &types[i].type, types[i].type.length) == 0)
      {
        if (use_operator)
          *use_operator = types[i].use_operator;
        return types[i].type._type;
      }
  }

 return SQLSYNC_SQLSERVER_DATA_CHAR;
}


c_vector_columns_ptr sqlsync_sqlserver_postgresql_get_primary_keys(sqlsync_sqlserver * server, c_string_elem_ptr table)
{
    return sqlsync_sqlserver_default_get_primary_keys(server, table);
}


sqlsync_sqlserver_impl * sqlsync_sqlserver_postgresql()
{
   sqlsync_sqlserver_impl * tmp = (sqlsync_sqlserver_impl *) std_malloc(sizeof(sqlsync_sqlserver_impl));
   if (tmp == NULL)
     return NULL;

   tmp->data_to_string = sqlsync_sqlserver_postgresql_data_to_string;
   tmp->column_identifier = sqlsync_sqlserver_postgresql_column_identifier;
   tmp->is_system_table = sqlsync_sqlserver_postgresql_is_system_table;
   tmp->does_table_exist = sqlsync_sqlserver_postgresql_does_table_exist;
   tmp->get_columns = sqlsync_sqlserver_postgresql_get_columns;
   tmp->get_primary_keys = sqlsync_sqlserver_postgresql_get_primary_keys;
   tmp->recognize_data_type = sqlsync_sqlserver_postgresql_recognize_data_type;
   tmp->get_tables = sqlsync_sqlserver_default_get_tables;
   return tmp;
}


