/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#include "sqlsync_dbstruct.h"

#include "sqlsync_compare_records.h"

#include "sqlsync_translation.h"

#include "sqlsync_debug.h"

#include "data_types/sqlsync_data_binary.h"
#include "data_types/sqlsync_data_numeric.h"
#include "data_types/sqlsync_data_misc.h"
#include "data_types/sqlsync_data_string.h"

#include "server/sqlsync_sqlserver_priv.h"
#include "server/sqlsync_sqlserver_data_types.h"

#include "sqlsync_user_internal.h"


SQLSYNC_RETURN
sqlsync_get_data (void *hstmt_ptr, uint8 nr_column, sqlsync_sqlserver_data_type data_type, uint32 max_data_size, c_data_elem_ptr * _elem)
{
  void *data = NULL;
  uint32 data_length = max_data_size;
  SQLINTEGER length = 0;
  SQLSMALLINT type = SQL_C_CHAR;
  SQLRETURN ret = 0;
  SQLHSTMT hstmt = (SQLHSTMT) hstmt_ptr;
  *_elem = NULL;

  type = sqlsync_sqlserver_map_to_odbc(data_type);

  sqlsync_sqlserver_type_size(data_type, &data_length);

  c_data_elem_ptr elem = sqlsync_sqlserver_create_data(data_type, data_length);

  if (elem == NULL)
    return SQLSYNC_FAILED_NULL;

  data = c_data_elem_data(elem);
  data_length = c_data_elem_size(elem);

  if(type == SQL_C_BIT) 
  {
      type = SQL_C_BINARY;
  }

  do
  {
    length = 0;
    ret = SQLGetData (hstmt, nr_column, type, data, data_length, &length);

    if (ret == SQL_NULL_DATA || ret == SQL_NO_DATA)
      return SQLSYNC_FAILED_NULL;

    if (SQLSYNC_IS_ERROR (ret))
    {
      /* Print some error info or .. */

      return SQLSYNC_FAILED;
    }

    if (length == SQL_NULL_DATA)
    {
      if (elem)
        c_data_free(elem);

      return SQLSYNC_FAILED_NULL;
    }

    if ((length - (int32) data_length) >= 0)
    {
      switch(elem->_type)
      {
        case c_binary_elem_type:
        case c_bit_elem_type:
          {
            c_binary_elem_ptr binary_elem = (c_binary_elem_ptr) elem;
    
            binary_elem->length += data_length;


            if ((length - (int32) data_length) > 0 && 
                c_binary_elem_append_reserve (binary_elem, length - data_length) !=
                SQLSYNC_SUCCESS)
              return SQLSYNC_FAILED;
    
            data = (char *) binary_elem->data + data_length ;
            data_length = length - data_length;

            break;
          }
          case c_string_elem_type:
          {
            c_string_elem_ptr string_elem = (c_string_elem_ptr) elem;

            if (length >= data_length && length > 0) 
              data_length --;

            string_elem->length += data_length;

            if (c_string_elem_append_reserve (string_elem, length + 1 - data_length) !=
                SQLSYNC_SUCCESS)
              return SQLSYNC_FAILED;
    
            data = (char *) string_elem->string + data_length;
            data_length = length - data_length + 1;
          }
      }
    }
    else
    {
      switch(elem->_type)
      {
        case c_binary_elem_type:
        case c_bit_elem_type:
        {
          c_binary_elem_ptr binary_elem = (c_binary_elem_ptr) elem;
  
          binary_elem->length += length;
          break;
        }

        case c_string_elem_type:
        {
          c_string_elem_ptr string_elem = (c_string_elem_ptr) elem;
  
          string_elem->length += length;
          break;
        }
      }
    }

  }
  while (length != SQL_NULL_DATA && (uint32) length > data_length);

 *_elem = elem;
  return SQLSYNC_SUCCESS;
}


c_vector_data_ptr
sqlsync_get_row (sqlsync_sqlserver * server, c_vector_columns_ptr columns)
{
  c_vector_data_ptr row = NULL;
  c_column_elem_ptr column = NULL;
  uint32 nr_columns = c_vector_columns_size (columns);
  uint32 i = 0;
  SQLRETURN ret = 0;

  row = c_vector_data_init (nr_columns);

  ret = SQLFetch (server->hstmt_query);

  if (ret == SQL_NO_DATA)
  {
    c_vector_data_destroy (row);
    return NULL;
  }
  if (SQLSYNC_IS_ERROR (ret))
  {
#ifdef DEBUG
    sqlsync_debug ("sqlsync_get_row failed %d\n", ret);
#endif

    server->last_bad_handle = SQL_HANDLE_STMT;
    c_vector_data_destroy(row);
    return NULL;
  }


  for (i = 1; i <= nr_columns; i++)
  {
    c_data_elem_ptr elem = NULL;

    column = c_vector_columns_pos (columns, i - 1);


    if (column == NULL)
      continue;

#ifdef DEBUG
    sqlsync_debug ("Column name ");
    sqlsync_println (SQLSYNC_DEBUG, column->column);
    sqlsync_debug ("Maximum length %ld\n", column->max_length);
#endif


    ret = sqlsync_get_data (server->hstmt_query, i, column->data_type, column->max_length, &elem);

    if (ret == SQLSYNC_FAILED_NULL)
    {
#ifdef DEBUG
      sqlsync_debug ("sqlsync_get_row failed SQLSYNC_FAILED_NULL\n", 0);
#endif

      c_vector_data_add (row, NULL, i - 1);

      if (elem != NULL)
        c_data_free (elem);
      continue;
    }
    else if (ret != SQLSYNC_SUCCESS)
    {
      if (row != NULL)
        c_vector_data_destroy (row);
      
      if (elem != NULL)
        c_data_free (elem);
#ifdef DEBUG
      sqlsync_debug ("sqlsync_get_row failed error %d\n", ret);
#endif
      server->error = SQLSYNC_FAILED_GET_ROW;
      server->last_bad_handle = SQL_HANDLE_STMT;

      return NULL;
    }
    c_vector_data_add (row, elem, i - 1);
  }

  return row;
}


void
sqlsync_get_rows (sqlsync_sqlserver * server_src, sqlsync_sqlserver * server_dst,
                  c_vector_columns_ptr src_columns, c_vector_columns_ptr dst_columns, c_grid_data_ptr grid_src,
                  c_grid_data_ptr grid_dst, c_vector_int32_ptr primary_keys_pos, uint32 * equal_rows)
{

  c_vector_data_ptr row_src = NULL;
  c_vector_data_ptr row_dst = NULL;

  int ret_cmp = SQLSYNC_REC_MATCH;
  int row_src_pos_added = -1;
  int row_dst_pos_added = -1;
  int row_src_pos = 0;
  int row_dst_pos = 0;
  short end = 0;

  *equal_rows = 0;


  do
  {
    /* Get row from second database, i-th table */
    if (ret_cmp != SQLSYNC_REC_MISS)
    {
      row_dst_pos_added = row_dst_pos++;
#ifdef DEBUG
      sqlsync_debug ("Getting dst row from destination DSN\n");
#endif
      row_dst = sqlsync_get_row (server_dst, dst_columns);

#ifdef DEBUG
      sqlsync_debug ("Getting dst row done from destination DSN\n");
#endif
    }

    /* Get row from first database, i-th table */
    if (ret_cmp != SQLSYNC_REC_ADD)
    {

      row_src_pos_added = row_src_pos++;
#ifdef DEBUG
      sqlsync_debug ("Getting src row from source DSN\n");
#endif

      row_src = sqlsync_get_row (server_src, src_columns);

#ifdef DEBUG
      sqlsync_debug ("Getting src row done from source DSN\n");
#endif


    }

    /* If we are retreiving rows from tables, 
     * there is a possibility that rows are in the same order.
     * To prevent sorting LARGE grids skip records 
     * which are exactly the same.
     */

#ifdef DEBUG
    sqlsync_debug ("Comparing two rows\n");

    sqlsync_debug ("row_src\n");
//    c_grid_print_elem (row_src, columns);

    sqlsync_debug ("row_dst\n");
//    c_grid_print_elem (row_dst, columns);
#endif

    if (SQLSYNC_REC_MATCH !=
        (ret_cmp = sqlsync_compare_records (row_src, row_dst, primary_keys_pos)))
    {
#ifdef DEBUG
      sqlsync_debug ("Result of comparing records cmp %ld\n", ret_cmp);
#endif

      if (row_src != NULL && (ret_cmp != SQLSYNC_REC_ADD) && (row_src_pos != row_src_pos_added))
      {
        row_src_pos_added = row_src_pos;
#ifdef DEBUG
        sqlsync_debug ("Added missing row to grid_src\n");
#endif
        (void) c_grid_data_add_last (grid_src, row_src);
      }

      if (row_dst != NULL && (ret_cmp != SQLSYNC_REC_MISS) && (row_dst_pos != row_dst_pos_added))
      {
        row_dst_pos_added = row_dst_pos;

#ifdef DEBUG
        sqlsync_debug ("Added additional row to grid_dst\n");
#endif

        (void) c_grid_data_add_last (grid_dst, row_dst);
      }

    }
    else
    {
      if (row_dst != NULL && row_src != NULL)
      {
#ifdef DEBUG
        sqlsync_debug ("Records are equal\n");
#endif
        (*equal_rows)++;
        row_src_pos_added = row_src_pos;
        row_dst_pos_added = row_dst_pos;
        (void) c_vector_data_destroy (row_src);
        (void) c_vector_data_destroy (row_dst);
        end = 0;
        continue;
      }
    }

    if (row_src != NULL || row_dst != NULL)
      end = 0;
    else
      end = 1;

  }
  while (end != 1);
}



/*Adding primary columns which are not added to columns list*/
void
sqlsync_add_primary_columns (sqlsync_sqlserver * server, c_string_elem_ptr table,
                             c_vector_columns_ptr columns)
{
  c_vector_columns_ptr primaries = NULL;
  SQLSYNC_RETURN error = SQLSYNC_FAILED;
  uint32 j = 0;
  uint32 size = 0;

  primaries = sqlsync_sqlserver_get_primary_keys (server, table);

  size = c_vector_columns_size (primaries);
  for (j = 0; j < size; j++)
  {
    c_column_elem_ptr column = c_vector_columns_pos (primaries, j);

    (void) c_vector_columns_find (columns, column, &error);

    if (error == SQLSYNC_FAILED)
    {
      c_vector_columns_add_last_copy (columns, column);
    }
  }
  c_vector_columns_destroy (primaries);
}

int8 sqlsync_check_icmp_column(c_data_elem_ptr src, c_data_elem_ptr dst)
{
  return c_string_elem_icmp(((c_column_elem_ptr)src)->column, ((c_column_elem_ptr)dst)->column);
}

c_vector_columns_ptr
sqlsync_get_valid_columns (sqlsync_sqlserver * server,
                           c_string_elem_ptr table, c_vector_columns_ptr columns)
{
  c_vector_columns_ptr valid_columns;
  c_vector_columns_ptr read_columns = NULL;
  valid_columns = c_vector_columns_init (DEFAULT_DATA_SIZE);
  uint32 i = 0;

  read_columns = sqlsync_sqlserver_get_columns(server, table);

  for (i = 0; i< c_vector_columns_size(read_columns); i++)
  {
    SQLSYNC_RETURN err = SQLSYNC_FAILED;
    if (sqlsync_sqlserver_cs_tables_columns(server) == SQLSYNC_TRUE)
      c_vector_data_find_wf((c_vector_data_ptr)columns, (c_data_elem_ptr)c_vector_columns_pos(read_columns, i),
                             sqlsync_check_icmp_column, &err);
    else 
      c_vector_columns_find(columns, c_vector_columns_pos(read_columns, i), &err);

    if (err == SQLSYNC_SUCCESS)
    {
        c_vector_columns_add_last_copy(valid_columns, c_vector_columns_pos(read_columns, i));
    }
  }

  c_vector_columns_destroy(read_columns);

  return valid_columns;

}

c_vector_columns_ptr
sqlsync_get_common_columns (sqlsync_sqlserver * server_src, sqlsync_sqlserver * server_dst,
                           c_string_elem_ptr table_src, c_string_elem_ptr table_dst)
{
  c_vector_columns_ptr elem_table_columns = NULL;
  c_vector_columns_ptr tmp_valid_columns = NULL;

  tmp_valid_columns = sqlsync_sqlserver_get_columns (server_src, table_src);
    
  elem_table_columns = sqlsync_get_valid_columns (server_dst, table_dst, tmp_valid_columns);
    
  c_vector_columns_destroy (tmp_valid_columns);

  return elem_table_columns;
}

SQLSYNC_RETURN
sqlsync_find_set_primary_keys (sqlsync_sqlserver * server_src, sqlsync_sqlserver * server_dst,
                               c_string_elem_ptr table,
                               c_vector_columns_ptr columns,
                               c_vector_columns_ptr * primary_keys, c_vector_int32_ptr * primary_keys_pos)
{

  int size = 0;
  int i = 0;
  c_vector_columns_ptr pk = NULL;


  if (server_src == NULL)
    return SQLSYNC_FAILED_NULL;

  c_vector_int32_destroy (*primary_keys_pos);

  c_vector_columns_destroy (*primary_keys);


  *primary_keys_pos = c_vector_int32_init (1);

  pk = sqlsync_sqlserver_get_primary_keys (server_src, table);
  *primary_keys = sqlsync_get_valid_columns(server_dst, table, pk);

  c_vector_columns_destroy(pk);

  size = c_vector_columns_size (*primary_keys);

#ifdef DEBUG
  sqlsync_debug ("Number of primary keys %ld\n", size);
#endif

  for (i = 0; i < size; i++)
  {
    SQLSYNC_RETURN error;

#ifdef DEBUG
    sqlsync_debug_details ("Primary column name %s\n",
                           ((c_column_elem_ptr)c_vector_columns_pos (*primary_keys, i))->column->string);
#endif

    int pos = c_vector_columns_find (columns,
                              c_vector_columns_pos (*primary_keys, i),
                              &error);


    if (error == SQLSYNC_FAILED)
      /*FIXME: fix that problem, 0 should not be added to primary_key_pos */
      pos = -1;

#ifdef DEBUG
    sqlsync_debug_details ("Primary KEY name %s\n",
                           ((c_string_elem_ptr) c_vector_data_pos (*primary_keys, i))->string);
#endif
    c_int32_elem p = { c_int32_elem_type, pos };
    c_vector_int32_add_last_copy (*primary_keys_pos, &p);
  }

  return SQLSYNC_SUCCESS;
}



void
sqlsync_retrieve_valid_tables_columns (sqlsync_sqlserver * server_src, sqlsync_sqlserver * server_dst,
                                       c_vector_strings_ptr tables_input,
                                       c_grid_columns_ptr tables_columns_input,
                                       sqlsync_tables_columns * valid_src,
                                       sqlsync_tables_columns * valid_dst)
{

  uint32 i = 0;
  uint8 tables_destroy = 0;
  uint8 columns_destroy = 0;
  uint32 nr_tables = 0;

  valid_src->valid_tables_columns = c_grid_columns_init (DEFAULT_DATA_SIZE);
  valid_dst->valid_tables_columns = c_grid_columns_init (DEFAULT_DATA_SIZE);


  if (tables_input == NULL)
  {
    tables_destroy = 1;

      tables_input = sqlsync_sqlserver_get_tables (server_src);
#ifdef DEBUG
    sqlsync_debug ("No tables were passed so %ld tables were retrieved\n", nr_tables);
#endif

    sqlsync_send_info_notification2 (_tr ("All available tables will be synchronized\n"
                                          "User didn't specify list of tables to synchronize"));
  }

  /* Get numer of tables */
  nr_tables = c_vector_strings_size (tables_input);

  sqlsync_send_info_notification2 (_tr ("Checking availability of specified tables"));

  c_vector_strings_ptr ctables = sqlsync_sqlserver_get_common_tables(server_src, 
                                                      tables_input);

  valid_dst->valid_tables = sqlsync_sqlserver_get_common_tables(server_dst, ctables);

  valid_src->valid_tables = sqlsync_sqlserver_get_common_tables(server_src, valid_dst->valid_tables);
  c_vector_strings_destroy(ctables);


  if (tables_columns_input == NULL)
  {
    columns_destroy = 1;
    nr_tables = c_vector_strings_size(valid_src->valid_tables);
    tables_columns_input = c_grid_columns_init (DEFAULT_DATA_SIZE);

    for (i = 0; i < nr_tables; i++)
    {
      c_vector_columns_ptr elem_table_columns = NULL;
      c_string_elem_ptr vtable_src = c_vector_strings_pos (valid_src->valid_tables, i);
      c_string_elem_ptr vtable_dst = c_vector_strings_pos (valid_dst->valid_tables, i);

      if (vtable_src == NULL || vtable_dst == NULL)
        continue;

      elem_table_columns = sqlsync_get_common_columns(server_src, server_dst, vtable_src, vtable_dst);

      if (c_vector_columns_size (elem_table_columns) == 0)
      {
        c_string_elem_ptr info_msg = c_string_elem_empty ();

        c_string_elem_append_cstring (info_msg, _tr ("Passed columns for "));
        c_string_elem_append_string (info_msg, vtable_src);
        c_string_elem_append_cstring (info_msg, _tr (" table don't exist"));
        sqlsync_send_info_notification (info_msg);
        c_string_elem_free (info_msg);

        c_vector_columns_destroy (elem_table_columns);
        c_vector_strings_delete (valid_src->valid_tables, i);
        c_vector_strings_delete (valid_dst->valid_tables, i);
        continue;
      }

      (void) c_grid_columns_add (tables_columns_input, elem_table_columns, i);
    }
  }


  /* For all tables - compare all rows */
  for (i = 0; i < nr_tables; i++)
  {
    c_vector_columns_ptr elem_table_columns = NULL;

    c_string_elem_ptr vtable_src = c_vector_strings_pos (valid_src->valid_tables, i);
    c_string_elem_ptr vtable_dst = c_vector_strings_pos (valid_dst->valid_tables, i);

    if (vtable_src == NULL || vtable_dst == NULL)
      continue;

    /* Get list of columns for specified table */
    elem_table_columns = c_grid_columns_pos (tables_columns_input, i);


#ifdef DEBUG
    sqlsync_debug ("Nr of columns %ld\n", c_vector_data_size (elem_table_columns));
#endif

    if (c_vector_columns_size (elem_table_columns) == 0)
    {
      elem_table_columns = sqlsync_get_common_columns(server_src,server_dst, vtable_src, vtable_dst);
      c_vector_columns_ptr cols_src = sqlsync_get_valid_columns(server_src, vtable_src, elem_table_columns);
      c_vector_columns_ptr cols_dst = sqlsync_get_valid_columns(server_dst, vtable_dst, elem_table_columns);
      c_vector_columns_destroy(elem_table_columns);

      sqlsync_add_primary_columns (server_dst, vtable_dst, cols_dst);
      sqlsync_add_primary_columns (server_src, vtable_src, cols_src);


      if (c_vector_columns_size (cols_src) != 0 && c_vector_columns_size (cols_dst) != 0)
      {
          sqlsync_add_primary_columns (server_src, vtable_src, cols_src);
          sqlsync_add_primary_columns (server_dst, vtable_dst, cols_dst);      
      }
      else
      {
        c_string_elem_ptr info_msg = c_string_elem_empty ();

        c_string_elem_append_cstring (info_msg, _tr ("Passed columns for "));
        c_string_elem_append_string (info_msg, vtable_src);
        c_string_elem_append_cstring (info_msg, _tr (" table don't exist"));
        sqlsync_send_info_notification (info_msg);
        c_string_elem_free (info_msg);

        c_vector_strings_delete (valid_src->valid_tables, i);
        c_vector_strings_delete (valid_dst->valid_tables, i);

        if (cols_src != NULL)
          c_vector_columns_destroy( cols_src);
        if (cols_dst != NULL)
          c_vector_columns_destroy( cols_dst);

        continue;
      }

      (void) c_grid_columns_add (valid_src->valid_tables_columns, cols_src, i);
      (void) c_grid_columns_add (valid_dst->valid_tables_columns, cols_dst, i);

#ifdef DEBUG
      sqlsync_debug_details ("Added column to valid_tables_columns\n");
#endif

      elem_table_columns = c_grid_columns_pos (valid_src->valid_tables_columns, i);
    }

    else
    {
      c_vector_columns_ptr vcolumns_src = sqlsync_get_valid_columns(server_src, vtable_src, elem_table_columns);
      c_vector_columns_ptr vcolumns_dst = sqlsync_get_valid_columns(server_dst, vtable_dst, elem_table_columns);
      

      if (c_vector_columns_size (vcolumns_src) != 0 && c_vector_columns_size (vcolumns_dst) != 0)
      {
          sqlsync_add_primary_columns (server_src, vtable_src, vcolumns_src);
          sqlsync_add_primary_columns (server_dst, vtable_dst, vcolumns_dst);      
      }
      else
      {
        c_string_elem_ptr info_msg = c_string_elem_empty ();

        c_string_elem_append_cstring (info_msg, _tr ("Passed columns for "));
        c_string_elem_append_string (info_msg, vtable_src);
        c_string_elem_append_cstring (info_msg, _tr (" table don't exist"));
        sqlsync_send_info_notification (info_msg);
        c_string_elem_free (info_msg);

        c_vector_columns_destroy (vcolumns_src);
        c_vector_columns_destroy (vcolumns_dst);
        c_vector_strings_delete (valid_src->valid_tables, i);
        c_vector_strings_delete (valid_dst->valid_tables, i);
        continue;
      }

      (void) c_grid_columns_add (valid_src->valid_tables_columns, vcolumns_src, i);
      (void) c_grid_columns_add (valid_dst->valid_tables_columns, vcolumns_dst, i);
    }
  }


  if (columns_destroy)
  {
#ifdef DEBUG
    sqlsync_debug ("Destroying tables_columns_input\n");
#endif

    (void) c_grid_columns_destroy (tables_columns_input);
  }

  if (tables_destroy)
  {
#ifdef DEBUG
    sqlsync_debug ("Destroying tables_input\n");
#endif
    (void) c_vector_strings_destroy (tables_input);
  }

}


