/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#ifndef SQLSYNC_DBSTRUCT_H
#define SQLSYNC_DBSTRUCT_H

#include "data_types/sqlsync_data_misc.h"
#include "data_types/sqlsync_data_grid.h"

#include "server/sqlsync_sqlserver.h"
#include "server/sqlsync_sqlserver_data_types.h"

/**
 * Function returns valid columns from table
 * @param info_ptr Pointer to main structure
 * @param nr_server Which server (SRC or DST)
 * @param table From which table column info should be received
 * @param columns List of columns to be checked
 * 
 * @return c_columns_ptr Pointer to structure which contains list of
 *         valid columns
 */
c_vector_columns_ptr sqlsync_get_valid_columns (sqlsync_sqlserver * server,
                                                    c_string_elem_ptr table,
                                                    c_vector_columns_ptr columns);



/**
 * Function returns row from table
 * @param info_ptr Pointer to main structure
 * @param nr_server Which server (SRC or DST)
 * @param columns Pointer to structure which contains list of
 *                columns (elements in c_data must be
 *                c_column_data, look at see section)
 * @see c_column_elem, sqlsync_get_column_info
 * @return c_data Pointer to structure which holds returned row,
 *         if error occured NULL is returned
 */

c_vector_data_ptr sqlsync_get_row (sqlsync_sqlserver * server, c_vector_columns_ptr columns);

void sqlsync_get_rows (sqlsync_sqlserver * server_src, sqlsync_sqlserver * server_dst,
                       c_vector_columns_ptr src_columns, c_vector_columns_ptr dst_columns, 
                       c_grid_data_ptr grid_src, c_grid_data_ptr grid_dst, 
                       c_vector_int32_ptr primary_keys_pos, uint32 * equal_rows);


SQLSYNC_RETURN
sqlsync_find_set_primary_keys (sqlsync_sqlserver * server_src, sqlsync_sqlserver * server_dst, 
                               c_string_elem_ptr table, c_vector_columns_ptr columns, 
                               c_vector_columns_ptr *primary_keys, c_vector_int32_ptr *primary_keys_pos);

/*Adding primary columns which are not added to columns list*/
void
sqlsync_add_primary_columns (sqlsync_sqlserver * server, c_string_elem_ptr table,
                             c_vector_columns_ptr columns);



typedef struct _sqlsync_table_columns
{
  c_string_elem_ptr table;
  c_vector_columns_ptr columns;
} sqlsync_table_columns;

typedef struct _sqlsync_tables_columns
{
  c_vector_strings_ptr valid_tables;
  c_grid_columns_ptr valid_tables_columns;
} sqlsync_tables_columns;

void
sqlsync_retrieve_valid_tables_columns (sqlsync_sqlserver * server_src, sqlsync_sqlserver * server_dst,
                                       c_vector_strings_ptr tables_input,
                                       c_grid_columns_ptr tables_columns_input,
                                       sqlsync_tables_columns * valid_src,
                                       sqlsync_tables_columns * valid_dst);

#endif /* SQLSYNC_DBSTRUCT_H */


