/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/


#include "sqlsync_compare_records.h"

#include "sqlsync_translation.h"

#ifdef DEBUG
#include "sqlsync_debug.h"
#endif

#include "data_types/sqlsync_data_binary.h"
#include "data_types/sqlsync_data_numeric.h"


#include "sqlsync_default_impl.h"
#include "server/sqlsync_sqlserver_priv.h"


c_vector_columns_ptr
sqlsync_sqlserver_default_old_get_primary_keys (sqlsync_sqlserver * server, c_string_elem_ptr table)
{
  c_string_elem_ptr column = NULL;
  SQLSYNC_RETURN result;
  SQLHSTMT hstmt = NULL;

  column = c_string_elem_empty();
  c_string_elem_append_reserve(column, 256);


  c_vector_columns_ptr primary_keys = c_vector_columns_init (1);

  result = SQLAllocHandle (SQL_HANDLE_STMT, server->hdbc, &hstmt);

  result = SQLPrimaryKeys (hstmt, NULL, 0, NULL, 0, (SQLCHAR *) table->string, table->length);
  result = SQLBindCol (hstmt, 4, SQL_C_CHAR, column->string, (SQLINTEGER) 256, (SQLLEN*)(&column->length));

  while (SQL_SUCCEEDED (result = SQLFetch (hstmt)))
  {
    c_column_elem_ptr column_data = NULL;
    column_data = c_column_elem_create (column, 0, 0, 0);

#ifdef DEBUG
    sqlsync_debug ("Primary key column %s for %s\n", column->string, table->string);
#endif
    c_vector_columns_add_last (primary_keys, column_data);

  }
  c_string_elem_free(column);

  (void) SQLFreeStmt (hstmt, SQL_CLOSE);
  return primary_keys;
}


c_vector_columns_ptr
sqlsync_sqlserver_default_get_primary_keys (sqlsync_sqlserver * server,
                          c_string_elem_ptr table)
{
  SQLSYNC_RETURN result;
  SQLHSTMT hstmt = NULL;

  c_vector_columns_ptr primary_keys = c_vector_columns_init (1);

  result = SQLAllocHandle (SQL_HANDLE_STMT, server->hdbc, &hstmt);

  result = SQLPrimaryKeys (hstmt, NULL, 0, NULL, 0, (SQLCHAR *) table->string, table->length);

#ifdef DEBUG
    sqlsync_printf (SQLSYNC_DEBUG, "Primary key columns for table %s\n", table->string);
#endif


  while (SQL_SUCCEEDED (result = SQLFetch (hstmt)))
  {
    c_data_elem_ptr name = NULL;

    sqlsync_get_data (hstmt, SQLColumns_COLUMN_NAME, SQLSYNC_SQLSERVER_DATA_CHAR, 128, &name);

    c_column_elem_ptr column_data = c_column_elem_create ((c_string_elem_ptr)name, 0, 0, 0);

#ifdef DEBUG
    sqlsync_printf (SQLSYNC_DEBUG, "Primary key column ", 0);
#endif

    c_string_elem_free ((c_string_elem_ptr)name);

    c_vector_columns_add_last (primary_keys, column_data);

  }

  (void) SQLFreeStmt (hstmt, SQL_CLOSE);
  return primary_keys;
}


void
sqlsync_create_stmt (sqlsync_sqlserver * server)
{
  (void) SQLFreeStmt (server->hstmt, SQL_CLOSE);
  (void) SQLAllocHandle (SQL_HANDLE_STMT, server->hdbc, &server->hstmt);
}


SQLSYNC_RETURN
sqlsync_sqlserver_default_get_data (void *hstmt_ptr, uint8 nr_column, sqlsync_sqlserver_data_type data_type, c_data_elem_ptr * _elem)
{
  void *data = NULL;
  uint32 data_length = 0;
  SQLINTEGER length = 0;
  SQLSMALLINT type = SQL_C_CHAR;
  SQLRETURN ret = 0;
  SQLCHAR test[2];
  SQLHSTMT hstmt = (SQLHSTMT) hstmt_ptr;
  *_elem = NULL;

  type = sqlsync_sqlserver_map_to_odbc(data_type);

  if (type == SQL_C_CHAR)
  {
      ret = SQLGetData (hstmt, nr_column, type, &test, 0, &length);
      data_length = length + 1;
      if (SQLSYNC_IS_ERROR(ret)) 
      {
        return SQLSYNC_FAILED;
      }
      if (length == 0)
      {
        *_elem = (c_data_elem_ptr)c_string_elem_empty();
        return SQLSYNC_SUCCESS;
      }
  }

  sqlsync_sqlserver_type_size(data_type, &data_length);

  c_data_elem_ptr elem = sqlsync_sqlserver_create_data(data_type, data_length);

  if (elem == NULL)
    return SQLSYNC_FAILED_NULL;

  data = c_data_elem_data(elem);
  data_length = c_data_elem_size(elem);

  if(type == SQL_C_BIT) 
  {
      type = SQL_C_BINARY;
  }

  do
  {
    length = 0;
    ret = SQLGetData (hstmt, nr_column, type, data, data_length, &length);

    if (ret == SQL_NULL_DATA || ret == SQL_NO_DATA)
      return SQLSYNC_FAILED_NULL;

    if (SQLSYNC_IS_ERROR (ret))
    {
      /* Print some error info or .. */

      return SQLSYNC_FAILED;
    }

    if ((length - (int32) data_length) > 0)
    {
      if (elem->_type == c_binary_elem_type || elem->_type == c_bit_elem_type)
      {
#ifdef DEBUG
        sqlsync_debug ("retrieved binary %d\n", length);
#endif
        c_binary_elem_ptr binary_elem = (c_binary_elem_ptr) elem;

        binary_elem->length = data_length;

        if (c_binary_elem_append_reserve (binary_elem, length - data_length) !=
            SQLSYNC_SUCCESS)
          return SQLSYNC_FAILED;

        data = (char *) binary_elem->data + data_length ;
        data_length = length - data_length;
      }
    }
    else
    {
      if (elem->_type == c_binary_elem_type || elem->_type == c_bit_elem_type)
      {
#ifdef DEBUG
        sqlsync_debug ("retrieved binary %d\n", length);
#endif
        c_binary_elem_ptr binary_elem = (c_binary_elem_ptr) elem;

#ifdef DEBUG
        sqlsync_debug ("retrieved binary length %d\n", binary_elem->length);
#endif

        binary_elem->length += length;
        data_length = data_length + length;
      }
    }

    if ((length - (int32) data_length) >= 0)
    {
      if (elem->_type == c_string_elem_type)
      {
#ifdef DEBUG
        sqlsync_debug ("retrieved string %d\n", length);
#endif
        c_string_elem_ptr string_elem = (c_string_elem_ptr) elem;

        string_elem->length = data_length - 1;
        if (c_string_elem_append_reserve (string_elem, length + 2 - data_length) !=
            SQLSYNC_SUCCESS)
          return SQLSYNC_FAILED;

        data = (char *) string_elem->string + data_length - 1;
        data_length = length - data_length + 2;
      }
    }
    else
    {
      if (length == SQL_NULL_DATA)
        return SQLSYNC_FAILED_NULL;

      if (elem->_type == c_string_elem_type)
      {

#ifdef DEBUG
        sqlsync_debug ("retrieved string %d\n", length);
#endif
        c_string_elem_ptr string_elem = (c_string_elem_ptr) elem;

        string_elem->length += length;
        data_length = data_length + length;
      }
    }

  }
  while (length != SQL_NULL_DATA && (uint32) length > data_length);

 *_elem = elem;
  return SQLSYNC_SUCCESS;
}


int8 sqlsync_check_icmp_table(c_data_elem_ptr src, c_data_elem_ptr dst)
{
  return c_string_elem_icmp((c_string_elem_ptr)src, (c_string_elem_ptr)dst);
}


SQLSYNC_RETURN
sqlsync_sqlserver_default_check_table (sqlsync_sqlserver * server, c_string_elem_ptr table)
{
  SQLSYNC_RETURN ret;

  if (server == NULL || table == NULL)
    return SQLSYNC_FAILED;

  c_vector_strings_ptr tables = sqlsync_sqlserver_default_get_tables(server);
  if (sqlsync_sqlserver_cs_tables_columns(server) == SQLSYNC_TRUE)
    c_vector_data_find_wf((c_vector_data_ptr)tables, (c_data_elem_ptr)table, sqlsync_check_icmp_table, &ret);
  else 
    c_vector_data_find((c_vector_data_ptr)tables, (c_data_elem_ptr)table, &ret);
  c_vector_columns_destroy(tables);
  return ret;
}

c_vector_strings_ptr
sqlsync_sqlserver_get_common_tables (sqlsync_sqlserver * server, c_vector_strings_ptr tables)
{
  SQLSYNC_RETURN ret;
  uint32 i = 0;
  uint32 pos = 0;
  c_string_elem_ptr table = NULL;
  c_vector_strings_ptr ctables = c_vector_strings_init(10);
  c_vector_strings_ptr ltables = NULL;
  if (server == NULL || tables == NULL)
    return SQLSYNC_FAILED;

  ltables = sqlsync_sqlserver_get_tables(server);

  for(i = 0; i< c_vector_strings_size(tables); i++)
  {
    table = c_vector_strings_pos(tables, i);

    if (sqlsync_sqlserver_cs_tables_columns(server) == SQLSYNC_TRUE)
      pos = c_vector_data_find_wf((c_vector_data_ptr)ltables, (c_data_elem_ptr)table, sqlsync_check_icmp_table, &ret);
    else 
      pos = c_vector_data_find((c_vector_data_ptr)ltables, (c_data_elem_ptr)table, &ret);

    if (ret == SQLSYNC_TRUE)
      (void)c_vector_strings_add_last_copy(ctables, c_vector_strings_pos(ltables,pos));
  }

  c_vector_columns_destroy(ltables);
  return ctables;
}


c_vector_strings_ptr
sqlsync_sqlserver_default_get_tables (sqlsync_sqlserver * server)
{
  SQLRETURN ret;
  c_vector_strings_ptr tables = NULL;

  if (server == NULL)
    return NULL;

  sqlsync_create_stmt (server);

  ret = SQLTables (server->hstmt, NULL, 0, NULL, 0, (SQLCHAR *) "%", SQL_NTS, (SQLCHAR *) "TABLE", SQL_NTS);


  if (SQLSYNC_IS_ERROR (ret))
  {
#ifdef DEBUG
    sqlsync_debug ("sqlsync_get_tables failed\n");
#endif

    server->error = SQLSYNC_FAILED_GET_TABLES;
    server->last_bad_handle = SQL_HANDLE_STMT;
    return NULL;
  }

  tables = c_vector_strings_init (DEFAULT_DATA_SIZE);

  while (SQL_SUCCEEDED (ret = SQLFetch (server->hstmt)))
  {
    c_data_elem_ptr name = NULL;

    sqlsync_get_data (server->hstmt, SQLTables_TABLE_NAME, SQLSYNC_SQLSERVER_DATA_CHAR, 128, &name);

    if (sqlsync_sqlserver_is_system_table (server, (c_string_elem_ptr) name) == SQLSYNC_FALSE)
    {
#ifdef DEBUG
      sqlsync_debug ("This is not a system table %s\n", ((c_string_elem_ptr)name)->string);
#endif
      (void) c_vector_strings_add_last (tables, (c_string_elem_ptr)name);
    }
    else
    {
      c_string_elem_free ((c_string_elem_ptr)name);
    }
  }

  return tables;
}

c_vector_columns_ptr
sqlsync_sqlserver_default_get_columns (sqlsync_sqlserver * server, c_string_elem_ptr table)
{
  SQLRETURN ret;
  c_string_elem_ptr name = NULL;
  c_vector_columns_ptr columns;
 
  c_column_elem_ptr column_data = NULL;

  sqlsync_create_stmt (server);

  ret =
    SQLColumns (server->hstmt, NULL, 0, NULL, 0, (SQLCHAR *) table->string, (SQLSMALLINT) table->length,
                NULL, 0);

  if (SQLSYNC_IS_ERROR (ret))
  {
#ifdef DEBUG
    sqlsync_debug ("sqlsync_get_columns failed %d\n", ret);
#endif

    server->last_bad_handle = SQL_HANDLE_STMT;
    return NULL;
  }


  columns = c_vector_columns_init (DEFAULT_DATA_SIZE);

#ifdef DEBUG
  sqlsync_debug ("Retrieving columns\n");
#endif

  while (SQL_SUCCEEDED (ret = SQLFetch (server->hstmt)))
  {
    column_data = NULL;
    {
      c_data_elem_ptr tmp = NULL;
      c_int32_elem_ptr max_size = NULL;
      c_string_elem_ptr data_type = NULL;

      sqlsync_get_data (server->hstmt, SQLColumns_COLUMN_NAME, SQLSYNC_SQLSERVER_DATA_CHAR, 128, &tmp);
      name =  (c_string_elem_ptr)tmp;
      sqlsync_get_data (server->hstmt, SQLColumns_COLUMN_SIZE, SQLSYNC_SQLSERVER_DATA_UINTEGER, 4, &tmp);
      max_size = (c_int32_elem_ptr) tmp;
      
      if (sqlsync_sqlserver_type(server) == sqlsync_server_mssql || sqlsync_sqlserver_type(server) == sqlsync_server_firebird)
      {
          c_int16_elem_ptr odbc_data_type = NULL;
          int8 use_operator = 0;
          sqlsync_get_data (server->hstmt, SQLColumns_SQL_DATA_TYPE, SQLSYNC_SQLSERVER_DATA_SMALLINT, 2, &tmp);
          odbc_data_type = (c_int16_elem_ptr)tmp;
					sqlsync_sqlserver_data_type t = sqlsync_sqlserver_recognize_data_type_odbc(server,odbc_data_type->value, &use_operator);

          column_data = c_column_elem_create (name, max_size->value, t, use_operator);
          c_int16_elem_free(odbc_data_type);
      }
      else
      {
        int8 use_operator = 0;
        sqlsync_get_data (server->hstmt, SQLColumns_TYPE_NAME, SQLSYNC_SQLSERVER_DATA_CHAR, 128, &tmp);
        data_type = (c_string_elem_ptr) tmp;
				sqlsync_sqlserver_data_type t = sqlsync_sqlserver_recognize_data_type(server,data_type, &use_operator);
        
        column_data = c_column_elem_create (name, max_size->value, t, use_operator);
      }

      c_string_elem_free (name);
      c_int32_elem_free (max_size);
      c_string_elem_free (data_type);
    }
    if (column_data == NULL)
      continue;

#ifdef DEBUG
    sqlsync_debug ("Adding column\n");
#endif

    (void) c_vector_columns_add_last (columns, column_data);
  }
  return columns;
}
