/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#include <stdio.h>


#include "server/sqlsync_sqlserver.h"
#include "server/sqlsync_sqlserver_priv.h"
#include "server/sqlsync_sqlserver_helpers.h"

#include "sqlsync_query.h"

#ifdef DEBUG
#include "sqlsync_debug.h"
#endif


SQLSYNC_RETURN
sqlsync_exec_query (sqlsync_sqlserver * server,
                    c_string_elem_ptr query)
{
  SQLRETURN ret;

  if (server == NULL || query == NULL || query->length == 0)
    return SQLSYNC_FAILED_NULL;

  (void) SQLFreeStmt (server->hstmt_query, SQL_CLOSE);
  (void) SQLAllocHandle (SQL_HANDLE_STMT, server->hdbc, &server->hstmt_query);

#ifdef DEBUG
  sqlsync_debug ("Running query\n", 0);
  sqlsync_debug ("Running query len %d\n", query->length);
  sqlsync_debug_println (query);
#endif

  ret = SQLExecDirect (server->hstmt_query, (SQLCHAR *) query->string, query->length);
  if (SQLSYNC_IS_ERROR (ret) && ret != SQL_NO_DATA)
  {
#ifdef DEBUG
    sqlsync_debug ("Error while running query %s\n", query->string);
    sqlsync_debug ("Error number %d\n", ret);
#endif

    server->error = SQLSYNC_FAILED_QUERY;
    server->last_bad_handle = SQL_HANDLE_STMT;
    return SQLSYNC_FAILED_QUERY;
  }
  return SQLSYNC_SUCCESS;
}


SQLSYNC_RETURN
sqlsync_execute_query_file (sqlsync_sqlserver * server, c_string_elem_ptr name)
{
  FILE *f;
  c_string_elem_ptr buff = NULL;
  int ret = 0;
  uint32 len = 0;

  f = fopen (name->string, "r");
  if (f == NULL)
  {
     server->last_bad_handle = 0;
     server->error = 0;
     return SQLSYNC_FAILED_NULL;
  }

  buff = c_string_elem_reserve (2000);

  while (fgets (buff->string + len, buff->reserved_length - len, f) != NULL)
  {
    uint32 skipEOL = 0;
    uint32 read_len = std_strlen (buff->string + len);

    buff->length += read_len;
    len += read_len;

    if (len == buff->reserved_length - 1)
    {
        c_string_elem_append_reserve (buff, 2000);
    }
    
    if (buff->string[len-1] == '\n')
      skipEOL++;
    if (buff->string[len-1 - skipEOL] == '\r')
      skipEOL++;

    if ((len > 2) && (buff->string[len -1 - skipEOL]) == ';')
    {
      ret = sqlsync_exec_query (server, buff);

      if (ret != SQLSYNC_SUCCESS)
      {
        server->error = SQLSYNC_FAILED_QUERY;
        server->last_bad_handle = SQL_HANDLE_STMT;
        sqlsync_sqlserver_print_error (server);
        break;
      }
      else
      {
          c_string_elem_free (buff);
          buff = c_string_elem_reserve (2000);
      }
      len = 0;
    }
  }
  c_string_elem_free (buff);
  return ret;
}
