/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#include "sqlsync_translation.h"
#include "sqlsync_user_notification.h"
#include "sqlsync_user_internal.h"

#include "server/sqlsync_sqlserver.h"
#include "server/sqlsync_sqlserver_priv.h"

#include "server/sqlsync_default_impl.h"

#include "server/oracle/sqlsync_sqlserver_oracle.h"
#include "server/mysql/sqlsync_sqlserver_mysql.h"
#include "server/mssql/sqlsync_sqlserver_mssql.h"
#include "server/firebird/sqlsync_sqlserver_firebird.h"
#include "server/postgresql/sqlsync_sqlserver_postgresql.h"

#ifdef DEBUG
#include "sqlsync_debug.h"
#endif

SQLSYNC_RETURN sqlsync_sqlserver_initialize(sqlsync_sqlserver * server)
{
    SQLRETURN ret = 0;

  /* 1. allocate Environment handle and register version */
  ret = SQLAllocHandle (SQL_HANDLE_ENV, (SQLHANDLE) SQL_NULL_HANDLE, &server->env);

  if (SQLSYNC_IS_ERROR (ret))
  {
#ifdef DEBUG
    sqlsync_debug ("SQLAllocHandle returned error\n");
#endif

    sqlsync_send_error_notification2 (_tr ("Internal error in sqlsync_init"));

    return SQLSYNC_FAILED_INIT;
  }

  ret = SQLSetEnvAttr (server->env, SQL_ATTR_ODBC_VERSION, (void *) SQL_OV_ODBC3, 0);
  if (SQLSYNC_IS_ERROR (ret))
  {
#ifdef DEBUG
    sqlsync_debug ("SQLSetEnvAttr returned error\n");
#endif
    server->last_bad_handle = SQL_HANDLE_ENV;

#ifdef DEBUG
    sqlsync_debug ("SQLSetEnvAttr returned error\n");
#endif
    server->last_bad_handle = SQL_HANDLE_ENV;

    sqlsync_send_error_notification2 (_tr ("Internal error in sqlsync_init"));

    return SQLSYNC_FAILED_INIT;
  }

  ret = SQLSetEnvAttr (server->env, SQL_ATTR_ODBC_VERSION, (void *) SQL_OV_ODBC3, 0);
  if (SQLSYNC_IS_ERROR (ret))
  {
#ifdef DEBUG
    sqlsync_debug ("SQLSetEnvAttr returned error\n");
#endif
    server->last_bad_handle = SQL_HANDLE_ENV;

    sqlsync_send_error_notification2 (_tr ("Internal error in sqlsync_init"));

    return SQLSYNC_FAILED_INIT;
  }

    /* 2. allocate connection handle, set timeout */
    ret = SQLAllocHandle (SQL_HANDLE_DBC, server->env, &server->hdbc);
    if (SQLSYNC_IS_ERROR (ret))
    {
  #ifdef DEBUG
      sqlsync_debug ("SQLAllocHandle returned error %d\n", ret);
  #endif

      sqlsync_send_error_notification2 (_tr ("Internal error in sqlsync_init"));

      return SQLSYNC_FAILED_INIT;
    }

    /* 2. allocate connection handle, set timeout */
    ret = SQLAllocHandle (SQL_HANDLE_DBC, server->env, &server->hdbc);
    if (SQLSYNC_IS_ERROR (ret))
    {
  #ifdef DEBUG
      sqlsync_debug ("SQLAllocHandle returned error %d\n", ret);
  #endif
 
      sqlsync_send_error_notification2 (_tr ("Internal error in sqlsync_init"));

      return  SQLSYNC_FAILED_INIT;
    }

    server->hstmt= NULL;

    server->hstmt_query = NULL;
    server->impl = NULL;
  return SQLSYNC_SUCCESS;
}


c_string_elem_ptr
sqlsync_get_dbms_name (sqlsync_sqlserver * server)
{
  SQLRETURN ret = SQLSYNC_FAILED;

#define DBMS_BUFFER_SIZE 255
  SQLCHAR buffer[DBMS_BUFFER_SIZE];
  SQLSMALLINT outlen = 0;

  c_string_elem_ptr dbms = NULL;


  ret = SQLGetInfo (server->hdbc, SQL_DBMS_NAME, buffer, DBMS_BUFFER_SIZE, &outlen);

  if (SQLSYNC_IS_ERROR (ret))
  {
#ifdef DEBUG
    sqlsync_debug ("Unable to return DBMS name\n");
#endif
    return NULL;
  }

  dbms = c_string_elem_empty ();
  c_string_elem_append (dbms, (char *) buffer, outlen);

  return dbms;
}

sqlsync_server_type sqlsync_sqlserver_determine_type(sqlsync_sqlserver * server)
{
    c_string_elem_ptr db = NULL;
    sqlsync_server_type type;

    db = sqlsync_get_dbms_name (server);
    if (db == NULL)
      return SQLSYNC_FAILED_NULL;

  #ifdef DEBUG
    sqlsync_debug ("Server type [%ld] is [", server->type);
    sqlsync_prints (SQLSYNC_DEBUG, db);
    sqlsync_printf (SQLSYNC_DEBUG, "]\n", 0);

  #endif

    if (std_strncmp (db->string, "MySQL", 5) == 0)
      type = sqlsync_server_mysql;
    else if (std_strncmp (db->string, "PostgreSQL", 10) == 0)
      type = sqlsync_server_postgres;
    else if (std_strncmp (db->string, "Microsoft SQL Server", 20) == 0)
      type = sqlsync_server_mssql;
    else if (std_strncmp (db->string, "Oracle", 6) == 0)
      type = sqlsync_server_oracle;
    else if (std_strncmp (db->string, "Firebird", 8) == 0)
      type = sqlsync_server_firebird;
    else
      type = sqlsync_server_unknown;

    c_string_elem_free (db);

    return type;
}


SQLSYNC_RETURN sqlsync_sqlserver_connect(sqlsync_sqlserver * server, c_string_elem_ptr dsn, 
                                        c_string_elem_ptr user, c_string_elem_ptr password)
{
    SQLSMALLINT dLen = dsn->length;
    SQLSMALLINT uLen = user->length;
    SQLSMALLINT pLen = password->length;

    SQLRETURN ret = SQLConnect (server->hdbc, (SQLCHAR *) dsn->string, dLen,
                    (SQLCHAR *) user->string, uLen,
                    (SQLCHAR *) password->string, pLen);

  if (SQLSYNC_IS_ERROR (ret))
  {
    server->last_bad_handle = SQL_HANDLE_DBC;
    return SQLSYNC_FAILED_CONNECT;
  }

  ret = SQLAllocHandle (SQL_HANDLE_STMT, server->hdbc, &server->hstmt);

  ret |= SQLAllocHandle (SQL_HANDLE_STMT, server->hdbc, &server->hstmt_query);


  if (SQLSYNC_IS_ERROR (ret))
  {
    server->last_bad_handle = SQL_HANDLE_DBC;

    sqlsync_send_string_notification (SQLSYNC_USER_CONNECTION_FAILED, dsn);

    return SQLSYNC_FAILED_CONNECT;
  }
  return SQLSYNC_SUCCESS;
}

sqlsync_sqlserver * sqlsync_sqlserver_create_dummy(sqlsync_server_type type)
{

    sqlsync_sqlserver * tmp =(sqlsync_sqlserver *) std_malloc(sizeof(sqlsync_sqlserver));
    tmp->type = type;
    tmp->env = NULL;
    tmp->hdbc = NULL;
    tmp->hstmt = NULL;
    tmp->hstmt_query = NULL;
    tmp->impl = NULL;

    switch(tmp->type) 
    {
      case sqlsync_server_oracle:
      {
        tmp->impl = sqlsync_sqlserver_oracle();
        break;
      }
      case sqlsync_server_mssql:
      {
        tmp->impl = sqlsync_sqlserver_mssql();
        break;
      }
      case sqlsync_server_firebird:
      {
        tmp->impl = sqlsync_sqlserver_firebird();
        break;
      }
      case sqlsync_server_mysql:
      {
        tmp->impl = sqlsync_sqlserver_mysql();
        break;
      }
      case sqlsync_server_postgres:
      {
        tmp->impl = sqlsync_sqlserver_postgresql();
        break;
      }
      default:
          tmp->impl = NULL;
    }

    if (tmp->impl == NULL)
    {
      sqlsync_sqlserver_destroy(tmp);
      return NULL;
    }

  return tmp;
}


SQLSYNC_RETURN sqlsync_sqlserver_create_connect(sqlsync_sqlserver ** server, c_string_elem_ptr dsn, 
                                        c_string_elem_ptr user, c_string_elem_ptr password, SQLSYNC_RETURN cs_tables_columns)

{
  SQLSYNC_RETURN ret = SQLSYNC_FAILED;
  c_string_elem _user;
  c_string_elem _password;

  sqlsync_sqlserver * tmp = (sqlsync_sqlserver *) std_malloc(sizeof(sqlsync_sqlserver));

  if (dsn == NULL)
    return SQLSYNC_FAILED_NULL;


  /* In case when user or/and password are empty strings 
     use default user and password from configuration
   */

  if (user == NULL || user->length == 0)
  {
    _user.string = NULL;
    _user.length = SQL_NTS;
    user = &_user;
  }

  if (password == NULL || password->length == 0)
  {
    _password.string = NULL;
    _password.length = SQL_NTS;
    password = &_password;
  }


  if (tmp == NULL)
    return SQLSYNC_FAILED_NULL;
  ret = sqlsync_sqlserver_initialize(tmp);
  if (ret != SQLSYNC_SUCCESS) 
  {
    std_free(tmp);
    return ret;
  }
  *server = tmp;
  (*server)->cs_tables_columns = cs_tables_columns;
  ret = sqlsync_sqlserver_connect(tmp,dsn,user,password);
  if (ret != SQLSYNC_SUCCESS)
      return ret;

  tmp->type = sqlsync_sqlserver_determine_type(tmp);

  switch(tmp->type) 
  {
    case sqlsync_server_oracle:
    {
      tmp->impl = sqlsync_sqlserver_oracle();
      break;
    }
    case sqlsync_server_mssql:
    {
      tmp->impl = sqlsync_sqlserver_mssql();
      break;
    }
    case sqlsync_server_firebird:
    {
      tmp->impl = sqlsync_sqlserver_firebird();
      break;
    }
    case sqlsync_server_mysql:
    {
      tmp->impl = sqlsync_sqlserver_mysql();
      break;
    }
    case sqlsync_server_postgres:
    {
      tmp->impl = sqlsync_sqlserver_postgresql();
      break;
    }
    default:
        tmp->impl = NULL;
  }

  if (tmp->impl == NULL)
  {
    sqlsync_sqlserver_destroy(tmp);
    *server = NULL;
    return SQLSYNC_FAILED_NULL;
  }

  *server = tmp;
  return SQLSYNC_SUCCESS;
}

SQLSYNC_RETURN sqlsync_sqlserver_destroy(sqlsync_sqlserver * server)
{
  SQLRETURN ret = 0;

#ifdef DEBUG
  sqlsync_debug_details ("sqlsync_destroy info: hstmt[0] %x\n"
                         "\nhstmt_query[0] %x\n",
                         server->hstmt,
                         server->hstmt_query);
#endif

    if (server->hstmt != NULL)
    {
      (void) SQLCloseCursor (server->hstmt);
      ret = SQLFreeHandle (SQL_HANDLE_STMT, server->hstmt);
    }

    if (server->hstmt_query != NULL)
    {
      (void) SQLCloseCursor (server->hstmt_query);
      ret |= SQLFreeHandle (SQL_HANDLE_STMT, server->hstmt_query);
    }

    if (server->hdbc != NULL)
      (void) SQLDisconnect (server->hdbc);

    if (server->hdbc!= NULL)
      ret |= SQLFreeHandle (SQL_HANDLE_DBC, server->hdbc);


    if (server->env != NULL)
      ret |= SQLFreeHandle (SQL_HANDLE_ENV, server->env);


	std_free(server->impl);
	std_free(server);
  return SQLSYNC_TRUE;
}


c_string_elem_ptr sqlsync_sqlserver_data_to_string(sqlsync_sqlserver * server, c_data_elem_ptr elem, SQLSYNC_RETURN with_escapes)
{
  return server->impl->data_to_string(elem, with_escapes);
}

c_string_elem_ptr sqlsync_sqlserver_column_identifier(sqlsync_sqlserver * server, c_string_elem_ptr elem)
{
  return server->impl->column_identifier(elem);
}

SQLSYNC_RETURN sqlsync_sqlserver_is_system_table(sqlsync_sqlserver * server, c_string_elem_ptr table)
{
  return server->impl->is_system_table(table);
}

SQLSYNC_RETURN sqlsync_sqlserver_does_table_exist(sqlsync_sqlserver * server, c_string_elem_ptr table)
{
  return server->impl->does_table_exist(server, table);
}

c_vector_columns_ptr sqlsync_sqlserver_get_columns(sqlsync_sqlserver * server, c_string_elem_ptr table)
{
    return server->impl->get_columns(server, table);
}

c_vector_columns_ptr sqlsync_sqlserver_get_primary_keys (sqlsync_sqlserver * server, c_string_elem_ptr table)
{
    return server->impl->get_primary_keys(server, table);
}

sqlsync_sqlserver_data_type sqlsync_sqlserver_recognize_data_type(sqlsync_sqlserver * server, c_string_elem_ptr data_type_name, int8 * use_operator)
{
 return server->impl->recognize_data_type(data_type_name, use_operator);
}

sqlsync_sqlserver_data_type sqlsync_sqlserver_recognize_data_type_odbc(sqlsync_sqlserver * server, int16 data_type, int8 * use_operator)
{
 return server->impl->recognize_data_type_odbc(data_type, use_operator);
}

sqlsync_server_type sqlsync_sqlserver_type(sqlsync_sqlserver * server)
{
  return server->type;
}

c_vector_strings_ptr sqlsync_sqlserver_get_tables (sqlsync_sqlserver * server)
{
  return server->impl->get_tables(server);
}

SQLSYNC_RETURN sqlsync_sqlserver_cs_tables_columns(sqlsync_sqlserver * server)
{
  return server->cs_tables_columns;
}
