/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#ifndef SQLSYNC_SQLSERVER_H
#define SQLSYNC_SQLSERVER_H

#include "sqlsync_def.h"
#include "data_types/sqlsync_data_string.h"
#include "data_types/sqlsync_data_vector_misc.h"

#include "server/sqlsync_sqlserver_data_types.h"

typedef struct _sqlsync_sqlserver sqlsync_sqlserver;

SQLSYNC_RETURN sqlsync_sqlserver_create_connect(sqlsync_sqlserver ** server, c_string_elem_ptr dsn, 
                                        c_string_elem_ptr user, c_string_elem_ptr password, SQLSYNC_RETURN cs_tables_columns);

sqlsync_sqlserver * sqlsync_sqlserver_create_dummy(sqlsync_server_type type);

c_string_elem_ptr sqlsync_sqlserver_data_to_string(sqlsync_sqlserver * server, c_data_elem_ptr elem, SQLSYNC_RETURN with_escapes);

c_string_elem_ptr sqlsync_sqlserver_column_identifier(sqlsync_sqlserver * server, c_string_elem_ptr elem);

/**
 * Function checks if selected table is system table
 * @param info Pointer to the main structure
 * @param nr_server Which server (SRC or DST)
 * @param table Table name to be checked
 * 
 * @return SQLSYNC_RETURN Returns SQLSYNC_TRUE  if this is
 *         system table otherwise SQLSYNC_FALSE
 */
SQLSYNC_RETURN sqlsync_sqlserver_is_system_table(sqlsync_sqlserver * server, c_string_elem_ptr table);

SQLSYNC_RETURN sqlsync_sqlserver_does_table_exist(sqlsync_sqlserver * server, c_string_elem_ptr table);


/**
 * Function returns available (not system) tables from database
 * @param info_ptr Pointer to main structure
 * @param nr_server Which server (SRC or DST)
 * 
 * @return c_data Pointer to structure which contains list of
 *         tables
 */
c_vector_strings_ptr sqlsync_sqlserver_get_tables (sqlsync_sqlserver * server);

c_vector_columns_ptr sqlsync_sqlserver_get_columns(sqlsync_sqlserver * server, c_string_elem_ptr table);

c_vector_columns_ptr sqlsync_sqlserver_get_primary_keys (sqlsync_sqlserver * server, c_string_elem_ptr table);

sqlsync_sqlserver_data_type sqlsync_sqlserver_recognize_data_type(sqlsync_sqlserver * server, c_string_elem_ptr data_type_name, int8 * use_operator);

sqlsync_sqlserver_data_type sqlsync_sqlserver_recognize_data_type_odbc(sqlsync_sqlserver * server, int16 data_type, int8 * use_operator);

sqlsync_server_type sqlsync_sqlserver_type(sqlsync_sqlserver * server);

/* Function returns if tables and columns should be compared as a case sensitive */
SQLSYNC_RETURN sqlsync_sqlserver_cs_tables_columns(sqlsync_sqlserver * server);

SQLSYNC_RETURN sqlsync_sqlserver_destroy(sqlsync_sqlserver * server);

#endif /* SQLSYNC_SQLSERVER_H */
