/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#include "data_types/sqlsync_data_binary.h"
#include "data_types/sqlsync_data_numeric.h"
#include "data_types/sqlsync_data_datetime.h"
#include "data_types/sqlsync_data_string.h"
#include "data_types/sqlsync_data_misc.h"

#include "server/sqlsync_sqlserver_data_types.h"


#if (defined(_WINDOWS) || defined(__MINGW32__))
#include <windef.h>
#include <winnt.h>
#endif

#include <sql.h>
#include <sqlext.h>

int16 sqlsync_sqlserver_map_to_odbc(sqlsync_sqlserver_data_type data_type)
{
  switch(data_type)
  {
    case SQLSYNC_SQLSERVER_DATA_UNKNOWN:
      return SQL_CHAR;
 
    case SQLSYNC_SQLSERVER_DATA_CHAR:
      return SQL_CHAR;

    case SQLSYNC_SQLSERVER_DATA_WCHAR:
      return SQL_WCHAR;

    case  SQLSYNC_SQLSERVER_DATA_BIT:
      return SQL_C_BIT;

    case  SQLSYNC_SQLSERVER_DATA_BINARY:
    case SQLSYNC_SQLSERVER_DATA_BLOB:
      return SQL_C_BINARY;

    case SQLSYNC_SQLSERVER_DATA_XML:
      return SQL_C_CHAR;

    case SQLSYNC_SQLSERVER_DATA_CBLOB:
      return SQL_CHAR; /* TODO: check that */

    case SQLSYNC_SQLSERVER_DATA_GUID:
      return SQL_GUID;

    case SQLSYNC_SQLSERVER_DATA_DATETIME:
    case SQLSYNC_SQLSERVER_DATA_DATE:
      return SQL_TYPE_DATE;

    case  SQLSYNC_SQLSERVER_DATA_TIME:
    case  SQLSYNC_SQLSERVER_DATA_TIME_TZ:
      return SQL_TYPE_TIME;
 
    case  SQLSYNC_SQLSERVER_DATA_YEAR:
    case  SQLSYNC_SQLSERVER_DATA_TIMESTAMP:
    case  SQLSYNC_SQLSERVER_DATA_TIMESTAMP_TZ:
      return SQL_TYPE_TIMESTAMP;

/*    case  SQLSYNC_SQLSERVER_DATA_INTERVAL:
      return SQL_TYPE_INTERVAL;
 */
    case  SQLSYNC_SQLSERVER_DATA_UTINYINT:
      return SQL_C_UTINYINT;
 
    case  SQLSYNC_SQLSERVER_DATA_TINYINT:
      return SQL_C_STINYINT;

    case  SQLSYNC_SQLSERVER_DATA_USMALLINT:
      return SQL_C_USHORT;

    case  SQLSYNC_SQLSERVER_DATA_SMALLINT:
      return SQL_C_SSHORT;

    case  SQLSYNC_SQLSERVER_DATA_UMEDIUMINT:
      return SQL_C_ULONG;

    case  SQLSYNC_SQLSERVER_DATA_MEDIUMINT:
      return SQL_C_LONG;

    case  SQLSYNC_SQLSERVER_DATA_UINTEGER:
      return SQL_C_ULONG;

    case  SQLSYNC_SQLSERVER_DATA_INTEGER:
      return SQL_C_SLONG;

    case  SQLSYNC_SQLSERVER_DATA_UBIGINT:
      return SQL_C_UBIGINT;

    case  SQLSYNC_SQLSERVER_DATA_BIGINT:
      return SQL_C_SBIGINT;

    case  SQLSYNC_SQLSERVER_DATA_UNUMERIC:
    case  SQLSYNC_SQLSERVER_DATA_NUMERIC:
      return SQL_NUMERIC;

    case  SQLSYNC_SQLSERVER_DATA_FLOAT:
      return SQL_C_FLOAT;

    case  SQLSYNC_SQLSERVER_DATA_DOUBLE:
      return SQL_C_DOUBLE;

    case  SQLSYNC_SQLSERVER_DATA_MONEY:
      return SQL_C_CHAR;

    case SQLSYNC_SQLSERVER_DATA_BOOL:
      return SQL_C_USHORT;

    default:
      return SQL_CHAR;
  }
}

void sqlsync_sqlserver_type_size(sqlsync_sqlserver_data_type data_type, uint32 * size)
{
  switch(data_type)
  {
    case SQLSYNC_SQLSERVER_DATA_WCHAR:
      if (*size == 0 || *size > 800)
        *size = 800;

      *size *=2;
      break;

    case SQLSYNC_SQLSERVER_DATA_UNKNOWN:
    case SQLSYNC_SQLSERVER_DATA_CHAR:
    
    case SQLSYNC_SQLSERVER_DATA_CBLOB:
    case SQLSYNC_SQLSERVER_DATA_XML:
    case  SQLSYNC_SQLSERVER_DATA_BINARY:
    case  SQLSYNC_SQLSERVER_DATA_BIT:
    case SQLSYNC_SQLSERVER_DATA_BLOB:
        if (*size == 0 || *size > 800)
          *size = 800;
      ;
      break;

    case SQLSYNC_SQLSERVER_DATA_DATETIME:
    case SQLSYNC_SQLSERVER_DATA_DATE:
      *size = sizeof(odbc_date_struct);
      break;

    case  SQLSYNC_SQLSERVER_DATA_TIME:
    case  SQLSYNC_SQLSERVER_DATA_TIME_TZ:
      *size = sizeof(odbc_time_struct);//
      break;

    case  SQLSYNC_SQLSERVER_DATA_YEAR:
    case  SQLSYNC_SQLSERVER_DATA_TIMESTAMP:
    case  SQLSYNC_SQLSERVER_DATA_TIMESTAMP_TZ:
      *size = sizeof(odbc_timestamp_struct);
      break;

    case SQLSYNC_SQLSERVER_DATA_GUID:
      *size = sizeof(odbc_guid_struct);
      break;

 /*    case  SQLSYNC_SQLSERVER_DATA_INTERVAL:
      return SQL_TYPE_INTERVAL;
 */
    case  SQLSYNC_SQLSERVER_DATA_UTINYINT:
      *size = 1;
      break;

    case  SQLSYNC_SQLSERVER_DATA_TINYINT:
      *size = 1;
      break;

    case  SQLSYNC_SQLSERVER_DATA_BOOL:
    case  SQLSYNC_SQLSERVER_DATA_USMALLINT:
      *size = 2;
      break;

    case  SQLSYNC_SQLSERVER_DATA_SMALLINT:
      *size = 2;
      break;

    case  SQLSYNC_SQLSERVER_DATA_UMEDIUMINT:
      *size = 4;
      break;

    case  SQLSYNC_SQLSERVER_DATA_MEDIUMINT:
      *size = 4;
      break;

    case  SQLSYNC_SQLSERVER_DATA_UINTEGER:
      *size = 4;
      break;

    case  SQLSYNC_SQLSERVER_DATA_INTEGER:
      *size = 4;
      break;

    case  SQLSYNC_SQLSERVER_DATA_UBIGINT:
      *size = 8;
      break;

    case  SQLSYNC_SQLSERVER_DATA_BIGINT:
      *size = 8;
      break;

    case  SQLSYNC_SQLSERVER_DATA_UNUMERIC:
    case  SQLSYNC_SQLSERVER_DATA_NUMERIC:
      *size = sizeof(odbc_numeric_struct);
      break;

    case  SQLSYNC_SQLSERVER_DATA_FLOAT:
      *size = sizeof(float);
      break;

    case  SQLSYNC_SQLSERVER_DATA_DOUBLE:
      *size = sizeof(double);
      break;

    case  SQLSYNC_SQLSERVER_DATA_MONEY:
      ;//return SQL_CHAR;
      break;

    default:
        ;
  }
}

c_data_elem_ptr sqlsync_sqlserver_create_data(sqlsync_sqlserver_data_type type, uint32 max_size)
{
  c_data_elem_ptr elem = NULL;

  switch (type)
  {
    case SQLSYNC_SQLSERVER_DATA_BIT:
    {
      elem = (c_data_elem_ptr) c_bit_elem_empty();
      c_binary_elem_append_reserve((c_binary_elem_ptr)elem, max_size/255);
      ((c_bit_elem_ptr)elem)->max_size = max_size;
      break;
    }

    case SQLSYNC_SQLSERVER_DATA_XML:
    case SQLSYNC_SQLSERVER_DATA_UNKNOWN:
    case SQLSYNC_SQLSERVER_DATA_CHAR:
    case SQLSYNC_SQLSERVER_DATA_WCHAR:
    case SQLSYNC_SQLSERVER_DATA_CBLOB:
    {
      if (type == SQLSYNC_SQLSERVER_DATA_XML && max_size < 10)
        max_size = 1000;

      elem = (c_data_elem_ptr) c_string_elem_reserve (max_size);
      break;
    }

    case SQLSYNC_SQLSERVER_DATA_GUID:
    {
      elem = (c_data_elem_ptr) c_guid_elem_empty();
      break;
    }

    case SQLSYNC_SQLSERVER_DATA_BLOB:
    case SQLSYNC_SQLSERVER_DATA_BINARY:
    { 
      elem = (c_data_elem_ptr) c_binary_elem_empty();
      c_binary_elem_append_reserve((c_binary_elem_ptr)elem, max_size);
      break;
    }
        
    case SQLSYNC_SQLSERVER_DATA_TINYINT:
    {
      elem = (c_data_elem_ptr) c_int8_elem_empty ();
      break;
    }

    case SQLSYNC_SQLSERVER_DATA_UTINYINT:
    {
      elem = (c_data_elem_ptr) c_uint8_elem_empty ();
      break;
    }

    case SQLSYNC_SQLSERVER_DATA_SMALLINT:
    {

      elem = (c_data_elem_ptr) c_int16_elem_empty ();
      break;
    }

    case SQLSYNC_SQLSERVER_DATA_USMALLINT:
    {
      elem = (c_data_elem_ptr) c_uint16_elem_empty ();
      break;
    }

    case SQLSYNC_SQLSERVER_DATA_BOOL:
    {
      elem = (c_data_elem_ptr) c_bool_elem_empty ();
      break;
    }

    case SQLSYNC_SQLSERVER_DATA_MEDIUMINT:
    case SQLSYNC_SQLSERVER_DATA_INTEGER:
    {  
      elem = (c_data_elem_ptr) c_int32_elem_empty ();
      break;
    }

    case SQLSYNC_SQLSERVER_DATA_UMEDIUMINT:
    case SQLSYNC_SQLSERVER_DATA_UINTEGER:
    {
      elem = (c_data_elem_ptr) c_uint32_elem_empty ();
      break;
    }

    case SQLSYNC_SQLSERVER_DATA_FLOAT:
      elem = (c_data_elem_ptr) c_float_elem_empty ();
      break;

    case SQLSYNC_SQLSERVER_DATA_DOUBLE:
     elem = (c_data_elem_ptr) c_double_elem_empty ();
      break;


    case SQLSYNC_SQLSERVER_DATA_DATE:
    case SQLSYNC_SQLSERVER_DATA_DATETIME:
    {
      elem = (c_data_elem_ptr) c_date_elem_empty ();
      break;
    }

    case SQLSYNC_SQLSERVER_DATA_TIME:
    case SQLSYNC_SQLSERVER_DATA_TIME_TZ:
    {
      elem = (c_data_elem_ptr) c_time_elem_empty ();
      break;
    }

    case SQLSYNC_SQLSERVER_DATA_YEAR:
    case SQLSYNC_SQLSERVER_DATA_TIMESTAMP:
    case SQLSYNC_SQLSERVER_DATA_TIMESTAMP_TZ:
    {
      elem = (c_data_elem_ptr) c_timestamp_elem_empty ();
      break;
    }

    case SQLSYNC_SQLSERVER_DATA_BIGINT:
    {
      elem = (c_data_elem_ptr) c_bigint_elem_empty ();
      break;
    }
    case SQLSYNC_SQLSERVER_DATA_UBIGINT:
    { 
      elem = (c_data_elem_ptr) c_ubigint_elem_empty ();
      break;
    }

    case SQLSYNC_SQLSERVER_DATA_UNUMERIC:
    case SQLSYNC_SQLSERVER_DATA_NUMERIC:
    {
      elem = (c_data_elem_ptr) c_numeric_elem_empty ();
      break;
    }


    case SQLSYNC_SQLSERVER_DATA_INTERVAL:
    case SQLSYNC_SQLSERVER_DATA_MONEY:
    default:
        elem = (c_data_elem_ptr) c_string_elem_reserve (max_size);
      break;
  }

  if (elem == NULL)
  {
    return NULL;
  }
  return elem;
}


