
#include "server/sqlsync_sqlserver.h"
#include "server/sqlsync_sqlserver_priv.h"
#include "server/sqlsync_sqlserver_helpers.h"

#include "sqlsync_user_internal.h"
#include "sqlsync_translation.h"

#ifdef DEBUG
#include "sqlsync_debug.h"
#endif

void
sqlsync_sqlserver_print_error (sqlsync_sqlserver * server)
{
  SQLRETURN ret;
  SQLCHAR stat[10];             /* Status SQL */
  SQLINTEGER err;
  SQLSMALLINT mlen;
  SQLCHAR msg[1000];
  SQLHANDLE handle = NULL;

  if (server->last_bad_handle == SQLSYNC_SUCCESS)
  {
#ifdef DEBUG
    sqlsync_debug ("Error type %d\n", server->error);
#endif
  }
  else
  {
#ifdef DEBUG
    sqlsync_debug ("Bad handle %d\n", server->last_bad_handle);
#endif

    switch (server->last_bad_handle)
    {
      case SQL_HANDLE_ENV:
      {
        handle = server->env;
        break;
      }
      case SQL_HANDLE_DBC:
      {
        handle = server->hdbc;
        break;
      }
      case SQL_HANDLE_STMT:
      {
        handle = server->hstmt;
        if (server->error == SQLSYNC_FAILED_QUERY)
          handle = server->hstmt_query;
        break;
      }
      default:
      {
        sqlsync_send_error_notification2 (_tr ("Bad handle in sqlsync_print_error()"));
        return;
      }
    }

    ret = SQLGetDiagRec (server->last_bad_handle, (SQLHANDLE) handle, 1,
                         stat, &err, msg, 1000, &mlen);

    sqlsync_send_error_notification2 ((char *) msg);
  }
}



SQLSYNC_RETURN
sqlsync_show_available_dsns ()
{
  SQLHENV env;
  SQLRETURN ret;
  SQLCHAR l_dsn[100], l_desc[100];
  SQLUSMALLINT l_len1, l_len2, l_next;


  ret = SQLAllocHandle (SQL_HANDLE_ENV, (SQLHANDLE) SQL_NULL_HANDLE, &env);
  if (SQLSYNC_IS_ERROR (ret))
  {
#ifdef DEBUG
    sqlsync_debug ("SQLAllocHandle returned error %d\n", ret);
#endif
    return SQLSYNC_FAILED;
  }

  ret = SQLSetEnvAttr (env, SQL_ATTR_ODBC_VERSION, (void *) SQL_OV_ODBC3, 0);

  if (SQLSYNC_IS_ERROR (ret))
  {
#ifdef DEBUG
    sqlsync_debug ("SQLSetEnvAttr returned error %d\n", ret);
#endif
    (void) SQLFreeHandle (SQL_HANDLE_ENV, env);
    return SQLSYNC_FAILED;
  }


  sqlsync_send_info_notification2 (_tr ("Available DSNs"));

  l_next = SQL_FETCH_FIRST;

  while (SQLDataSources (env, l_next, l_dsn, (SQLSMALLINT) sizeof (l_dsn),
                         (SQLSMALLINT *) (&l_len1), l_desc,
                         (SQLSMALLINT) sizeof (l_desc), (SQLSMALLINT *) & l_len2) == SQL_SUCCESS)
  {
    sqlsync_user_data info_user_data;
    c_string_elem_ptr info_dsn = c_string_elem_empty ();
    c_string_elem_ptr info_description = c_string_elem_empty ();

    c_string_elem_append (info_dsn, (char *) l_dsn, l_len1);

    c_string_elem_append (info_description, (char *) l_desc, l_len2);

    info_user_data.user_data.dsn_info.dsn = info_dsn;
    info_user_data.user_data.dsn_info.description = info_description;
    sqlsync_send_user_notification (SQLSYNC_USER_DSN_INFO, &info_user_data);

    c_string_elem_free (info_dsn);
    c_string_elem_free (info_description);

    l_next = SQL_FETCH_NEXT;
  }

  (void) SQLFreeHandle (SQL_HANDLE_ENV, env);
  return SQLSYNC_SUCCESS;
}


