/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#ifndef SQLSYNC_SQLSERVER_PRIV_H
#define SQLSYNC_SQLSERVER_PRIV_H

#if (defined(_WINDOWS) || defined(__MINGW32__))
#include <windef.h>
#include <winnt.h>
#endif

#include <sql.h>
#include <sqltypes.h>
#include <sqlext.h>

#ifdef WIN32
#define SQLTables_TABLE_NAME 3
#define SQLColumns_COLUMN_NAME 4
#define SQLColumns_DATA_TYPE 5
#define SQLColumns_TYPE_NAME 6
#define SQLColumns_COLUMN_SIZE 7
#define SQLColumns_SQL_DATA_TYPE 14
#endif

#include "sqlsync_def.h"
#include "data_types/sqlsync_data_string.h"
#include "data_types/sqlsync_data_misc.h"

typedef struct _sqlsync_sqlserver_impl sqlsync_sqlserver_impl;



struct _sqlsync_sqlserver
{
  sqlsync_server_type type;
  sqlsync_sqlserver_impl * impl;
  SQLHENV env;               /*!< environment handle */
  SQLHDBC hdbc;              /*!< connection handle */
  SQLHSTMT hstmt;            /*!< statement handle */
  SQLHSTMT hstmt_query;      /*!< statement handle for query */

  SQLSMALLINT last_bad_handle;  /*!< stores handle type - 
                                   used for determine what 
                                   kind of error should be printed 
                                 */
  SQLSYNC_RETURN error;
  SQLSYNC_RETURN cs_tables_columns;
};

typedef c_string_elem_ptr (*sqlsync_sqlserver_data_to_string_t)(c_data_elem_ptr elem, SQLSYNC_RETURN with_escapes);

typedef c_string_elem_ptr (*sqlsync_sqlserver_column_identifier_t)(c_string_elem_ptr elem);

typedef SQLSYNC_RETURN (*sqlsync_sqlserver_is_system_table_t)(c_string_elem_ptr table);

typedef SQLSYNC_RETURN (*sqlsync_sqlserver_does_table_exist_t)(sqlsync_sqlserver * server, c_string_elem_ptr table);

typedef c_vector_columns_ptr (*sqlsync_sqlserver_get_columns_t)(sqlsync_sqlserver* server, c_string_elem_ptr table);

typedef c_vector_columns_ptr (*sqlsync_get_primary_keys_t) (sqlsync_sqlserver * server, c_string_elem_ptr table);

typedef sqlsync_sqlserver_data_type (*sqlsync_sqlserver_recognize_data_type_t)(c_string_elem_ptr data_type_name, int8 * use_operator);

typedef sqlsync_sqlserver_data_type (*sqlsync_sqlserver_recognize_data_type_odbc_t)(int16 data_type, int8 * use_operator);

typedef c_vector_strings_ptr (*sqlsync_sqlserver_get_tables_t)(sqlsync_sqlserver * server);


struct _sqlsync_sqlserver_impl
{
  sqlsync_sqlserver_data_to_string_t data_to_string;
  sqlsync_sqlserver_column_identifier_t column_identifier;
  sqlsync_sqlserver_is_system_table_t is_system_table;
  sqlsync_sqlserver_does_table_exist_t does_table_exist;
  sqlsync_sqlserver_get_columns_t get_columns;
  sqlsync_get_primary_keys_t get_primary_keys;
  sqlsync_sqlserver_recognize_data_type_t recognize_data_type;
  sqlsync_sqlserver_recognize_data_type_odbc_t recognize_data_type_odbc;
  sqlsync_sqlserver_get_tables_t get_tables;
};


#endif /* SQLSYNC_SQLSERVER_PRIV_H */
