/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#ifndef SQLSYNC_H
#define SQLSYNC_H

#include "sqlsync_def.h"
#include "data_types/sqlsync_data_string.h"
#include "data_types/sqlsync_data_misc.h"

/** 
 *    This function initalizes all internal data to work with library.
 *   Function must be executed before any other from this library.
 *   @return Pointer to the structure that will be used by sqlsync library
 */
sqlsync_info_ptr sqlsync_init (SQLSYNC_RETURN * error);

/** 
 *   This function finalize all internal data to work with library.
 *   Function must be executed after any other from this library.
 *   @param info Pointer to the structure that will be destroyed
 *               in that function, internal handles will be
 *               destroyed
 *   @return Returns SQLSYNC_FAILED_DESTROY if something went
 *           wrong otherwise SQLSYNC_SUCCESS is returned
 */
SQLSYNC_RETURN sqlsync_destroy (sqlsync_info_ptr info_ptr);

/**
 *  This function sets or clear case sensitivity 
 *  when comparing table or column names
 *   
 */
void sqlsync_ignore_case_sensitive(sqlsync_info_ptr info_ptr, SQLSYNC_RETURN ignore);

/**
 *   Connects to specified server
 *   @param info Pointer to the main structure
 *  @param nr_server Which server (SRC or DST)
 *  @param dsn DSN name
 *  @param user User name
 *  @param password Password
 *   @return Returns SQLSYNC_FAILED_NULL if pointer were null,if
 *           something went wrong in connection phase
 *           SQLSYNC_FAILED_CONNECT is returned otherwise
 *           SQLSYNC_SUCCESS is returned
 */
SQLSYNC_RETURN sqlsync_connect (sqlsync_info_ptr info_ptr,
                                sqlsync_server nr_server,
                                c_string_elem_ptr dsn,
                                c_string_elem_ptr user, c_string_elem_ptr password);


/**
 * Function shows available DSNs
 * 
 * @return SQLSYNC_RETURN Returns SQLSYNC_FAILED if something
 *         went wrong otherwise SQLSYNC_SUCCESS
 */
SQLSYNC_RETURN sqlsync_show_available_dsns ();


/**
 *   Shows error message
 *   Function prints message to stderr
 *   @param info Pointer to the main structure
 */
void sqlsync_print_error (sqlsync_info_ptr info_ptr, sqlsync_server nr_server);

uint32
sqlsync_compare_databases (sqlsync_info_ptr info_ptr, c_vector_strings_ptr tables,
                           c_grid_columns_ptr tables_columns,
                           SQLSYNC_RETURN show_summary,
                           SQLSYNC_RETURN show_queries, SQLSYNC_RETURN run_queries);

/**
 *   Executes SQL queries from file
 *   @param info Pointer to main structure
 *   @param nr_server Which server (SRC or DST) should receive
 *                    query
 *   @param name Filename with queries to run
 *   @param name_size Size of filename
 *   @return Returns SQLSYNC_FAILED_QUERY if something went wrong
 *   if pointers are null function returns SQLSYNC_FAILED_NULL
 *           otherwise SQLSYNC_SUCCESS
 */
SQLSYNC_RETURN sqlsync_run_query_file (sqlsync_info_ptr /*@null@ */ info_ptr,
                                       sqlsync_server nr_server, c_string_elem_ptr name);



void sqlsync_parse_tables_columns (c_string_elem_ptr tables_columns,
                                   /*@out@ */ c_vector_strings_ptr * tables,
                                   /*@out@ */ c_grid_columns_ptr * columns);

#endif /* SQLSYNC_H */
