/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#include "sqlsync_debug.h"

#include "data_types/sqlsync_data_numeric.h"

#include "data_types/sqlsync_data_converter.h"

#include "sqlsync_compare_records.h"


int
sqlsync_compare_records (c_vector_data_ptr row_src, c_vector_data_ptr row_dst,
                         c_vector_int32_ptr primary_keys_pos)
{
  int match_count = 0;
  int miss = 0, add = 0, diff = 0;
  int cmp = 0;
  int max_diff = (SQLSYNC_COMPARE_DIFF * c_vector_data_size (row_dst)) / 100;
  int i, size = 0;
  int primary_size = 0;
  c_data_elem_ptr src;
  c_data_elem_ptr dst;


#ifdef DEBUG
  /*@unused@ */ static int c_debug = 0;

  sqlsync_debug ("Comparing records (compared records till now %ld)\n", c_debug++);
#endif

  if (row_src == NULL && row_dst != NULL)
  {
#ifdef DEBUG
    sqlsync_debug ("Additional record\n");
#endif
    return SQLSYNC_REC_ADD;
  }

  if (row_src == NULL && row_dst == NULL)
  {
#ifdef DEBUG
    sqlsync_debug ("Records are empty\n");
#endif
    return SQLSYNC_REC_EMPTY;
  }

  if (row_src != NULL && row_dst == NULL)
  {
#ifdef DEBUG
    sqlsync_debug ("Missing record\n");
#endif
    return SQLSYNC_REC_MISS;
  }

  if (row_src->size != row_dst->size)
  {
#ifdef DEBUG
    sqlsync_debug ("Records have different number of elements\n");
#endif

    return SQLSYNC_FAILED;
  }

  size = c_vector_data_size (row_src);

  primary_size = c_vector_int32_size (primary_keys_pos);
  if (primary_size > 0)
  {
    for (i = 0; i < primary_size; i++)
    {
      int pos = ((c_int32_elem_ptr) c_vector_int32_pos (primary_keys_pos, i))->value;

      src = c_vector_data_pos (row_src, pos);
      dst = c_vector_data_pos (row_dst, pos);

      if (src && src->_type == c_string_elem_type)
        c_string_elem_strip_lspaces((c_string_elem_ptr)src);

      if (dst && dst->_type == c_string_elem_type)
        c_string_elem_strip_lspaces((c_string_elem_ptr)dst);

      if ((cmp = c_data_elem_cmp (src, dst)) == 0)
        match_count++;
      else if (diff == 0 && cmp < 0)
      {
        diff = 1;
        miss = 1;
      }
      else if (diff == 0 && cmp > 0)
      {
        diff = 1;
        add = 1;
      }
      max_diff = primary_size;
#ifdef DEBUG
      sqlsync_debug_details ("Result of compare %d \n", cmp);
#endif

    }
  }
  if (primary_size == match_count && diff == 0 && miss == 0 && add == 0)
  {
    for (i = 0; i < size; i++)
    {
      src = c_vector_data_pos (row_src, i);
      dst = c_vector_data_pos (row_dst, i);

      if (src && src->_type == c_string_elem_type)
        c_string_elem_strip_lspaces((c_string_elem_ptr)src);

      if (dst && dst->_type == c_string_elem_type)
        c_string_elem_strip_lspaces((c_string_elem_ptr)dst);


      if ((cmp = c_data_elem_cmp (src, dst)) == 0)
        match_count++;
      else if (diff == 0 && cmp < 0)
      {
        diff = 1;
        miss = 1;
      }
      else if (diff == 0 && cmp > 0)
      {
        diff = 1;
        add = 1;
      }

#ifdef DEBUG
      sqlsync_debug_details ("Result of compare %d \n", cmp);
#endif

    }
  }

#ifdef DEBUG
  sqlsync_debug ("Result of compare %d with max_diff %d and match_count "
                 "%d miss %d add %d\n", cmp, max_diff, match_count, miss, add);
#endif

  if (diff == 0)
    return SQLSYNC_REC_MATCH;

  else if (miss == 1 && max_diff >= match_count)
  {
#ifdef DEBUG
    sqlsync_debug ("Missing record\n");
#endif
    return SQLSYNC_REC_MISS;
  }

  else if (add == 1 && max_diff >= match_count)
  {
#ifdef DEBUG
    sqlsync_debug ("Additional record\n");
#endif
    return SQLSYNC_REC_ADD;
  }

  else
  {
#ifdef DEBUG
    sqlsync_debug ("Records are different\n");
#endif
    return SQLSYNC_REC_DIFF;
  }

}


void
sqlsync_compare_grids (c_grid_data_ptr grid_src,
                       c_grid_data_ptr grid_dst,
                       c_vector_int32_ptr key_pos,
                       c_grid_data_ptr grid_miss,
                       c_grid_data_ptr grid_add,
                       c_grid_data_ptr grid_diff_src,
                       c_grid_data_ptr grid_diff_dst, uint32 * equals)
{

  int r_cmp;
  c_vector_data_ptr row_src;
  c_vector_data_ptr row_dst;

  uint32 row_src_pos = 0;
  uint32 row_dst_pos = 0;

  (void) sqlsync_compare_sort_grids (grid_src, grid_dst, key_pos);


  row_src = c_grid_data_pos (grid_src, row_src_pos);
  row_dst = c_grid_data_pos (grid_dst, row_dst_pos);

  do
  {

#ifdef DEBUG
    sqlsync_debug ("Comparing records row_dst %p row_src %p on (%ld and %ld) positions\n",
                   row_dst, row_src, row_dst_pos, row_src_pos);
#endif

    /* Compare rows from tables */
    r_cmp = sqlsync_compare_records (row_src, row_dst, key_pos);

#ifdef DEBUG
    sqlsync_debug_details ("Result of comparing records r_cmp %d on (%ld and %ld) positions\n",
                           r_cmp, row_dst_pos, row_src_pos);
#endif


#ifdef DEBUG
    sqlsync_debug ("Comparing two rows\n");

    sqlsync_debug ("row_src\n");
    c_grid_data_elem_print (row_src, 0, NULL);
    //c_grid_print_elem (row_src, NULL);

    sqlsync_debug ("row_dst\n");
    c_grid_data_elem_print (row_dst, 0, NULL);
//    c_grid_print_elem (row_dst, NULL);
#endif


    /* Checking if rows are different */
    if (r_cmp == SQLSYNC_REC_DIFF)
    {
      /* Adding rows that are different */
      if (row_src != NULL && row_dst != NULL)
      {
        (void) c_grid_data_add_last (grid_diff_src, row_src);
        (void) c_grid_data_add_last (grid_diff_dst, row_dst);
      }

      /* Get next rows from tables */
      row_dst = c_grid_data_pos (grid_dst, ++row_dst_pos);
      row_src = c_grid_data_pos (grid_src, ++row_src_pos);

      continue;
    }

    /* Checking if row exists in fist database but doesn't exist in second */
    else if (r_cmp == SQLSYNC_REC_MISS)
    {
      /* Adding row from first table to missing rows */
      if (row_src != NULL)
      {
        (void) c_grid_data_add_last (grid_miss, row_src);
      }

      /* Getting row from second table */
      row_src = c_grid_data_pos (grid_src, ++row_src_pos);
      continue;
    }

    /* Checking if row exists in second database but doesn't exist in first */
    else if (r_cmp == SQLSYNC_REC_ADD)
    {
      /* Adding row from second table to additional rows */
      if (row_dst != NULL)
      {
        (void) c_grid_data_add_last (grid_add, row_dst);
      }
      /* Getting row from first table */
      row_dst = c_grid_data_pos (grid_dst, ++row_dst_pos);
      continue;
    }

    else
    {
      /* Update info about number of rows that are equal */
      if (row_dst != NULL && row_src != NULL)
        (*equals)++;

      /* If rows are equal then get row from tables */
      row_dst = c_grid_data_pos (grid_dst, ++row_dst_pos);
      row_src = c_grid_data_pos (grid_src, ++row_src_pos);

      continue;
    }

    /* If there are no rows in first table and rows are
     * not different (other state) get row from second table
     *
     */

    if (row_src == NULL)
    {
      row_dst = c_grid_data_pos (grid_dst, ++row_dst_pos);
      continue;
    }

    /* If there are no rows in second table and rows are
     * not different (other state) get row first second table
     */

    if (row_dst == NULL)
    {
      row_src = c_grid_data_pos (grid_src, ++row_src_pos);
      continue;
    }

  }
  /* Run until there are rows to fetch */
  while (!(row_src == NULL && row_dst == NULL));

}

void c_grid_data_elem_print (c_vector_data_ptr container, uint32 pos, void *user);

void
sqlsync_compare_sort_grids (c_grid_data_ptr grid_src,
                            c_grid_data_ptr grid_dst, c_vector_int32_ptr primary_keys_pos)
{


#ifdef DEBUG
  sqlsync_debug ("Unsorted source rows\n");
  c_grid_data_all_elems (grid_src, c_grid_data_elem_print, NULL);

  sqlsync_debug ("Unsorted destination rows\n");
  c_grid_data_all_elems (grid_dst, c_grid_data_elem_print, NULL);
  sqlsync_debug ("Source grid size to sort (%ld)\n", c_grid_data_size (grid_src));
#endif

  if (c_vector_int32_size (primary_keys_pos) != 0)
  {
    (void) c_grid_data_sort_by_primary_keys_pos (grid_src, primary_keys_pos);
  }
  else
    (void) c_grid_data_sort_by_all (grid_src);

#ifdef DEBUG
  sqlsync_debug ("Sorted source grid\n");
  c_grid_data_all_elems (grid_src, c_grid_data_elem_print, NULL);
  sqlsync_debug ("Destination grid size to sort(%ld)\n", c_grid_data_size (grid_dst));
#endif

  if (c_vector_int32_size (primary_keys_pos) != 0)
  {
    (void) c_grid_data_sort_by_primary_keys_pos (grid_dst, primary_keys_pos);
  }
  else
    (void) c_grid_data_sort_by_all (grid_dst);

#ifdef DEBUG
  sqlsync_debug ("Sorted destination grid\n");
  c_grid_data_all_elems (grid_dst, c_grid_data_elem_print, NULL);
#endif

}
