/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#ifndef SQLSYNC_COMPARE_RECORDS_H
#define SQLSYNC_COMPARE_RECORDS_H

#include "data_types/sqlsync_data_vector_misc.h"
#include "data_types/sqlsync_data_grid.h"

/**
 * How many columns can be different in row, to be still
 * different row not additional or missing, in percents. I.e 70
 * means that 70% of columns can be different and compared rows
 * still will be different.
 */
#define SQLSYNC_COMPARE_DIFF 70
/**
 * Results of matching records
 */
enum sqlsync_match
{
  SQLSYNC_REC_MATCH = 2,        /*!< records are equal */
  SQLSYNC_REC_MISS = 4,         /*!< src record is missing in destination table */
  SQLSYNC_REC_ADD = 8,          /*!< dst record exists only in destination table */
  SQLSYNC_REC_DIFF = 16,        /*!< compared rows are different */
  SQLSYNC_REC_EMPTY = 32        /*!< both records are NULL */
};


/**
 * Function compares two rows
 * @param row_src Pointer to row from SRC server
 * @param row_dst Pointer to row from DST server
 * @param primary_keys_pos List of posititions which are used for
 *                  sorting (elements in that list are "int"
 *                  values)
 * 
 * @return int Result of comparison (enum sqlsync_match)

 */
int sqlsync_compare_records (c_vector_data_ptr row_src,
                             c_vector_data_ptr row_dst, c_vector_int32_ptr primary_keys_pos);


void
sqlsync_compare_grids (c_grid_data_ptr grid_src,
                       c_grid_data_ptr grid_dst,
                       c_vector_int32_ptr key_pos,
                       c_grid_data_ptr grid_miss,
                       c_grid_data_ptr grid_add,
                       c_grid_data_ptr grid_diff_src,
                       c_grid_data_ptr grid_diff_dst, uint32 * equals);

void
sqlsync_compare_sort_grids (c_grid_data_ptr grid_src,
                            c_grid_data_ptr grid_dst, c_vector_int32_ptr primary_keys_pos);

#endif /* SQLSYNC_COMPARE_RECORDS_H */
