/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#include "sqlsync_compare_results.h"

c_cmp_ptr
c_cmp_init (sqlsync_server_type type)
{
  c_cmp_ptr q;

  q = (c_cmp_ptr) std_malloc (sizeof (c_cmp_results));
  if (q == NULL)
    return NULL;

  q->row_diff_src = c_grid_data_init (DEFAULT_DATA_SIZE);
  q->row_diff_dst = c_grid_data_init (DEFAULT_DATA_SIZE);
  q->row_miss = c_grid_data_init (DEFAULT_DATA_SIZE);
  q->row_add = c_grid_data_init (DEFAULT_DATA_SIZE);

  q->table_name = NULL;
  q->columns = NULL;
  q->primary_keys = NULL;
  q->primary_keys_pos = NULL;
  q->sql_query_add = NULL;
  q->sql_query_miss = NULL;
  q->sql_query_diff = NULL;
  q->equals = 0;
  q->type = type;
  return q;
}


SQLSYNC_RETURN
c_cmp_set_primary_keys (c_cmp_ptr container, c_vector_columns_ptr primary_keys,
                        c_vector_int32_ptr primary_keys_pos)
{
  if (container == NULL)
    return SQLSYNC_FAILED_NULL;

  container->primary_keys = primary_keys;
  container->primary_keys_pos = primary_keys_pos;

  return SQLSYNC_SUCCESS;
}

SQLSYNC_RETURN
c_cmp_set_name (c_cmp_ptr container, c_string_elem_ptr name)
{
  if (container == NULL || name == NULL)
    return SQLSYNC_FAILED_NULL;

  container->table_name = c_string_elem_copy (name);

  return SQLSYNC_SUCCESS;
}

SQLSYNC_RETURN
c_cmp_increment_equals_count (c_cmp_ptr container)
{
  if (container == NULL)
    return SQLSYNC_FAILED_NULL;

  container->equals++;
  return SQLSYNC_SUCCESS;
}

SQLSYNC_RETURN
c_cmp_add_columns (c_cmp_ptr container, c_vector_columns_ptr columns)
{
  if (container == NULL || columns == NULL)
    return SQLSYNC_FAILED_NULL;

  container->columns = c_vector_columns_copy (columns);

  return SQLSYNC_SUCCESS;
}

SQLSYNC_RETURN
c_cmp_add_rows_diff (c_cmp_ptr container, c_grid_data_ptr row_src, c_grid_data_ptr row_dst)
{
  if (container == NULL || container->row_diff_src == NULL || container->row_diff_dst == NULL)
    return SQLSYNC_FAILED_NULL;

  container->row_diff_src = row_src;
  container->row_diff_dst = row_dst;

  return SQLSYNC_SUCCESS;
}

SQLSYNC_RETURN
c_cmp_add_row_miss (c_cmp_ptr container, c_grid_data_ptr row_miss)
{
  if (container == NULL || container->row_miss == NULL)
    return SQLSYNC_FAILED_NULL;

  container->row_miss = row_miss;

  return SQLSYNC_SUCCESS;
}


SQLSYNC_RETURN
c_cmp_add_row_add (c_cmp_ptr container, c_grid_data_ptr row_add)
{

  if (container == NULL || container->row_add == NULL)
    return SQLSYNC_FAILED_NULL;

  container->row_add = row_add;

  return SQLSYNC_SUCCESS;
}


SQLSYNC_RETURN
c_cmp_clear (c_cmp_ptr container)
{
  if (container == NULL)
    return SQLSYNC_FAILED_NULL;

  if (container->row_diff_src)
    (void) c_grid_data_destroy (container->row_diff_src);

  if (container->row_diff_dst)
    (void) c_grid_data_destroy (container->row_diff_dst);

  if (container->row_miss)
    (void) c_grid_data_destroy (container->row_miss);

  if (container->row_add)
    (void) c_grid_data_destroy (container->row_add);

  if (container->columns)
    (void) c_vector_columns_destroy (container->columns);

  if (container->sql_query_add)
    (void) c_vector_strings_destroy (container->sql_query_add);

  if (container->sql_query_miss)
    (void) c_vector_strings_destroy (container->sql_query_miss);

  if (container->sql_query_diff)
    (void) c_vector_strings_destroy (container->sql_query_diff);

  c_vector_columns_destroy (container->primary_keys);

  c_vector_int32_destroy (container->primary_keys_pos);


  c_string_elem_free (container->table_name);
  container->table_name = NULL;

  return SQLSYNC_SUCCESS;
}

SQLSYNC_RETURN
c_cmp_destroy_all (c_cmp_ptr ** container, uint32 size)
{
  if (container != NULL && *container != NULL)
  {
    uint32 i = 0;

    for (i = 0; i < size; i++)
    {
      (void) c_cmp_destroy (&(*container)[i]);
    }

    std_free (*container);
    *container = NULL;
  }
  return SQLSYNC_SUCCESS;

}

SQLSYNC_RETURN
c_cmp_destroy (c_cmp_ptr * container)
{
  if (container)
  {
    (void) c_cmp_clear (*container);
    std_free (*container);
    *container = NULL;
  }
  return SQLSYNC_SUCCESS;
}
