/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#ifndef SQLSYNC_COMPARE_RESULTS_H
#define SQLSYNC_COMPARE_RESULTS_H

#include "data_types/sqlsync_data_misc.h"
#include "data_types/sqlsync_data_string.h"
#include "data_types/sqlsync_data_vector_misc.h"
#include "data_types/sqlsync_data_grid.h"

typedef struct
{
  c_string_elem_ptr table_name; /*!< Table name */

  c_vector_columns_ptr primary_keys;    /*!<Primary keys names */
  c_vector_int32_ptr primary_keys_pos;  /*!< Positions of primary keys */
  c_vector_columns_ptr columns; /*!< List of columns */
  uint32 equals;                /*!< Number of rows that are equal in compared tables */

  c_vector_strings_ptr sql_query_diff;  /*!<Queries which modifies rows */
  c_vector_strings_ptr sql_query_miss;  /*!<Queries which adds rows to table */
  c_vector_strings_ptr sql_query_add;   /*!<Queries which deletes rows from table */

  c_grid_data_ptr row_miss;     /*!< Grid with missing rows */
  c_grid_data_ptr row_add;      /*!<Grid with additional rows */
  c_grid_data_ptr row_diff_src; /*!< Grid with different rows from source table */
  c_grid_data_ptr row_diff_dst; /*!< Grid with different rows 
                                   from destination table */
  sqlsync_server_type type;     /*!<Destination server type */
} c_cmp_results;

typedef c_cmp_results *c_cmp_ptr;


/**
 * Function creates structure where results of comparizion will
 * be stored (rows, queries, etc)
 * 
 * @return c_cmp_ptr
 */
c_cmp_ptr c_cmp_init (sqlsync_server_type type);


SQLSYNC_RETURN c_cmp_set_primary_keys (c_cmp_ptr container,
                                       c_vector_columns_ptr primary_keys,
                                       c_vector_int32_ptr primary_keys_pos);

/**
 * Function sets table name
 * @param container Pointer to the container
 * @param name Table name (copied to the internal structure)
 * 
 * @return SQLSYNC_RETURN Returns SQLSYNC_FAILED_NULL if pointer
 *         were null or SQLSYNC_SUCCESS otherwise
 * 
 */
SQLSYNC_RETURN c_cmp_set_name (c_cmp_ptr container, c_string_elem_ptr name);

/**
 * Function copies list of columns
 * @param container Pointer to the container
 * @param columns List of columns stored in c_data_ptr (see
 *                c_column_elem or sqlsync_get_columns)
 * @see c_column_elem, sqlsync_get_columns
 * 
 * @return SQLSYNC_RETURN Returns SQLSYNC_FAILED_NULL if
 *         something went wrong otherwise SQLSYNC_SUCCESS is
 *         returned
 */
SQLSYNC_RETURN c_cmp_add_columns (c_cmp_ptr container, c_vector_columns_ptr columns);

/**
 * Increases number of equals rows
 * @param container Pointer to the container
 * 
 * @return SQLSYNC_RETURN Returns SQLSYNC_FAILED_NULL if
 *         something went wrong otherwise SQLSYNC_SUCCESS is
 *         returned
 */
SQLSYNC_RETURN c_cmp_increment_equals_count (c_cmp_ptr container);

/**
 * Function copies rows that are different
 * @param container Pointer to the container
 * @param row_src Row from source table
 * @param row_dst Row from destination table
 * 
 * @return SQLSYNC_RETURN Returns SQLSYNC_FAILED_NULL if
 *         something went wrong otherwise SQLSYNC_SUCCESS is
 *         returned
 */
SQLSYNC_RETURN c_cmp_add_rows_diff (c_cmp_ptr container, c_grid_data_ptr row_src,
                                    c_grid_data_ptr row_dst);

/**
 * Function copies row that is missing in destination table
 * @param container Pointer to the container
 * @param row_miss Missing row
 * 
 * @return SQLSYNC_RETURN Returns SQLSYNC_FAILED_NULL if
 *         something went wrong otherwise SQLSYNC_SUCCESS is
 *         returned
 */
SQLSYNC_RETURN c_cmp_add_row_miss (c_cmp_ptr container, c_grid_data_ptr row_miss);

/**
 * Function copies row that is additional in destination table
 * @param container Pointer to the container
 * @param row_add Additional row
 * 
 * @return SQLSYNC_RETURN Returns SQLSYNC_FAILED_NULL if
 *         something went wrong otherwise SQLSYNC_SUCCESS is
 *         returned
 */
SQLSYNC_RETURN c_cmp_add_row_add (c_cmp_ptr container, c_grid_data_ptr row_add);

/**
 * Function clears whole internal structure (all rows, columns,
 * names etc are freed
 * @param container Pointer to the container
 * @return SQLSYNC_RETURN Returns SQLSYNC_FAILED_NULL if
 *         something went wrong otherwise SQLSYNC_SUCCESS is
 *         returned
 */
SQLSYNC_RETURN c_cmp_clear (c_cmp_ptr container);

/**
 * Function destroyes container
 * @param container Pointer to the container
 * 
 * @return SQLSYNC_RETURN Returns SQLSYNC_FAILED_NULL if
 *         something went wrong otherwise SQLSYNC_SUCCESS is
 *         returned
 */
SQLSYNC_RETURN c_cmp_destroy ( /*@out@ */ c_cmp_ptr * container);

/**
 * Function destroyes array of containers
 * @param container Pointer to the array of containers
 * @param size Numer of containers in array
 * 
 * @return SQLSYNC_RETURN Returns SQLSYNC_FAILED_NULL if
 *         something went wrong otherwise SQLSYNC_SUCCESS is
 *         returned
 */
SQLSYNC_RETURN c_cmp_destroy_all ( /*@out@ */ c_cmp_ptr ** container, uint32 size);

/**
 * Function creates queries based on collected data
 * @param container Pointer to the container
 * 
 * @return SQLSYNC_RETURN Returns SQLSYNC_SUCCESS
 */
SQLSYNC_RETURN c_cmp_create_synchronization_queries (c_cmp_ptr container, SQLSYNC_RETURN save);

SQLSYNC_RETURN sqlsync_query_create_INSERT_queries (c_cmp_ptr container, SQLSYNC_RETURN save);

SQLSYNC_RETURN sqlsync_query_create_DELETE_queries (c_cmp_ptr container, SQLSYNC_RETURN save);

SQLSYNC_RETURN sqlsync_query_create_UPDATE_queries (c_cmp_ptr container, SQLSYNC_RETURN save);

/**
 * Function prints informations that are stored in container,
 * summary, different, missing, additional rows etc.
 * @param container Pointer to the container
 */
void c_cmp_print_all_elems (c_cmp_ptr container);


#endif /* SQLSYNC_COMPARE_RESULTS_H */
