/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#include <stdarg.h>
#include <stdio.h>

#include "sqlsync_debug.h"


void
sqlsync_fprintf (char *file, uint32 line, SQLSYNC_RETURN silent, char *format, ...)
{

  /*  printf("silent %ld\n", silent); */
  if (silent & SQLSYNC_PRINT)
  {
    va_list args;

    va_start (args, format);

    if (file != NULL && line != 0)
      (void) fprintf (stdout, "PRINT (%s:%u)\t\t", file, line);

#if !defined(WINDOWS) && !defined(WIN32)
    if (silent & SQLSYNC_COLOR)
      (void) vfprintf (stdout, "\033[30;1m", 0);
#endif
    (void) vfprintf (stdout, format, args);
#if !defined(WINDOWS) && !defined(WIN32)
    if (silent & SQLSYNC_COLOR)
      (void) vfprintf (stdout, "\033[0m", 0);
#endif
    va_end (args);
  }
  else if (silent & SQLSYNC_DEBUG)
  {
    va_list args;

    va_start (args, format);

    if (file != NULL && line != 0)
      (void) fprintf (stderr, "DEBUG (%s:%u)\t\t", file, line);

#if !defined(WINDOWS) && !defined(WIN32)
    if (silent & SQLSYNC_COLOR)
      (void) vfprintf (stderr, "\033[31;1m", 0);
#endif
    (void) vfprintf (stderr, format, args);
#if !defined(WINDOWS) && !defined(WIN32)
    if (silent & SQLSYNC_COLOR)
      (void) vfprintf (stderr, "\033[0m", 0);
#endif
    va_end (args);
  }
  else if (silent & SQLSYNC_DEBUG_ALL)
  {
    /*    
       va_list args;
       va_start(args, format);
       vfprintf(stderr, "\033[32;1m", 0);
       vfprintf(stderr, format, args);
       vfprintf(stderr, "\033[0m", 0);
       va_end(args);
     */

  }
  fflush (stdout);
  fflush (stderr);

}

void
sqlsync_fwrite (char *file, uint32 line, SQLSYNC_RETURN silent, void *data, uint32 size)
{

  if (silent & SQLSYNC_PRINT)
  {
    if (file != NULL && line != 0)
      (void) fprintf (stdout, "WRITE PRINT (%s:%u)\n", file, line);

#if !defined(WINDOWS) && !defined(WIN32)
    if (silent & SQLSYNC_COLOR)
      (void) vfprintf (stdout, "\033[30;1m", 0);
#endif

    (void) fwrite (data, 1, size, stdout);
#if !defined(WINDOWS) && !defined(WIN32)
    if (silent & SQLSYNC_COLOR)
      (void) vfprintf (stdout, "\033[0m", 0);
#endif
  }
  else if (silent & SQLSYNC_DEBUG)
  {
    if (file != NULL && line != 0)
      (void) fprintf (stderr, "WRITE DEBUG (%s:%u)\t\t", file, line);

#if !defined(WINDOWS) && !defined(WIN32)
    if (silent & SQLSYNC_COLOR)
      (void) vfprintf (stderr, "\033[31;1m", 0);
#endif
    (void) fwrite (data, 1, size, stderr);
#if !defined(WINDOWS) && !defined(WIN32)
    if (silent & SQLSYNC_COLOR)
      (void) vfprintf (stderr, "\033[0m", 0);
#endif

  }
  else if (silent & SQLSYNC_DEBUG_ALL)
  {
    /*    
       va_list args;
       va_start(args, format);
       vfprintf(stderr, "\033[32;1m", 0);
       vfprintf(stderr, format, args);
       vfprintf(stderr, "\033[0m", 0);
       va_end(args);
     */

  }
  fflush (stdout);
  fflush (stderr);
}
