/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#ifndef SQLSYNC_DEBUG_H
#define SQLSYNC_DEBUG_H

#include "sqlsync_def.h"

enum
{
  SQLSYNC_PRINT = 1,
  SQLSYNC_SILENT = 2,
  SQLSYNC_DEBUG = 4,
  SQLSYNC_DEBUG_ALL = 8,
  SQLSYNC_COLOR = 16
};

#ifdef DEBUG
#define sqlsync_debug(format, args...) sqlsync_fprintf(__FILE__, __LINE__, SQLSYNC_DEBUG, format, ##args)
#define sqlsync_debug_details(format, args...) sqlsync_fprintf(__FILE__, __LINE__, SQLSYNC_DEBUG_ALL, format, ##args)
#define sqlsync_debug_println(string_elem) sqlsync_debug_write(SQLSYNC_DEBUG, string_elem->string, string_elem->length); \
                                            sqlsync_printf(SQLSYNC_DEBUG, "\n", 0)
#define sqlsync_debug_write(silent, data, size) sqlsync_fwrite(__FILE__, __LINE__, silent, data, size)
#endif

/**
 *  Prints string to screen (syntax the same as printf after silent parameter)
 * @param file File name to be printed
 * @param line Line number in file
 * @param silent 0 - string is printed otherwise nothing is printed
 * @param format printf format
 */
void sqlsync_fprintf (char *file, uint32 line, SQLSYNC_RETURN silent, char *format, ...);

void sqlsync_fwrite (char *file, uint32 line, SQLSYNC_RETURN silent, void *data, uint32 size);

#define sqlsync_write(silent, data, size) sqlsync_fwrite(0, 0, silent, data, size)

#define sqlsync_prints(silent, string_elem) sqlsync_fwrite(0, 0, silent, (string_elem)->string, (string_elem)->length)
#define sqlsync_println(silent, string_elem) sqlsync_fwrite(0, 0, silent, (string_elem)->string, (string_elem)->length); sqlsync_printf(silent, "\n", 0)
#define sqlsync_printf(silent, format, ...) sqlsync_fprintf(0, 0, silent, format, __VA_ARGS__)


#endif /* SQLSYNC_DEBUG_H */
