/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#ifndef SQLSYNC_DEF_H
#define SQLSYNC_DEF_H

typedef unsigned int uint32;
typedef signed int int32;
typedef signed short int int16;
typedef unsigned short int uint16;
typedef signed char int8;
typedef unsigned char uint8;

typedef void *sqlsync_info_ptr;

enum
{
  SQLSYNC_FALSE = 0,
  SQLSYNC_FAILED = 0,
  SQLSYNC_TRUE = 1,
  SQLSYNC_SUCCESS = 1,
  SQLSYNC_FAILED_CONNECT,
  SQLSYNC_FAILED_INIT,
  SQLSYNC_FAILED_DESTROY,
  SQLSYNC_FAILED_QUERY,
  SQLSYNC_FAILED_GET_TABLES,
  SQLSYNC_FAILED_GET_COLUMNS,
  SQLSYNC_FAILED_GET_ROW,
  SQLSYNC_FAILED_NULL,
  SQLSYNC_DISCARD,
  SQLSYNC_UNKNOWN
};

#define SQLSYNC_RETURN int8

#undef MIN
#undef MAX

#define MIN(a,b) ((a)<(b)? (a): (b))
#define MAX(a,b) ((a)<(b)? (b): (a))

#define CMP_RESULT(elem1, elem2) ( elem1 > elem2 ? 1 : (elem1 < elem2 ? -1 : 0))

#define SQLSYNC_IS_ERROR(ret) ((ret & (~1))!=0)

typedef enum _sqlsync_server
{
  SERVER_SRC = 0,
  SERVER_DST
} sqlsync_server;


typedef enum _sqlsync_server_type
{
  sqlsync_server_unknown,
  sqlsync_server_postgres,
  sqlsync_server_mysql,
  sqlsync_server_mssql,
  sqlsync_server_firebird,
  sqlsync_server_oracle
} sqlsync_server_type;

#ifndef NULL
#define NULL (void*)0
#endif

#ifndef SQLSYNC_STDLIB

void *std_malloc (uint32 size);
void std_free (void *ptr);
void *std_realloc (void *ptr, uint32 size);


uint32 std_strlen (char *ptr);
int8 std_strncmp (const char *str1, const char *str2, uint32 num);
char std_toupper (char c);
char *std_strchr (char *str, char character);

void *std_memcpy (void *destination, const void *source, uint32 num);
int8 std_memcmp (void *destination, const void *source, uint32 num);
void *std_memset (void *ptr, uint8 value, uint32 num);

void std_qsort (void *base, uint32 num, uint32 size,
                int (*comparator) (const void *, const void *));

int32 std_snprintf (char *str, uint32 count, const char *fmt, ...);

#else

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <stdarg.h>

#define std_malloc malloc
#define std_free free
#define std_realloc realloc

#define std_strlen strlen
#define std_strncmp strncmp
#define std_toupper toupper
#define std_strchr strchr

#define std_memcpy memcpy
#define std_memcmp memcmp

#define std_memset memset

#define std_qsort qsort
#define std_snprintf snprintf

#endif

#endif /* SQLSYNC_DEF_H */
