/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#include "data_types/sqlsync_data_string.h"
#include "data_types/sqlsync_data_misc.h"

#include "data_types/sqlsync_data_converter.h"

#include "sqlsync_query_data.h"


c_string_elem_ptr
sqlsync_query_create_assign (sqlsync_sqlserver * server, c_vector_columns_ptr columns,
                             c_vector_data_ptr values)
{
  uint32 i = 0;
  uint32 column_size = 0;

  c_column_elem_ptr column_data = NULL;
  c_data_elem_ptr value_data = NULL;
  c_string_elem_ptr tmp = NULL;
  uint8 question_marker = 0;

  c_string_elem_ptr assign = c_string_elem_empty ();

  if (columns == NULL)
    return NULL;

  if (values == NULL)
    question_marker = 1;

  column_size = c_vector_columns_size (columns);

  for (i = 0; i < column_size; i++)
  {
    column_data = c_vector_columns_pos (columns, i);

    if (column_data == NULL)
      continue;

    tmp = sqlsync_sqlserver_column_identifier (server, column_data->column);
    if (tmp == NULL)
      continue;

    c_string_elem_append_string (assign, tmp);
    c_string_elem_free (tmp);

    c_string_elem_append (assign, "=", 1);

    if (question_marker == 1)
    {
      tmp = c_string_elem_empty ();
      c_string_elem_append (tmp, "?", 1);
    }
    else
    {
      value_data = c_vector_data_pos (values, i);
      if (value_data == NULL)
      {
        tmp = c_string_elem_empty ();
        c_string_elem_append (tmp, "NULL", 4);
      }
      else
        tmp = sqlsync_sqlserver_data_to_string (server, value_data, SQLSYNC_TRUE);
    }

    c_string_elem_append_string (assign, tmp);

    c_string_elem_free (tmp);

    if (i < column_size - 1)
      c_string_elem_append (assign, ", ", 2);

  }
  return assign;
}


c_string_elem_ptr
sqlsync_query_create_where (sqlsync_sqlserver * server, c_vector_columns_ptr columns,
                            c_vector_data_ptr values)
{
  uint32 i = 0;
  uint32 column_size = 0;

  c_column_elem_ptr column_data = NULL;
  c_data_elem_ptr value_data = NULL;
  c_string_elem_ptr tmp = NULL;
  c_string_elem_ptr where = NULL;
  uint8 question_marker = 0;

  if (columns == NULL)
    return NULL;

  if (values == NULL)
    question_marker = 1;

  column_size = c_vector_columns_size (columns);

  where = c_string_elem_empty ();

  for (i = 0; i < column_size; i++)
  {
    column_data = c_vector_columns_pos (columns, i);

    tmp = sqlsync_sqlserver_column_identifier (server, column_data->column);

    if (tmp == NULL)
      continue;

/* TODO check that */
    if (column_data->data_type == SQLSYNC_SQLSERVER_DATA_CBLOB)
      c_string_elem_append (where, "TO_CHAR(", 8);

    c_string_elem_append_string (where, tmp);

    if (column_data->data_type == SQLSYNC_SQLSERVER_DATA_CBLOB)
      c_string_elem_append (where, ")", 1);


    c_string_elem_free (tmp);

    if (c_vector_data_pos (values, i) != NULL)
    {
       if (column_data->use_operator == 1)
         c_string_elem_append (where, " LIKE ", 6);
       else if(column_data->use_operator == 2)
         c_string_elem_append (where, "~=", 2);
       else
         c_string_elem_append (where, "=", 1);

    }
    else
      c_string_elem_append (where, " IS ", 4);


    if (question_marker == 1)
    {
      tmp = c_string_elem_empty ();
      c_string_elem_append (tmp, "?", 1);
    }
    else
    {
      value_data = c_vector_data_pos (values, i);
      if (value_data == NULL)
      {
        tmp = c_string_elem_empty ();
        c_string_elem_append (tmp, "NULL", 4);
      }
      else
        tmp = sqlsync_sqlserver_data_to_string (server, value_data, SQLSYNC_TRUE);

    }

    c_string_elem_append_string (where, tmp);
    c_string_elem_free (tmp);

    if (i < column_size - 1)
      c_string_elem_append (where, " AND ", 5);

  }

  return where;
}

c_string_elem_ptr
sqlsync_query_create_SET_part (sqlsync_sqlserver * server, c_vector_columns_ptr columns,
                               c_vector_data_ptr values)
{
  c_string_elem_ptr assign = NULL;
  c_string_elem_ptr set = NULL;

  assign = sqlsync_query_create_assign (server, columns, values);
  if (assign == NULL)
    return NULL;

  set = c_string_elem_empty ();

  c_string_elem_append (set, "SET ", 4);
  c_string_elem_append_string (set, assign);

  c_string_elem_free (assign);
  return set;
}

c_string_elem_ptr
sqlsync_query_create_WHERE_part (sqlsync_sqlserver * server,
                                 c_vector_columns_ptr columns, c_vector_data_ptr values)
{
  c_string_elem_ptr where = NULL;
  c_string_elem_ptr where_part = NULL;

  where = sqlsync_query_create_where (server, columns, values);
  if (where == NULL)
    return NULL;

  where_part = c_string_elem_empty ();

  c_string_elem_append (where_part, "WHERE ", 6);
  c_string_elem_append_string (where_part, where);

  c_string_elem_free (where);

  return where_part;
}

c_string_elem_ptr
sqlsync_query_create_INSERT_columns (sqlsync_sqlserver * server, c_vector_columns_ptr columns)
{
  uint32 i = 0;
  uint32 columns_size = 0;

  c_column_elem_ptr column_data = NULL;
  c_string_elem_ptr tmp = NULL;

  c_string_elem_ptr insert_cols = NULL;

  if (columns == NULL)
    return NULL;

  insert_cols = c_string_elem_empty ();

  columns_size = c_vector_columns_size (columns);

  for (i = 0; i < columns_size; i++)
  {
    column_data = c_vector_columns_pos (columns, i);
    tmp = sqlsync_sqlserver_column_identifier (server, column_data->column);

    if (tmp == NULL)
      continue;

    c_string_elem_append_string (insert_cols, tmp);

    c_string_elem_free (tmp);

    if (i < columns_size - 1)
      c_string_elem_append (insert_cols, ", ", 2);
  }

  return insert_cols;
}

c_string_elem_ptr
sqlsync_query_create_INSERT_values (sqlsync_sqlserver * server, c_vector_columns_ptr columns,
                                    c_vector_data_ptr values)
{
  uint32 i = 0;
  uint32 columns_size = 0;

  c_string_elem_ptr tmp = NULL;
  c_string_elem_ptr insert_values = NULL;
  c_data_elem_ptr value_data = NULL;
  uint8 question_marker = 0;

  if (values == NULL)
    question_marker = 1;

  columns_size = c_vector_strings_size (columns);

  insert_values = c_string_elem_empty ();

  for (i = 0; i < columns_size; i++)
  {
    if (question_marker == 1)
    {
      tmp = c_string_elem_empty ();
      c_string_elem_append (tmp, "?", 1);
    }
    else
    {
      value_data = c_vector_data_pos (values, i);
      if (value_data == NULL)
      {
        tmp = c_string_elem_empty ();
        c_string_elem_append (tmp, "NULL", 4);
      }
      else
        tmp = sqlsync_sqlserver_data_to_string (server, value_data, SQLSYNC_TRUE);
    }

    if (tmp == NULL)
      continue;

    c_string_elem_append_string (insert_values, tmp);
    c_string_elem_free (tmp);

    if (i < columns_size - 1)
      c_string_elem_append (insert_values, ", ", 2);
  }

  return insert_values;
}

c_string_elem_ptr
sqlsync_query_prepare_columns (sqlsync_sqlserver * server, c_vector_columns_ptr columns)
{
  c_column_elem_ptr column_data = NULL;
  c_string_elem_ptr tmp = NULL;
  uint32 columns_size = 0;
  uint32 i = 0;
  c_string_elem_ptr columns_idents = NULL;

  columns_size = c_vector_columns_size (columns);

  columns_idents = c_string_elem_empty ();

  for (i = 0; i < columns_size; i++)
  {
    column_data = c_vector_columns_pos (columns, i);

    if (column_data == NULL)
      continue;

    tmp = sqlsync_sqlserver_column_identifier (server, column_data->column);

    if (tmp == NULL)
      continue;

    c_string_elem_append_string (columns_idents, tmp);

    c_string_elem_free (tmp);

    if (i < columns_size - 1)
      c_string_elem_append (columns_idents, ", ", 2);
  }

  return columns_idents;
}

c_string_elem_ptr
sqlsync_create_SELECT_query (sqlsync_server_type type,
                             c_string_elem_ptr table, c_vector_columns_ptr columns)
{
  c_string_elem_ptr columns_idents = NULL;
  c_string_elem_ptr query = NULL;
  c_string_elem_ptr table_ident = NULL;
  sqlsync_sqlserver *server = sqlsync_sqlserver_create_dummy (type);

  if (server == NULL)
    return NULL;

  /* Create string with columns separated by commas */
  columns_idents = sqlsync_query_prepare_columns (server, columns);
  table_ident = sqlsync_sqlserver_column_identifier (server, table);


  query = c_string_elem_empty ();
  c_string_elem_append (query, "SELECT ", 7);
  c_string_elem_append_string (query, columns_idents);
  c_string_elem_append (query, " FROM ", 6);
  c_string_elem_append_string (query, table_ident);
  c_string_elem_append (query, ";", 1);


  c_string_elem_free (columns_idents);
  c_string_elem_free (table_ident);
  sqlsync_sqlserver_destroy (server);
  return query;
}


c_string_elem_ptr
sqlsync_create_INSERT_query (sqlsync_server_type type,
                             c_string_elem_ptr table, c_vector_columns_ptr columns,
                             c_vector_data_ptr values)
{
  c_string_elem_ptr query = NULL;
  c_string_elem_ptr table_ident = NULL;
  c_string_elem_ptr values_ident = NULL;
  c_string_elem_ptr columns_ident = NULL;
  sqlsync_sqlserver *server = sqlsync_sqlserver_create_dummy (type);

  if (server == NULL)
    return NULL;

  columns_ident = sqlsync_query_create_INSERT_columns (server, columns);
  table_ident = sqlsync_sqlserver_column_identifier (server, table);
  values_ident = sqlsync_query_create_INSERT_values (server, columns, values);


  query = c_string_elem_empty ();
  c_string_elem_append (query, "INSERT INTO ", 12);
  c_string_elem_append (query, table_ident->string, table_ident->length);
  c_string_elem_append (query, " (", 2);
  c_string_elem_append_string (query, columns_ident);
  c_string_elem_append (query, ") VALUES (", 10);
  c_string_elem_append_string (query, values_ident);
  c_string_elem_append (query, ");", 2);


  c_string_elem_free (table_ident);
  c_string_elem_free (columns_ident);
  c_string_elem_free (values_ident);
  sqlsync_sqlserver_destroy (server);
  return query;

}


c_string_elem_ptr
sqlsync_create_DELETE_query (sqlsync_server_type type,
                             c_string_elem_ptr table, c_vector_columns_ptr columns,
                             c_vector_data_ptr values)
{

  c_string_elem_ptr where_query = NULL;
  c_string_elem_ptr table_ident = NULL;
  c_string_elem_ptr where = NULL;
  sqlsync_sqlserver *server = sqlsync_sqlserver_create_dummy (type);

  if (server == NULL)
    return NULL;

  table_ident = sqlsync_sqlserver_column_identifier (server, table);
  where = sqlsync_query_create_WHERE_part (server, columns, values);


  where_query = c_string_elem_empty ();
  c_string_elem_append (where_query, "DELETE FROM ", 12);
  c_string_elem_append_string (where_query, table_ident);
  c_string_elem_append (where_query, " ", 1);
  c_string_elem_append_string (where_query, where);
  c_string_elem_append (where_query, ";", 1);

  c_string_elem_free (table_ident);
  c_string_elem_free (where);
  sqlsync_sqlserver_destroy (server);
  return where_query;
}


c_string_elem_ptr
sqlsync_create_UPDATE_query (sqlsync_server_type type,
                             c_string_elem_ptr table,
                             c_vector_columns_ptr set_columns,
                             c_vector_data_ptr set_values,
                             c_vector_columns_ptr where_columns, c_vector_data_ptr where_values)
{
  c_string_elem_ptr query_where = NULL;
  c_string_elem_ptr query_set = NULL;
  c_string_elem_ptr table_ident = NULL;
  c_string_elem_ptr query = NULL;
  sqlsync_sqlserver *server = sqlsync_sqlserver_create_dummy (type);

  if (server == NULL)
    return NULL;


  query_where = sqlsync_query_create_WHERE_part (server, where_columns, where_values);
  query_set = sqlsync_query_create_SET_part (server, set_columns, set_values);
  table_ident = sqlsync_sqlserver_column_identifier (server, table);


  query = c_string_elem_empty ();
  c_string_elem_append (query, "UPDATE ", 7);
  c_string_elem_append_string (query, table_ident);
  c_string_elem_append (query, " ", 1);
  c_string_elem_append_string (query, query_set);
  c_string_elem_append (query, " ", 1);
  c_string_elem_append_string (query, query_where);
  c_string_elem_append (query, ";", 1);

  c_string_elem_free (table_ident);
  c_string_elem_free (query_where);
  c_string_elem_free (query_set);
  sqlsync_sqlserver_destroy (server);
  return query;
}
