/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#include "sqlsync_def.h"

#ifndef SQLSYNC_STDLIB

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <stdarg.h>

#ifdef SQLSYNC_LOGMEM
typedef struct _sqlsync_memelem
{
  void * ptr;
  uint32 line;
  uint32 size;
  uint32 alloc_idx;
} sqlsync_memelem;

sqlsync_memelem sqlsync_mem[100000];
uint32 sqlsync_mem_size = 0;
uint32 sqlsync_mem_max_size = 0;
uint32 sqlsync_mem_cnt = 0;
uint32 sqlsync_mem_aidx = 0;
#endif /* SQLSYNC_LOGMEM */

void *
std_malloc (uint32 size)
{
  void * ptr = malloc (size);
#ifdef SQLSYNC_LOGMEM
  if (sqlsync_mem_cnt < sizeof(sqlsync_mem)/sizeof(sqlsync_mem[0]))
  {
    sqlsync_mem[sqlsync_mem_cnt].ptr = ptr;
  
    sqlsync_mem[sqlsync_mem_cnt].alloc_idx = sqlsync_mem_aidx++;
    sqlsync_mem[sqlsync_mem_cnt++].size = size;
    sqlsync_mem_size += size;
    sqlsync_mem_max_size = MAX(sqlsync_mem_size,sqlsync_mem_max_size);
  }
#endif /* SQLSYNC_LOGMEM */
  return ptr;
}

void
std_free (void *ptr)
{
#ifdef SQLSYNC_LOGMEM
  uint32 i = 0;
  int8 found = 0;
  for (; i< sqlsync_mem_cnt; i++)
  {
    if (sqlsync_mem[i].ptr == ptr)
    {
     
      sqlsync_mem_size -= sqlsync_mem[i].size;
      memmove(sqlsync_mem + i, sqlsync_mem + i + 1, sizeof(sqlsync_memelem) *(sqlsync_mem_cnt - i));
      memset(sqlsync_mem + sqlsync_mem_cnt - 1, 0, sizeof(sqlsync_memelem));
      sqlsync_mem_cnt--;
      found = 1;
      break;
    }

  }
  if (found == 0)
  {
    ptr = NULL;
  }
#endif /* SQLSYNC_LOGMEM */
  return free (ptr);
}

uint32
std_strlen (char *ptr)
{
  return strlen (ptr);
}

int8
std_strncmp (const char *str1, const char *str2, uint32 num)
{
  return strncmp (str1, str2, num);
}

void *
std_memcpy (void *destination, const void *source, uint32 num)
{
  return memcpy (destination, source, num);
}

int8
std_memcmp (void *destination, const void *source, uint32 num)
{
  return memcmp (destination, source, num);
}

void *
std_memset (void *ptr, uint8 value, uint32 num)
{
  return memset (ptr, value, num);
}

void *
std_realloc (void *ptr, uint32 size)
{
  void * r = realloc (ptr, size);
#ifdef SQLSYNC_LOGMEM
  int8 found = 0;
  uint32 i = 0;
  if (r != NULL)
    for (; i< sqlsync_mem_cnt; i++)
    {
      if (sqlsync_mem[i].ptr == ptr)
      {
  
        sqlsync_mem_size -= sqlsync_mem[i].size;
        sqlsync_mem_size += size;
        sqlsync_mem[i].ptr = r;
        sqlsync_mem[i].size = size;
        sqlsync_mem[i].alloc_idx = sqlsync_mem_aidx++;
        sqlsync_mem_max_size = MAX(sqlsync_mem_size,sqlsync_mem_max_size);
        found = 1;
        break;
      }
  
    }
#endif /* SQLSYNC_LOGMEM */

  return r;
}

char *
std_strchr (char *str, char character)
{
  return strchr (str, character);
}

int32
std_snprintf (char *str, uint32 count, const char *fmt, ...)
{
  int32 ret = 0;
  va_list args;

  va_start (args, fmt);
  ret = vsnprintf (str, count, fmt, args);
  va_end (args);
  return ret;
}


char
std_toupper (char c)
{
  return toupper (c);
}

void
std_qsort (void *base, uint32 num, uint32 size, int (*comparator) (const void *, const void *))
{
  qsort (base, num, size, comparator);
}

#endif
