/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#include "server/sqlsync_dbstruct.h"
#include "sqlsync_synchronization.h"
#include "sqlsync_user_internal.h"
#include "sqlsync_query_data.h"

#include "sqlsync_translation.h"        /* For translations _tr() */

#ifdef DEBUG
#include "sqlsync_debug.h"
#endif

#include "data_types/sqlsync_data_numeric.h"

#include "sqlsync_internal.h"
#include "server/sqlsync_query.h"

SQLSYNC_RETURN
sqlsync_create_synchronization_queries (c_cmp_ptr container, SQLSYNC_RETURN save)
{

  if (container == NULL)
    return SQLSYNC_FAILED_NULL;

  sqlsync_send_string_notification (SQLSYNC_USER_CREATE_QUERIES_BEGIN, container->table_name);

  sqlsync_send_string_notification (SQLSYNC_USER_CREATE_DIFF_QUERIES, container->table_name);
  (void) sqlsync_query_create_UPDATE_queries (container, save);

  sqlsync_send_string_notification (SQLSYNC_USER_CREATE_ADD_QUERIES, container->table_name);
  (void) sqlsync_query_create_DELETE_queries (container, save);

  sqlsync_send_string_notification (SQLSYNC_USER_CREATE_MISS_QUERIES, container->table_name);
  (void) sqlsync_query_create_INSERT_queries (container, save);

  sqlsync_send_string_notification (SQLSYNC_USER_CREATE_QUERIES_END, container->table_name);

  return SQLSYNC_SUCCESS;
}


SQLSYNC_RETURN
sqlsync_create_run_synchronization_queries (sqlsync_info_ptr info_ptr, c_cmp_ptr container)
{
  sqlsync_info *info = (sqlsync_info *) info_ptr;

  info->run_queries = SQLSYNC_TRUE;
  sqlsync_create_synchronization_queries (container, SQLSYNC_FALSE);
  info->run_queries = SQLSYNC_FALSE;

  return SQLSYNC_SUCCESS;
}


SQLSYNC_RETURN
sqlsync_send_user_notification_synchronization_handler (sqlsync_user_message message,
                                                        sqlsync_user_data * data)
{
  static uint8 send_once = 0;

  if (data == NULL || ((sqlsync_info *) data->user_ptr)->run_queries == SQLSYNC_FALSE)
    return SQLSYNC_SUCCESS;


  if (send_once == 0)
  {
    send_once = 1;

    switch (message)
    {
    case SQLSYNC_USER_CREATE_QUERIES_BEGIN:
      sqlsync_send_query_notification(SQLSYNC_USER_RUN_QUERIES_BEGIN, data->user_data.string, 
                                      data->user_data.query_info.query_idx,
                                      data->user_data.query_info.nr_of_queries);
        break;


      case SQLSYNC_USER_CREATE_DIFF_QUERY:
      case SQLSYNC_USER_CREATE_MISS_QUERY:
      case SQLSYNC_USER_CREATE_ADD_QUERY:
      {
        sqlsync_user_message m;

        switch(message)
        {
            case SQLSYNC_USER_CREATE_DIFF_QUERY:
              m = SQLSYNC_USER_RUN_DIFF_QUERY;
              break;
            case SQLSYNC_USER_CREATE_MISS_QUERY:
              m = SQLSYNC_USER_RUN_MISS_QUERY;
              break;
            case SQLSYNC_USER_RUN_ADD_QUERY:
              m = SQLSYNC_USER_RUN_ADD_QUERY;
              break;
            default:
              m = SQLSYNC_USER_RUN_QUERIES_ERROR;
              break;
        }
        sqlsync_send_query_notification(m, data->user_data.string, 
                                        data->user_data.query_info.query_idx,
                                        data->user_data.query_info.nr_of_queries);

        if (sqlsync_exec_query
            (((sqlsync_info *) data->user_ptr)->server[SERVER_DST],
             data->user_data.string) != SQLSYNC_SUCCESS)
        {
          extern void sqlsync_print_error (sqlsync_info_ptr info_ptr, sqlsync_server nr_server);

          sqlsync_print_error ((sqlsync_info_ptr) data->user_ptr, SERVER_DST);
          sqlsync_send_string_notification (SQLSYNC_USER_RUN_QUERIES_ERROR, NULL);
        }

        send_once = 0;
        return SQLSYNC_DISCARD;
        break;
      }

      case SQLSYNC_USER_CREATE_DIFF_QUERIES:
      {
        sqlsync_send_string_notification (SQLSYNC_USER_RUN_DIFF_QUERIES, data->user_data.string);
        break;
      }

      case SQLSYNC_USER_CREATE_MISS_QUERIES:
      {
        sqlsync_send_string_notification (SQLSYNC_USER_RUN_MISS_QUERIES, data->user_data.string);
        break;
      }

      case SQLSYNC_USER_CREATE_ADD_QUERIES:
      {
        sqlsync_send_string_notification (SQLSYNC_USER_RUN_ADD_QUERIES, data->user_data.string);
        break;
      }

      case SQLSYNC_USER_CREATE_QUERIES_END:
      {
        sqlsync_send_string_notification (SQLSYNC_USER_RUN_QUERIES_END, data->user_data.string);
        break;
      }

      default:
        break;
    }

  }
  send_once = 0;

  return SQLSYNC_SUCCESS;
}


SQLSYNC_RETURN
sqlsync_query_create_INSERT_queries (c_cmp_ptr container, SQLSYNC_RETURN save)
{
  uint32 i = 0;
  uint32 grid_size = 0;
  c_string_elem_ptr query = NULL;

  container->sql_query_miss = c_vector_strings_init (container->row_miss->size);

  grid_size = c_grid_data_size (container->row_miss);

  for (i = 0; i < grid_size; i++)
  {

    query = sqlsync_create_INSERT_query (container->type,
                                         container->table_name,
                                         container->columns,
                                         c_grid_data_pos (container->row_miss, i));

    sqlsync_send_query_notification (SQLSYNC_USER_CREATE_MISS_QUERY, query, 
                                     i, grid_size);

    if (save == SQLSYNC_TRUE)
      (void) c_vector_strings_add_last (container->sql_query_miss, query);
    else
      c_string_elem_free (query);

  }
  return SQLSYNC_SUCCESS;
}

SQLSYNC_RETURN
sqlsync_query_create_DELETE_queries (c_cmp_ptr container, SQLSYNC_RETURN save)
{
  uint32 j = 0;
  uint32 grid_size = 0;

  uint8 primary_keys = 0;

  c_string_elem_ptr query = NULL;
  c_vector_columns_ptr columns = NULL;
  c_vector_data_ptr values = NULL;

  grid_size = c_grid_data_size (container->row_add);

  primary_keys = (container->primary_keys != NULL &&
                  c_vector_int32_size (container->primary_keys_pos) != 0);


  container->sql_query_add = c_vector_strings_init (grid_size);


  for (j = 0; j < grid_size; j++)
  {

    if (primary_keys != 0)
    {

      uint32 i = 0;
      uint32 primary_keys_size = 0;
      c_vector_data_ptr primary_values = NULL;

      primary_keys_size = c_vector_columns_size (container->primary_keys);
      primary_values = c_vector_data_init (primary_keys_size);

      for (i = 0; i < primary_keys_size; i++)
      {
        int primary_pos =
          ((c_int32_elem_ptr) c_vector_int32_pos (container->primary_keys_pos, i))->value;
        c_data_elem_ptr value =
          c_vector_data_pos (c_grid_data_pos (container->row_add, j), primary_pos);

        c_vector_data_add_last_copy (primary_values, value);
      }

      columns = container->primary_keys;
      values = primary_values;

    }
    else
    {
      columns = container->columns;
      values = c_grid_data_pos (container->row_add, j);
    }

    query = sqlsync_create_DELETE_query (container->type, container->table_name, columns, values);

    sqlsync_send_query_notification (SQLSYNC_USER_CREATE_ADD_QUERY, query, 
                                     j, grid_size);

    if (save == SQLSYNC_TRUE)
      (void) c_vector_strings_add_last (container->sql_query_add, query);
    else
      c_string_elem_free (query);

    if (primary_keys != 0)
      c_vector_data_destroy (values);

  }
  return SQLSYNC_SUCCESS;
}

SQLSYNC_RETURN
sqlsync_query_create_UPDATE_queries (c_cmp_ptr container, SQLSYNC_RETURN save)
{
  uint32 j = 0;
  uint32 grid_size = 0;
  uint8 primary_keys = 0;


  c_vector_columns_ptr where_columns = NULL;
  c_vector_data_ptr where_values = NULL;
  c_string_elem_ptr query = NULL;

  grid_size = c_grid_data_size (container->row_diff_src);

  container->sql_query_diff = c_vector_strings_init (grid_size);

  primary_keys = (container->primary_keys != NULL &&
                  c_vector_int32_size (container->primary_keys_pos) != 0);


  for (j = 0; j < grid_size; j++)
  {
    if (primary_keys != 0)
    {

      uint32 i = 0;
      uint32 primary_keys_size = 0;
      c_vector_data_ptr primary_values = NULL;

      primary_keys_size = c_vector_columns_size (container->primary_keys);
      primary_values = c_vector_data_init (primary_keys_size);

      for (i = 0; i < primary_keys_size; i++)
      {
        int primary_pos = (c_vector_int32_pos (container->primary_keys_pos, i))->value;

        c_data_elem_ptr value =
          c_vector_data_pos (c_grid_data_pos (container->row_diff_dst, j), primary_pos);
        c_vector_data_add_last_copy (primary_values, value);
      }

      where_columns = container->primary_keys;
      where_values = primary_values;

    }
    else
    {
      where_columns = container->columns;
      where_values = c_grid_data_pos (container->row_diff_dst, j);
    }


    query = sqlsync_create_UPDATE_query (container->type, container->table_name,
                                         container->columns,
                                         c_grid_data_pos (container->row_diff_src, j),
                                         where_columns, where_values);

    sqlsync_send_query_notification (SQLSYNC_USER_CREATE_DIFF_QUERY, query, 
                                     j, grid_size);



    if (save == SQLSYNC_TRUE)
      (void) c_vector_strings_add_last (container->sql_query_diff, query);
    else
      c_string_elem_free (query);

    if (primary_keys != 0)
      c_vector_data_destroy (where_values);

  }
  return SQLSYNC_SUCCESS;
}

#ifdef SQLSYNC_DEPRECATED

SQLSYNC_RETURN
sqlsync_print_synchronization_queries (c_cmp_ptr container)
{
  if (container == NULL)
    return SQLSYNC_FAILED_NULL;

  c_strings_all_elems (container->sql_query_diff, c_string_elem_print, NULL);
  c_strings_all_elems (container->sql_query_add, c_string_elem_print, NULL);
  c_strings_all_elems (container->sql_query_miss, c_string_elem_print, NULL);

  return SQLSYNC_SUCCESS;
}

SQLSYNC_RETURN
sqlsync_run_selected_synchronization_queries (sqlsync_info_ptr info_ptr,
                                              enum sqlsync_server_type type, c_strings_ptr queries)
{
  uint32 j = 0;
  SQLSYNC_RETURN ret = 0;

  sqlsync_info *info = (sqlsync_info *) info_ptr;

  uint32 queries_size = c_strings_size (queries);

  c_string_elem_ptr query = NULL;

  for (j = 0; j < queries_size; j++)
  {
    query = c_strings_pos (queries, j);

    if (query == NULL)
      continue;

    ret = sqlsync_exec_query (info, type, query);

    if (ret != SQLSYNC_SUCCESS)
      return ret;
  }
  return SQLSYNC_SUCCESS;
}

SQLSYNC_RETURN
sqlsync_run_synchronization_queries (sqlsync_info_ptr info_ptr, c_cmp_ptr container)
{

  sqlsync_info *info = (sqlsync_info *) info_ptr;

  if (info == NULL && container == NULL)
    return SQLSYNC_FAILED_NULL;

  sqlsync_send_string_notification (SQLSYNC_USER_RUN_QUERIES_BEGIN, container->table_name);

  sqlsync_send_string_notification (SQLSYNC_USER_RUN_DIFF_QUERIES, container->table_name);
  if (sqlsync_run_selected_synchronization_queries (info, SERVER_DST, container->sql_query_diff) !=
      SQLSYNC_SUCCESS)
    sqlsync_send_string_notification (SQLSYNC_USER_RUN_QUERIES_ERROR, container->table_name);

  sqlsync_send_string_notification (SQLSYNC_USER_RUN_ADD_QUERIES, container->table_name);
  if (sqlsync_run_selected_synchronization_queries (info, SERVER_DST, container->sql_query_add) !=
      SQLSYNC_SUCCESS)
    sqlsync_send_string_notification (SQLSYNC_USER_RUN_QUERIES_ERROR, container->table_name);

  sqlsync_send_string_notification (SQLSYNC_USER_RUN_MISS_QUERIES, container->table_name);
  if (sqlsync_run_selected_synchronization_queries (info, SERVER_DST, container->sql_query_miss) !=
      SQLSYNC_SUCCESS)
    sqlsync_send_string_notification (SQLSYNC_USER_RUN_QUERIES_ERROR, container->table_name);

  sqlsync_send_string_notification (SQLSYNC_USER_RUN_QUERIES_END, container->table_name);


  return SQLSYNC_SUCCESS;
}
#endif

SQLSYNC_RETURN
sqlsync_send_table_summary (c_cmp_ptr container)
{
  if (container == NULL)
    return SQLSYNC_FAILED_NULL;

  c_cmp_print_all_elems (container);

  return SQLSYNC_SUCCESS;
}


SQLSYNC_RETURN
sqlsync_run_SELECT_queries (sqlsync_info_ptr info_ptr, sqlsync_table_columns * tc_src, sqlsync_table_columns * tc_dst)
{

  SQLSYNC_RETURN ret;
  c_string_elem_ptr query = NULL;

  sqlsync_info *info = (sqlsync_info *) info_ptr;

  query =
    sqlsync_create_SELECT_query (sqlsync_sqlserver_type (info->server[SERVER_SRC]), tc_src->table, tc_src->columns);

  /* Run query on first database */
#ifdef DEBUG
  sqlsync_debug ("Running source query\n");
#endif

  ret = sqlsync_exec_query (info->server[SERVER_SRC], query);

#ifdef DEBUG
  sqlsync_debug ("Generated SELECT query for source table\n", 0);
  sqlsync_debug_println (query);
#endif
  c_string_elem_free (query);
  query = NULL;

  /* If error occured print error info but continue */
  if (SQLSYNC_IS_ERROR (ret))
  {

    c_string_elem_ptr error_msg = c_string_elem_empty ();

    c_string_elem_append_cstring (error_msg, _tr ("Error occured while running SELECT query "
                                                  "on source table,\n probably "
                                                  "generated query is corrupted for "));
    c_string_elem_append_string (error_msg, tc_src->table);
    c_string_elem_append_cstring (error_msg, _tr (" table"));

    sqlsync_send_error_notification (error_msg);
    c_string_elem_free (error_msg);

    extern void sqlsync_print_error (sqlsync_info_ptr info_ptr, sqlsync_server nr_server);

    sqlsync_print_error (info, SERVER_SRC);


    return SQLSYNC_FALSE;
  }

  query =
    sqlsync_create_SELECT_query (sqlsync_sqlserver_type (info->server[SERVER_DST]), tc_dst->table, tc_dst->columns);

  /* Run query on second database */
  ret = sqlsync_exec_query (info->server[SERVER_DST], query);

#ifdef DEBUG
  sqlsync_debug ("Generated SELECT query for destination table\n", 0);
  sqlsync_debug_println (query);
#endif


  c_string_elem_free (query);
  query = NULL;

  /* If error occured print error info but continue */
  if (SQLSYNC_IS_ERROR (ret))
  {

    c_string_elem_ptr error_msg = c_string_elem_empty ();

    c_string_elem_append_cstring (error_msg, _tr ("Error occured while running SELECT query "
                                                  "on destination table,\n probably "
                                                  "generated query is corrupted for "));
    c_string_elem_append_string (error_msg, tc_dst->table);
    c_string_elem_append_cstring (error_msg, _tr (" table"));

    sqlsync_send_error_notification (error_msg);
    c_string_elem_free (error_msg);

    extern void sqlsync_print_error (sqlsync_info_ptr info_ptr, sqlsync_server nr_server);

    sqlsync_print_error (info, SERVER_DST);


    return SQLSYNC_FALSE;
  }

  return SQLSYNC_SUCCESS;
}
