/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#include "sqlsync_user_notification.h"
#include "sqlsync_user_internal.h"

#include "data_types/sqlsync_data_vector.h"
#include "data_types/sqlsync_data_misc.h"

c_notification_elem_ptr
c_notification_elem_copy (c_notification_elem_ptr elem)
{
  c_notification_elem *src = NULL;

  if (elem == NULL)
    return NULL;

  c_notification_elem_ptr copy = (c_notification_elem *) std_malloc (sizeof (c_notification_elem));

  if (copy == NULL)
    return NULL;

  src = (c_notification_elem *) elem;

  copy->_type = c_notification_elem_type;
  copy->f_callback = src->f_callback;
  copy->user_ptr = src->user_ptr;

  return copy;
}

void
c_notification_elem_free (c_notification_elem_ptr elem)
{
  std_free (elem);
}

c_notification_elem_Vtbl *
c_notification_elem_create_Vtbl (uint32 vtbl_size)
{
  c_notification_elem_Vtbl *vtbl = std_malloc (vtbl_size);

  if (vtbl == NULL)
    return NULL;

  vtbl->elem_empty = NULL;
  vtbl->elem_free = c_notification_elem_free;
  vtbl->elem_cmp = NULL;
  vtbl->elem_copy = c_notification_elem_copy;
  vtbl->get_type = NULL;

  return vtbl;
}


static c_vector_data_ptr notification_list = NULL;

void
sqlsync_send_user_notification_init ()
{
  notification_list = c_vector_data_init (2);
}

void
sqlsync_send_user_notification_free ()
{
  c_vector_data_destroy (notification_list);
}


void
sqlsync_user_notification_register (sqlsync_user_notification f_callback, void *user_ptr)
{
  c_notification_elem elem;

  elem._type = c_notification_elem_type;
  elem.f_callback = f_callback;
  elem.user_ptr = user_ptr;

  c_vector_data_add_last_copy (notification_list, (c_data_elem_ptr) & elem);

}

void
sqlsync_send_user_notification (sqlsync_user_message message, sqlsync_user_data * data)
{
  uint32 i = 0;
  uint8 ret = 0;
  c_notification_elem *elem = NULL;

  for (i = 0; i < c_vector_data_size ((c_vector_data_ptr) notification_list); i++)
  {
    elem = (c_notification_elem *) c_vector_data_pos ((c_vector_data_ptr) notification_list, i);


    if (data != NULL)
      data->user_ptr = elem->user_ptr;

    if (elem != NULL && elem->f_callback != NULL)
      ret = elem->f_callback (message, data);

    if (ret == SQLSYNC_DISCARD)
      break;
  }

}

void
sqlsync_send_error_notification (c_string_elem_ptr error)
{
  sqlsync_user_data error_user_data;

  error_user_data.user_data.string = error;
  sqlsync_send_user_notification (SQLSYNC_USER_ERROR_MESSAGE, &error_user_data);

}

void
sqlsync_send_error_notification2 (char *data)
{
  c_string_elem elem;

  elem._type = c_string_elem_type;
  elem.string = data;
  elem.length = std_strlen (data);
  elem.reserved_length = elem.length;

  sqlsync_send_error_notification (&elem);
}

void
sqlsync_send_info_notification (c_string_elem_ptr info)
{
  sqlsync_user_data info_user_data;

  info_user_data.user_data.string = info;

  sqlsync_send_user_notification (SQLSYNC_USER_INFO_MESSAGE, &info_user_data);

}

void
sqlsync_send_info_notification2 (char *data)
{
  c_string_elem elem;

  elem._type = c_string_elem_type;
  elem.string = data;
  elem.length = std_strlen (data);
  elem.reserved_length = elem.length;
  sqlsync_send_info_notification (&elem);

}


void
sqlsync_send_string_notification (sqlsync_user_message message, c_string_elem_ptr data)
{
  sqlsync_user_data info_user_data;

  std_memset(&info_user_data, 0, sizeof(sqlsync_user_data));
  info_user_data.user_data.string = data;

  sqlsync_send_user_notification (message, &info_user_data);

}

void sqlsync_send_query_notification (sqlsync_user_message message, 
                                      c_string_elem_ptr query, uint32 idx, uint32 size)
{
  sqlsync_user_data info_user_data;

  info_user_data.user_data.string = query;
  info_user_data.user_data.query_info.query_idx = idx;
  info_user_data.user_data.query_info.nr_of_queries = size;

  sqlsync_send_user_notification (message, &info_user_data);
}
