/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#include "sqlsync_lib_handle.h"
#include "sqlsync_debug.h"
#include "sqlsync_translation.h"

#include "data_types/sqlsync_data_print.h"

#include <stdlib.h>

void
sqlsync_lib_user_handler_create_queries (sqlsync_user_message message, sqlsync_user_data * data)
{
  lib_handler_data *handler_data = (lib_handler_data *) data->user_ptr;

  data = NULL;
  switch (message)
  {
    case SQLSYNC_USER_CREATE_QUERIES_BEGIN:
    {
      if (handler_data->silent == SQLSYNC_SILENT)
        break;

      sqlsync_printf (SQLSYNC_PRINT, _tr ("Creating synchronization queries\n"), 0);
      break;
    }

    case SQLSYNC_USER_CREATE_DIFF_QUERIES:
    {

      break;
    }
    case SQLSYNC_USER_CREATE_MISS_QUERIES:
    {

      break;
    }
    case SQLSYNC_USER_CREATE_ADD_QUERIES:
    {

      break;
    }

    case SQLSYNC_USER_CREATE_QUERIES_END:
    {
      if (handler_data->silent == SQLSYNC_SILENT)
        break;

      sqlsync_printf (SQLSYNC_PRINT, _tr ("Creating synchronization queries completed\n"), 0);
      break;
    }
    default:
      break;
  }

}

void
sqlsync_lib_user_handler_run_queries (sqlsync_user_message message, sqlsync_user_data * data)
{
  lib_handler_data *handler_data = (lib_handler_data *) data->user_ptr;

  switch (message)
  {
    case SQLSYNC_USER_RUN_QUERIES_BEGIN:
    {
      if (handler_data->silent == SQLSYNC_SILENT)
        break;

      sqlsync_printf (SQLSYNC_PRINT, _tr ("Running synchronization queries\n"), 0);
      break;
    }

    case SQLSYNC_USER_RUN_ADD_QUERIES:
    {
      if (handler_data->silent == SQLSYNC_SILENT)
        break;

      sqlsync_printf (SQLSYNC_PRINT, _tr ("Running DELETE synchronization queries\n"), 0);

      break;
    }

    case SQLSYNC_USER_RUN_MISS_QUERIES:
    {
      if (handler_data->silent == SQLSYNC_SILENT)
        break;

      sqlsync_printf (SQLSYNC_PRINT, _tr ("Running INSERT synchronization queries\n"), 0);
      break;
    }

    case SQLSYNC_USER_RUN_DIFF_QUERIES:
    {
      if (handler_data->silent == SQLSYNC_SILENT)
        break;

      sqlsync_printf (SQLSYNC_PRINT, _tr ("Running UPDATE synchronization queries\n"), 0);
      break;
    }

    case SQLSYNC_USER_RUN_DIFF_QUERY:
    case SQLSYNC_USER_RUN_MISS_QUERY:
    case SQLSYNC_USER_RUN_ADD_QUERY:
    {
      if (handler_data->silent == SQLSYNC_SILENT)
        break;

      sqlsync_printf (SQLSYNC_PRINT, _tr ("Running synchronization query (%d/%d)\n"), 
                      data->user_data.query_info.query_idx,
                      data->user_data.query_info.nr_of_queries
                      );
      break;
    }

    case SQLSYNC_USER_RUN_QUERIES_ERROR:
    {
      sqlsync_printf (SQLSYNC_PRINT, _tr ("Running synchronization queries (error)\n"), 0);
      break;
    }

    case SQLSYNC_USER_RUN_QUERIES_END:
    {
      if (handler_data->silent == SQLSYNC_SILENT)
        break;

      sqlsync_printf (SQLSYNC_PRINT, _tr ("Running synchronization queries completed\n"), 0);
      break;
    }
    default:
      break;
  }

}

void
sqlsync_lib_user_handler_print_row_elem (sqlsync_user_message message, sqlsync_user_data * data)
{
  lib_handler_data *handler_data = (lib_handler_data *) data->user_ptr;

  switch (message)
  {
    case SQLSYNC_USER_DIFF_SRC_ROW_ELEM:
    {
      if (data->user_data.row_elem.position == 0)
        sqlsync_printf (SQLSYNC_PRINT, _tr ("[SRC] "), 0);
      break;
    }
    case SQLSYNC_USER_DIFF_DST_ROW_ELEM:
    {
      if (data->user_data.row_elem.position == 0)
        sqlsync_printf (SQLSYNC_PRINT, _tr ("[DST] "), 0);
      break;
    }

    case SQLSYNC_USER_MISS_ROW_ELEM:
    case SQLSYNC_USER_ADD_ROW_ELEM:
    {
      break;
    }
    default:
      break;
  }
  c_string_elem_print_in_row (data->user_data.row_elem.elem,
                              data->user_data.row_elem.position, handler_data->columns);

  if (data->user_data.row_elem.position == c_vector_columns_size (handler_data->columns) - 1)
    sqlsync_printf (SQLSYNC_PRINT, "\n", 0);
}

void
sqlsync_lib_user_handler_print_query (sqlsync_user_message message, sqlsync_user_data * data)
{
  switch (message)
  {
    case SQLSYNC_USER_CREATE_DIFF_QUERY:
    case SQLSYNC_USER_CREATE_MISS_QUERY:
    case SQLSYNC_USER_CREATE_ADD_QUERY:
    {
      sqlsync_println (SQLSYNC_PRINT, data->user_data.string);
      break;
    }
    default:
      break;
  }
}

void
sqlsync_lib_user_print_summary (sqlsync_user_message message, sqlsync_user_data * data)
{

  switch (message)
  {
    case SQLSYNC_USER_SUMMARY:
    {

      sqlsync_printf (SQLSYNC_PRINT, _tr ("[BEGIN] -----------------------------\n"), 0);
      sqlsync_printf (SQLSYNC_PRINT, _tr ("[SUMMARY] ---------------------------\n"), 0);
      sqlsync_printf (SQLSYNC_PRINT, _tr ("Table "), 0);
      sqlsync_prints (SQLSYNC_PRINT, data->user_data.summary.table);
      sqlsync_printf (SQLSYNC_PRINT, "\n", 0);

      sqlsync_printf (SQLSYNC_PRINT, _tr ("Number of columns %ld\n"),
                      data->user_data.summary.nr_columns);
      sqlsync_printf (SQLSYNC_PRINT, _tr ("Equal rows %ld\n"),
                      data->user_data.summary.nr_equal_rows);

      sqlsync_printf (SQLSYNC_PRINT, _tr ("Different rows %ld\n"),
                      data->user_data.summary.nr_diff_rows);

      sqlsync_printf (SQLSYNC_PRINT, _tr ("Missing rows %ld\n"),
                      data->user_data.summary.nr_miss_rows);

      sqlsync_printf (SQLSYNC_PRINT, _tr ("Additional rows %ld\n"),
                      data->user_data.summary.nr_add_rows);

      sqlsync_printf (SQLSYNC_PRINT, _tr ("[COLUMNS] ---------------------------\n"), 0);
      c_vector_columns_all_elems (data->user_data.summary.columns, c_column_elem_print, NULL);
      ((lib_handler_data *) data->user_ptr)->columns = data->user_data.summary.columns;
      sqlsync_printf (SQLSYNC_PRINT, "\n", 0);
      break;
    }

    case SQLSYNC_USER_SUMMARY_DIFF:
    {
      sqlsync_printf (SQLSYNC_PRINT, _tr ("[DIFF] ------------------------------\n"), 0);
      break;
    }

    case SQLSYNC_USER_SUMMARY_MISS:
    {
      sqlsync_printf (SQLSYNC_PRINT, _tr ("[MISSING] ---------------------------\n"), 0);
      break;
    }

    case SQLSYNC_USER_SUMMARY_ADD:
    {
      sqlsync_printf (SQLSYNC_PRINT, _tr ("[ADDITIONAL] ------------------------\n"), 0);
      break;
    }
    case SQLSYNC_USER_SUMMARY_END:
    {
      sqlsync_printf (SQLSYNC_PRINT, _tr ("[END] -------------------------------\n"), 0);
      break;
    }
    default:
      break;
  }

}

SQLSYNC_RETURN
sqlsync_lib_user_handler (sqlsync_user_message message, sqlsync_user_data * data)
{
  lib_handler_data *handler_data = NULL;

  if (data != NULL)
    handler_data = (lib_handler_data *) data->user_ptr;

  switch (message)
  {
    case SQLSYNC_USER_ERROR_MESSAGE:
    case SQLSYNC_USER_INFO_MESSAGE:
    {

#ifdef DEBUG
      sqlsync_debug_println (data->user_data.string);
#endif

      if (message != SQLSYNC_USER_ERROR_MESSAGE && handler_data->silent == SQLSYNC_SILENT)
        break;

      sqlsync_println (SQLSYNC_PRINT, data->user_data.string);
      break;
    }

    case SQLSYNC_USER_DSN_INFO:
    {

      sqlsync_printf (SQLSYNC_PRINT, _tr ("(DSN "), 0);
      sqlsync_prints (SQLSYNC_PRINT, data->user_data.dsn_info.dsn);
      sqlsync_printf (SQLSYNC_PRINT, _tr (") Description("), 0);
      sqlsync_prints (SQLSYNC_PRINT, data->user_data.dsn_info.description);
      sqlsync_printf (SQLSYNC_PRINT, _tr (")\n"), 0);
      break;
    }

    case SQLSYNC_USER_SUMMARY:
    case SQLSYNC_USER_SUMMARY_END:
    case SQLSYNC_USER_SUMMARY_DIFF:
    case SQLSYNC_USER_SUMMARY_MISS:
    case SQLSYNC_USER_SUMMARY_ADD:
    {
      sqlsync_lib_user_print_summary (message, data);
      break;
    }

    case SQLSYNC_USER_CONNECTION_FAILED:
    {
      if (handler_data->silent == SQLSYNC_SILENT)
        break;

      sqlsync_printf (SQLSYNC_PRINT, _tr ("Failed to connect (DSN "), 0);
      sqlsync_prints (SQLSYNC_PRINT, data->user_data.string);
      sqlsync_printf (SQLSYNC_PRINT, " )\n", 0);

      break;
    }

    case SQLSYNC_USER_DIFF_SRC_ROW_ELEM:
    case SQLSYNC_USER_DIFF_DST_ROW_ELEM:
    case SQLSYNC_USER_MISS_ROW_ELEM:
    case SQLSYNC_USER_ADD_ROW_ELEM:
    {
      sqlsync_lib_user_handler_print_row_elem (message, data);
      break;
    }

    case SQLSYNC_USER_CREATE_DIFF_QUERY:
    case SQLSYNC_USER_CREATE_MISS_QUERY:
    case SQLSYNC_USER_CREATE_ADD_QUERY:
    {
      sqlsync_lib_user_handler_print_query (message, data);
      break;
    }


    case SQLSYNC_USER_COMPARE_TABLES_BEGIN:
    {
      if (handler_data->silent == SQLSYNC_SILENT)
        break;

      sqlsync_printf (SQLSYNC_PRINT, _tr ("Comparing tables "), 0);
      sqlsync_prints (SQLSYNC_PRINT, data->user_data.string);
      sqlsync_printf (SQLSYNC_PRINT, "\n", 0);
      break;
    }

    case SQLSYNC_USER_COMPARE_TABLES_END:
    {
      if (handler_data->silent == SQLSYNC_SILENT)
        break;

      sqlsync_printf (SQLSYNC_PRINT, _tr ("Comparing tables "), 0);
      sqlsync_prints (SQLSYNC_PRINT, data->user_data.string);
      sqlsync_printf (SQLSYNC_PRINT, _tr (" completed\n"), 0);
      break;
    }

    case SQLSYNC_USER_CREATE_QUERIES_BEGIN:
    case SQLSYNC_USER_CREATE_DIFF_QUERIES:
    case SQLSYNC_USER_CREATE_MISS_QUERIES:
    case SQLSYNC_USER_CREATE_ADD_QUERIES:
    case SQLSYNC_USER_CREATE_QUERIES_END:
    {
      sqlsync_lib_user_handler_create_queries (message, data);
      break;
    }

    case SQLSYNC_USER_RUN_QUERIES_BEGIN:
    case SQLSYNC_USER_RUN_DIFF_QUERIES:
    case SQLSYNC_USER_RUN_MISS_QUERIES:
    case SQLSYNC_USER_RUN_ADD_QUERIES:
    case SQLSYNC_USER_RUN_QUERIES_END:
    case SQLSYNC_USER_RUN_DIFF_QUERY:
    case SQLSYNC_USER_RUN_MISS_QUERY:
    case SQLSYNC_USER_RUN_ADD_QUERY:
    {
      sqlsync_lib_user_handler_run_queries (message, data);

      break;
    }

    default:
      break;
  }

  return SQLSYNC_SUCCESS;
}
