/****************************************************************************
**
** SqlSync - Comparing and synchronizing data in SQL databases
** Copyright (c) 2006-2008, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=sqlsync
** Project authors:  Pawel 'pawel_p' Pluciennik
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#include <string.h>             /* For memset and strlen */
#include <stdlib.h>             /* For malloc and free */

//#define SQLSYNC_PARAMS_FILE
#ifdef SQLSYNC_PARAMS_FILE
#include <stdio.h>
#endif

#include <getopt.h>

#include "sqlsync.h"

#include "sqlsync_help.h"

#include "sqlsync_translation.h"
#include "sqlsync_lib_handle.h"

#include "data_types/sqlsync_data_string.h"

#include "sqlsync_debug.h"

#define NOT_EMPTY_STRING(string) (string[0] != 0)

typedef struct
{
  sqlsync_info_ptr info;

  c_vector_strings_ptr tables;
  c_grid_columns_ptr columns;

  c_string_elem_ptr dsn_src;
  c_string_elem_ptr dsn_dst;
  c_string_elem_ptr user_src;
  c_string_elem_ptr user_dst;
  c_string_elem_ptr pass_src;
  c_string_elem_ptr pass_dst;
  c_string_elem_ptr query_file;

  short run_query_file;
  short show_summary;
  short show_queries;
  short run_queries;
  short silent;
  int8 cs;
  lib_handler_data handler_data;
} sqlsync_prog;

sqlsync_prog *
sqlsync_prog_create ()
{
  SQLSYNC_RETURN ret;
  sqlsync_prog *prog;

  prog = malloc (sizeof (sqlsync_prog));
  if (prog == NULL)
    return NULL;

  memset (prog, 0, sizeof (sqlsync_prog));
  prog->info = NULL;

  prog->dsn_src = c_string_elem_empty ();
  prog->dsn_dst = c_string_elem_empty ();

  prog->user_src = c_string_elem_empty ();
  prog->pass_src = c_string_elem_empty ();

  prog->user_dst = c_string_elem_empty ();
  prog->pass_dst = c_string_elem_empty ();

  prog->query_file = c_string_elem_empty ();

  prog->run_query_file = 0;
  prog->show_summary = 0;
  prog->show_queries = 0;
  prog->run_queries = 0;
  prog->silent = 0;
  prog->cs = 0;

  sqlsync_translation_init ();

  prog->info = sqlsync_init (&ret);

  return prog;

}

SQLSYNC_RETURN
sqlsync_prog_destroy (sqlsync_prog ** prog)
{
  SQLSYNC_RETURN ret = SQLSYNC_TRUE;

  c_string_elem_free ((*prog)->dsn_src);
  c_string_elem_free ((*prog)->dsn_dst);

  c_string_elem_free ((*prog)->user_src);
  c_string_elem_free ((*prog)->pass_src);

  c_string_elem_free ((*prog)->user_dst);
  c_string_elem_free ((*prog)->pass_dst);
  c_string_elem_free ((*prog)->query_file);

  c_vector_strings_destroy ((*prog)->tables);
  c_grid_columns_destroy ((*prog)->columns);

  ret = sqlsync_destroy ((*prog)->info);

  free (*prog);
  *prog = NULL;

  return ret;
}


int
parse (sqlsync_prog * prog, int argc, char **argv)
{
  short required = 0;
  int parse_pos = 0;

  int option_index = 0;
  int c = 0;
  static struct option long_options[] = {
    {"src-dsn", 1, 0, 'a'},
    {"src-user", 1, 0, 'b'},
    {"src-password", 1, 0, 'c'},

    {"dst-dsn", 1, 0, 'd'},
    {"dst-user", 1, 0, 'e'},
    {"dst-password", 1, 0, 'f'},

    {"query-file", 1, 0, 'q'},

    {"show-summary", 0, 0, 'g'},
    {"show-queries", 0, 0, 'h'},

    {"run-queries", 0, 0, 'i'},
    {"ignore-case-sensitive", 0, 0, 'j'},

    {"silent", 0, 0, 's'},
    {"tables", 1, 0, 't'},
    {0, 0, 0, 0}
  };

  opterr = 0;

  if (prog == NULL)
    return SQLSYNC_FAILED_NULL;

  prog->run_query_file = 0;
  prog->show_queries = 0;
  prog->show_summary = 0;
  prog->run_queries = 0;
  prog->silent = SQLSYNC_PRINT;
  prog->columns = NULL;
  prog->tables = NULL;

#ifdef SQLSYNC_PARAMS_FILE
  opterr = 0;
  optind = 0;
  optopt = 0;
  //optreset = 0;
  optarg = NULL;
#endif

  option_index = 0;
  while (1)
  {
    c = getopt_long (argc, argv, "ghis", long_options, &option_index);


    if (c == -1)
      break;

    switch (c)
    {
      case 'a':
        c_string_elem_append (prog->dsn_src, optarg, strlen (optarg) + 1);
        required++;
        break;

      case 'b':
        c_string_elem_append (prog->user_src, optarg, strlen (optarg) + 1);
        break;

      case 'c':
        c_string_elem_append (prog->pass_src, optarg, strlen (optarg) + 1);
        break;

      case 'd':
        c_string_elem_append (prog->dsn_dst, optarg, strlen (optarg) + 1);
        required++;
        break;

      case 'e':
        c_string_elem_append (prog->user_dst, optarg, strlen (optarg) + 1);
        break;

      case 'f':
        c_string_elem_append (prog->pass_dst, optarg, strlen (optarg) + 1);
        break;

      case 'q':
        c_string_elem_append (prog->query_file, optarg, strlen (optarg) + 1);
        prog->run_query_file = 1;
        break;

      case 'j':
        prog->cs = 1;

      case 't':
        parse_pos = optind - 1;
        break;

      case 'g':
        prog->show_summary = 1;
        break;

      case 'h':
        prog->show_queries = 1;
        break;

      case 'i':
        prog->run_queries = 1;
        break;

      case 's':
        prog->silent = SQLSYNC_SILENT;
        break;

      case '?':
      default:
        sqlsync_printf (SQLSYNC_PRINT, _tr ("Wrong parameter\n"), 0);

    }
  }
  /* set printf mask before sqlsync_parse_tables_columns 
     because this function writes some messages on display
   */
  prog->handler_data.silent = prog->silent;

  if (parse_pos > 0)
  {
    c_string_elem arg;

#ifdef DEBUG
    sqlsync_debug ("Parsing tables and columns %s\n", argv[parse_pos]);
#endif
    arg.string = argv[parse_pos];
    arg.length = strlen (argv[parse_pos]);
    sqlsync_parse_tables_columns (&arg, &prog->tables, &prog->columns);

#ifdef DEBUG
    sqlsync_debug_details ("Number of tables %ld, number of elems"
                           "in columns grid %ld\n",
                           c_vector_strings_size (prog->tables),
                           c_grid_columns_size (prog->columns));
#endif
  }

  if ((prog->show_queries | prog->run_queries) == 0)
    prog->show_summary = 1;

  if (required != 2 && prog->query_file->length == 0)
    sqlsync_printf (prog->silent, _tr ("Not enough parameters\n"), 0);

  if (prog->dsn_src->length == 0 && prog->query_file->length == 0)
  {
    sqlsync_printf (prog->silent, _tr ("Source DSN wasn't specified\n"), 0);
    sqlsync_show_available_dsns ();
    return SQLSYNC_FAILED;
  }

  if (prog->dsn_dst->length == 0 && prog->query_file->length == 0)
  {
    sqlsync_printf (prog->silent, _tr ("Destination DSN wasn't specified\n"), 0);
    sqlsync_show_available_dsns ();

    return SQLSYNC_FAILED;
  }


  return SQLSYNC_SUCCESS;
}

void
usage ()
{
  sqlsync_printf (SQLSYNC_PRINT, sqlsync_help, 0);

}


SQLSYNC_RETURN
sqlsync_prog_queries_from_file (sqlsync_prog * prog)
{
  if (prog == NULL)
    return SQLSYNC_FAILED_NULL;

  if (NOT_EMPTY_STRING (prog->query_file->string) && NOT_EMPTY_STRING (prog->dsn_src->string))
  {
    if (sqlsync_run_query_file (prog->info, SERVER_SRC, prog->query_file) != SQLSYNC_SUCCESS)
    {

#ifdef DEBUG
      sqlsync_debug ("Executing query file failed\n");
#endif

      sqlsync_printf (prog->silent, _tr ("Executing query failed\n"), 0);
      sqlsync_print_error (prog->info, SERVER_SRC);

      return SQLSYNC_FAILED;
    }
  }

  return SQLSYNC_SUCCESS;
}

SQLSYNC_RETURN
sqlsync_prog_init (sqlsync_prog * prog)
{
  SQLSYNC_RETURN ret;

  sqlsync_ignore_case_sensitive(prog->info, prog->cs == 1 ? SQLSYNC_TRUE : SQLSYNC_FALSE);

  ret = sqlsync_connect (prog->info, SERVER_SRC, prog->dsn_src, prog->user_src, prog->pass_src);
  if (SQLSYNC_IS_ERROR (ret))
  {
    sqlsync_printf (prog->silent, _tr ("Connection failed\n"), 0);
    sqlsync_print_error (prog->info, SERVER_SRC);
    return SQLSYNC_FAILED;
  }

  sqlsync_printf (prog->silent, _tr ("Connected to dsn_src server %s\n"), prog->dsn_src->string);

  if (prog->run_query_file == 1)
  {
    sqlsync_printf (prog->silent, _tr ("Running queries from file...%s\n"),
                    prog->query_file->string);

    if ((ret = sqlsync_prog_queries_from_file (prog)) != SQLSYNC_SUCCESS)
      return ret;
  }
  else
  {
    ret = sqlsync_connect (prog->info, SERVER_DST, prog->dsn_dst, prog->user_dst, prog->pass_dst);
    if (SQLSYNC_IS_ERROR (ret))
    {
      sqlsync_printf (prog->silent, _tr ("Connection failed\n"), 0);
      sqlsync_print_error (prog->info, SERVER_DST);
      return SQLSYNC_FAILED;
    }
    sqlsync_printf (prog->silent, _tr ("Connected to dsn_dst server %s\n"), prog->dsn_dst->string);
  }

  return SQLSYNC_SUCCESS;
}


int
sqlsync_main (int argc, char **argv)
{
  SQLSYNC_RETURN ret = SQLSYNC_SUCCESS;

  sqlsync_prog *prog = sqlsync_prog_create ();

  sqlsync_user_notification_register (sqlsync_lib_user_handler, &prog->handler_data);

  if (argc == 1)
  {
    usage ();
    sqlsync_prog_destroy (&prog);
    return -1;
  }

  if (parse (prog, argc, argv) != SQLSYNC_SUCCESS)
  {
    usage ();
    sqlsync_prog_destroy (&prog);
    return -1;
  }

  ret = sqlsync_prog_init (prog);
  if (ret != SQLSYNC_SUCCESS)
  {
    sqlsync_prog_destroy (&prog);
    return -2;
  }

  if (prog->run_query_file == 0)
  {
    uint32 nr_results = 0;

    nr_results = sqlsync_compare_databases (prog->info, prog->tables, prog->columns,
                                            prog->show_summary, prog->show_queries,
                                            prog->run_queries);

#ifdef DEBUG
    sqlsync_debug ("Number of results %ld\n", nr_results);
#endif

    if (nr_results == 0)
      sqlsync_printf (prog->silent, _tr ("Passed tables don't exist on both DSNs\n"), 0);

  }
  sqlsync_prog_destroy (&prog);

  return 0;
}

#ifdef SQLSYNC_PARAMS_FILE
int
read_arguments (int *argc, int argc_size, char *argv[][20])
{
  FILE *f = fopen ("sqlsync_params.txt", "r");

  if (f != NULL)
  {
    char tmp[1000];
    char t[100];
    int i = 0;
    int j = 0;
    int pos = 0;

    while (fgets (tmp, 1000, f) != 0)
    {
      i = 0;
      argv[j][i++] = strdup ("dummy");
      while (sscanf (tmp + pos, "%s", t) > 0)
      {
        argv[j][i] = strdup (t);
        pos += strlen (t) + 1;
        i++;
        if (i == 20)
          break;                //max position
      }
      argc[j] = i;
      j++;
      pos = 0;
    }
    fclose (f);
    return j;
  }
  return 0;
}

void
free_arguments (int *argc, int argc_size, char *argv[][20])
{
  int j = 0;

  for (j = 0; j < argc_size; j++)
  {
    if (argc[j] != 0)
    {
      int i = 0;

      for (i = 0; i < argc[j]; i++)
        free (argv[j][i]);
    }
  }
}

#endif


int
main (int argc, char **argv)
{

#ifdef SQLSYNC_PARAMS_FILE
  char *fargv[1000][20];
  int fargc[1000];
  int i = 0;
  int size = read_arguments (fargc, 1000, fargv);

  while (i < size)
  {
    sqlsync_main (fargc[i], fargv[i]);
    i++;

  }
  free_arguments (fargc, size, fargv);
  return 0;
#else
  return sqlsync_main (argc, argv);
#endif

}
