
use strict;

use File::Compare;
use Switch;

#
#cmd_sync_show_all
#cmd_sync_run_all
#cmd_sync_show
#cmd_sync_run
#cmd_query_run
#cmd_db_prepare
#cmd_db_erase
#cmd_exec
#

my $db_src_user="";
my $db_src_password="";
my $db_dst_user="";
my $db_dst_password="";

sub parse_line
{


  my $line = $_[0];

  my $command_queries="./scripts/queries_file_dir.pl";
  my $command_rqueries="./scripts/queries_run.pl";
  my $command_squeries="./scripts/queries_show.pl";
  my $command_r_all_queries="./scripts/queries_run_all.pl";
  my $command_s_all_queries="./scripts/queries_show_all.pl";
 
  my %cmd_run = (
                  "cmd_sync_show_all"  => $command_s_all_queries,
                  "cmd_sync_run_all"   => $command_r_all_queries,
                  "cmd_sync_show"      => $command_squeries,
                  "cmd_sync_run"       => $command_rqueries,
                  "cmd_query_run_src"      => "$command_queries  $_[1]",
                  "cmd_query_run_dst"      => "$command_queries  $_[1]",
		  "cmd_db_prepare"     => "",
                  "cmd_db_drop"        => "",
                  "cmd_exec"           => "",
                  "cmd_comment"          => "cmd_comment",
		  "db_src_user"            => "",
		  "db_src_password"        => "",
		  "db_dst_user"            => "",
		  "db_dst_password"        => ""
	  );



  my $cmd;
  my $rest;
    
  ($cmd, $rest) = split(/[ \t]+/, $line, 2);
  
  chomp($cmd);
  chomp($rest);
  
  my $usr_pass="";
  
  if ($db_src_user ne "")
  {
     $usr_pass= $usr_pass . " --src-user $db_src_user ";
  }

  if ($db_src_password ne "")
  {
     $usr_pass= $usr_pass . " --src-password $db_src_password ";
  }

  if ($db_dst_user ne "")
  {
     $usr_pass= $usr_pass . " --dst-user $db_dst_user ";
  }

  if ($db_dst_password ne "")
  {
     $usr_pass= $usr_pass . " --dst-password $db_dst_password ";
  }

  my $ret = $cmd_run{$cmd};

  switch($cmd)
  {
    case "db_src_user" { $db_src_user =$rest}
    case "db_src_password" { $db_src_password = $rest}
    case "db_dst_user" { $db_dst_user =$rest}
    case "db_dst_password" { $db_dst_password = $rest}
  }
  
  if ($cmd eq "cmd_query_run_src")
  {
     $usr_pass = "--src-user $db_src_user --src-password $db_src_password";
  }

  if ($cmd eq "cmd_query_run_dst")
  {
     $usr_pass = "--src-user $db_dst_user --src-password $db_dst_password";
  }

  if ($cmd eq "cmd_comment")
  {
    return "-- " . $rest;
  }

  my $ret = $cmd_run{$cmd};
  if ($ret ne "")
  {
    $ret ." ". $rest ." ". $usr_pass;
  }
}


sub set_lib_path
{
my $os=$^O;

  if ($os eq "MSWin32")
  {
    $ENV{'PATH'}="$ENV{'PATH'}:../lib";
  }
  else
  {
    $ENV{'LD_LIBRARY_PATH'}="$ENV{'LD_LIBRARY_PATH'}:../lib";
  }

}


sub clean_dir
{
  unlink "$_[0]/debug*";
  unlink "$_[0]/results";
  unlink "$_[0]/tests.err";
}


sub parse_file
{
  my $dir = $_[0];


  set_lib_path();

  my $handler;
  my $line;

  my $cmd_found = 0;

  open($handler, '<', "$dir/script.dat") or return;
  my $log_nr=0;
  while($line = <$handler>)
  {
  
    my $cmd = parse_line($line, $dir);
    if ($cmd eq "")
      { next;}
    
    
    if ($cmd =~ /^--/)
    {
      open(bline, ">>$dir/results");
      print bline "$cmd\n";
      close(bline);
      next;
    }

    $cmd_found = 1;
 
    my $execute="perl ". $cmd . ">>$dir/results";


    print "===================================================\n";
    print "Script: $cmd\n";
    print "===================================================\n";

    my $err;
    $err=system($execute);

    if ($err ne 0)
    {
      print "Error occured ($err) see $dir/results for more details\n";
    }
    $log_nr++;
    my $debug_log = time();
    my @dirs= split(/[\/]/,$dir);
    my $d= pop(@dirs);
    $debug_log="debug\_$d\_$log_nr\_$debug_log";
    rename('debug.txt', "$debug_log.txt");
  }
  
  if ($cmd_found == 1 && File::Compare::compare_text("$dir/expected","$dir/results") != 0) 
  {
    print "Expected results and actual are different see $dir/expected and $dir/results files for more information\n";
  }  

  close($handler);

}


my $dir=$ARGV[0];

  if ($dir eq "")
  {
    print "First argument - directory with tests i.e. testing/generic\n";
    exit;
  }

  clean_dir($dir);
  parse_file($dir);  
