#include "data_types/sqlsync_data_type.h"

#include <stdlib.h>
#include "sqlsync_debug.h"

#define TEST(message, expr) sqlsync_printf(SQLSYNC_PRINT, "%s is %s\n", message,(expr ? "OK" : "ERROR"))

#define TEST_NAME( name ) sqlsync_printf(SQLSYNC_PRINT, "\n======== %s ========\n\n", name)
#define TEST_NAME_END( name ) sqlsync_printf(SQLSYNC_PRINT, "\n======== %s end ========\n", name)


c_data_ptr
test_data_init ()
{
  c_data_ptr data_ptr = NULL;

  data_ptr = c_data_init (0, c_factory_copy, c_factory_free);
  sqlsync_printf (SQLSYNC_PRINT, "data_ptr %p\n", data_ptr);

  data_ptr = c_data_init (1, NULL, NULL);
  sqlsync_printf (SQLSYNC_PRINT, "data_ptr %p\n", data_ptr);
  return data_ptr;
}


void
test_data_boundaries (c_data_ptr container)
{
  TEST ("container size", c_data_size (container) == 0);
  TEST ("container data pos (bound check) (pos 0)", c_data_pos (container, 0) == NULL);
  TEST ("container data pos (bound check) (pos 10)", c_data_pos (container, 10) == NULL);
  TEST ("container data (bound check) delete (pos 2)",
        c_data_delete (container, 2) == SQLSYNC_FAILED_NULL);
  TEST ("container data delete (bound check) (pos 0)",
        c_data_delete (container, 0) == SQLSYNC_FAILED_NULL);
  TEST ("container data clear  (bound check) ", c_data_clear (container) == SQLSYNC_SUCCESS);
}

c_data_ptr
test_data_copy (c_data_ptr container)
{
  c_data_ptr new_container = c_data_copy (container);

  TEST ("new_container", new_container != NULL);
  return new_container;
}


void
test_data_add (c_data_ptr container)
{

  c_binary_elem elem1;
  c_binary_elem elem2;

  elem1.type = c_binary_elem_type;
  elem1.data = (uint8*)"ala";
  elem1.length = 4;
  
  elem2.type = c_binary_elem_type;
  elem2.data = (uint8*)"kot";
  elem2.length = 4;

  TEST ("container data add on 1 pos", c_data_add (container, (c_data_elem_ptr)&elem1, 1));

  TEST ("container data add on 20 pos", c_data_add (container, (c_data_elem_ptr)&elem2, 20));

  TEST ("container data add with 0 size on 20 pos",
        c_data_add (container, NULL, 20) == NULL);

  TEST ("container data add on existing elem", c_data_add (container, NULL, 20) == NULL);

  TEST ("container data add last", c_data_add_last (container, NULL) == NULL);

  TEST ("container data add reserve on 1 elem", c_data_add_reserve (container, c_data_elem_type, 1));

}

void
test_data_find (c_data_ptr container)
{
  SQLSYNC_RETURN ret = 0;

  TEST ("container data find free", c_data_find_free (container, NULL) == 0);

  TEST ("container data find with NULLs", c_data_find (container, NULL, NULL, &ret) == 0
        && ret == SQLSYNC_FAILED);

}

void
test_sqlsync_data ()
{
  c_data_ptr new_container = NULL;

  TEST_NAME ("data_init");
  c_data_ptr container = test_data_init ();

  TEST_NAME_END ("data_init");

  TEST_NAME ("data_boundaries");
  test_data_boundaries (container);
  TEST_NAME_END ("data_boundaries");

  TEST_NAME ("data_add");
  test_data_add (container);
  c_data_clear (container);
  TEST_NAME_END ("data_add");

  TEST_NAME ("data_add");
  test_data_add (container);
  TEST_NAME_END ("data_add");

  TEST_NAME ("data_find");
  test_data_find (container);
  TEST_NAME_END ("data_find");

  TEST_NAME ("data_copy");
  new_container = test_data_copy (container);
  TEST_NAME_END ("data_copy");

  TEST_NAME ("data_find");
  test_data_find (new_container);
  TEST_NAME_END ("data_find");

  c_data_destroy (&container);
  c_data_destroy (&new_container);
}
