
#include <stdlib.h>
#include "sqlsync_debug.h"

#include "data_types/sqlsync_data_types.h"

#define TEST(message, expr) sqlsync_printf(SQLSYNC_PRINT, "%s is %s\n", message,(expr ? "OK" : "ERROR"))

#define TEST_NAME( name ) sqlsync_printf(SQLSYNC_PRINT, "\n======== %s ========\n\n", name)
#define TEST_NAME_END( name ) sqlsync_printf(SQLSYNC_PRINT, "\n======== %s end ========\n", name)

c_strings_ptr
test_data_types_init ()
{
  c_strings_ptr data_ptr = NULL;

  data_ptr = c_strings_init (0);
  sqlsync_printf (SQLSYNC_PRINT, "data_ptr %p\n", data_ptr);

  data_ptr = c_strings_init (1);
  sqlsync_printf (SQLSYNC_PRINT, "data_ptr %p\n", data_ptr);
  return data_ptr;
}


void
test_data_types_boundaries (c_strings_ptr container)
{
  TEST ("container size", c_strings_size (container) == 0);
  TEST ("container data pos (bound check) (pos 0)", c_strings_pos (container, 0) == NULL);
  TEST ("container data pos (bound check) (pos 10)", c_strings_pos (container, 10) == NULL);
  TEST ("container data (bound check) delete (pos 2)",
        c_strings_delete (container, 2) == SQLSYNC_FAILED_NULL);
  TEST ("container data delete (bound check) (pos 0)",
        c_strings_delete (container, 0) == SQLSYNC_FAILED_NULL);
  TEST ("container data clear  (bound check) ", c_strings_clear (container) == SQLSYNC_SUCCESS);
}

c_strings_ptr
test_data_types_copy (c_strings_ptr container)
{
  c_strings_ptr new_container = c_strings_copy (container);

  TEST ("new_container", new_container != NULL);
  return new_container;
}



void
test_data_types_add_append ()
{
/*
  c_data_elem elem;


  elem.data = NULL;
  elem.length = 0;

  TEST ("Append data ", c_data_elem_append (&elem, "ola ma psa", 10) == SQLSYNC_SUCCESS);
  sqlsync_write (SQLSYNC_PRINT, elem.data, elem.length);
  sqlsync_printf (SQLSYNC_PRINT, "\n", 0);

  TEST ("Append data ", c_data_elem_append (&elem, ", pies..", 8) == SQLSYNC_SUCCESS);
  sqlsync_write (SQLSYNC_PRINT, elem.data, elem.length);
  sqlsync_printf (SQLSYNC_PRINT, "\n", 0);

  TEST ("Append data ", c_data_elem_append (&elem, " final", 7) == SQLSYNC_SUCCESS);
  sqlsync_write (SQLSYNC_PRINT, elem.data, elem.length);
  sqlsync_printf (SQLSYNC_PRINT, "\n", 0);

  free (elem.data);*/
}

void
test_data_types_add (c_strings_ptr container)
{
  c_strings_elem_ptr str;

  c_strings_elem tmp;

  tmp.string = "ala";
  tmp.length = 4;
  TEST ("container data add on 1 pos", c_strings_add_copy (container, &tmp, 1));

  TEST ("container data add on 20 pos", c_strings_add_copy (container, &tmp, 20));

  TEST ("container data add on existing elem",
        c_strings_add (container, NULL, 20) == NULL);

  TEST ("container data add last", c_strings_add_last (container, NULL) == NULL);

  TEST ("container data add reserve on 1 elem", c_data_add_reserve (container, c_data_elem_type, 1));

  TEST ("container data resize on 21 elem",
        c_strings_append_data (container, 21, "ala ma kota", 11) == SQLSYNC_SUCCESS);

  str = c_strings_pos (container, 21);
  sqlsync_write (SQLSYNC_PRINT, str->string, str->length);
  sqlsync_printf (SQLSYNC_PRINT, "\n", 0);

  TEST ("container data resize on 21 elem",
        c_strings_append_data (container, 21, " a las psa", 10) == SQLSYNC_SUCCESS);

  str = c_strings_pos (container, 21);
  sqlsync_write (SQLSYNC_PRINT, str->string, str->length);
  sqlsync_printf (SQLSYNC_PRINT, "\n", 0);

  TEST ("container data resize on 21 elem",
        c_strings_append_data (container, 21, ".. i tyle", 9) == SQLSYNC_SUCCESS);

  str = c_strings_pos (container, 21);
  sqlsync_write (SQLSYNC_PRINT, str->string, str->length);
  sqlsync_printf (SQLSYNC_PRINT, "\n", 0);

  test_data_types_add_append ();

}

void
test_data_types_find (c_strings_ptr container)
{
  SQLSYNC_RETURN ret = 0;
  c_strings_elem elem;

  TEST ("container data find free", c_strings_find_free (container, NULL) == 0);

  TEST ("container data find with NULLs", c_strings_find (container, NULL, &ret) == 0
        && ret == SQLSYNC_FAILED_NULL);

  elem.string = "t";
  elem.length = 2;
  TEST ("container data find with NULL (not found), ",
        c_strings_find (container, &elem, &ret) == 0 && ret == SQLSYNC_FAILED);

  elem.string = "ala ma kota a las psa.. i tyle";
  elem.length = 30;
  TEST ("container data find with NULL (found)", c_strings_find (container, &elem, &ret) == 21
        && ret == SQLSYNC_SUCCESS);
}

void
test_sqlsync_data_types ()
{
  c_strings_ptr new_container = NULL;

  TEST_NAME ("data_types_init");
  c_strings_ptr container = test_data_types_init ();

  TEST_NAME_END ("data_types_init");

  TEST_NAME ("data_types_boundaries");
  test_data_types_boundaries (container);
  TEST_NAME_END ("data_types_boundaries");

  TEST_NAME ("data_types_add");
  test_data_types_add (container);
  c_strings_clear (container);
  TEST_NAME_END ("data_types_add");

  TEST_NAME ("data_types_add");
  test_data_types_add (container);
  TEST_NAME_END ("data_types_add");

  TEST_NAME ("data_types_find");
  test_data_types_find (container);
  TEST_NAME_END ("data_types_find");

  TEST_NAME ("data_types_copy");
  new_container = test_data_types_copy (container);
  TEST_NAME_END ("data_types_copy");

  TEST_NAME ("data_types_find");
  test_data_types_find (new_container);
  TEST_NAME_END ("data_types_find");

  c_strings_destroy (&container);
  c_strings_destroy (&new_container);
}
