
#include "sqlsync_query_data.h"

#include "test_macros.h"

void
test_sqlsync_query_data_create_INSERT ()
{
  c_strings_elem table = {c_strings_elem_type, "test_table", 10, 10 };

  c_strings_elem column_name1 = {c_strings_elem_type, "column_1", 8, 8 };
  c_strings_elem column_name2 = {c_strings_elem_type, "column_2", 8, 8 };
  c_strings_elem column_name3 = {c_strings_elem_type, "column_3", 8, 8 };

  c_columns_elem column1 = {c_columns_elem_type, &column_name1, 10, 0 };
  c_columns_elem column2 = {c_columns_elem_type,  &column_name2, 10, 0 };
  c_columns_elem column3 = {c_columns_elem_type,  &column_name3, 10, 0 };
  c_strings_elem_ptr query_result = NULL;

  c_strings_elem query_ref_postgres =
    {c_strings_elem_type, "INSERT INTO \"test_table\" (\"column_1\", \"column_2\", \"column_3\") VALUES (?, ?, ?);"
    , 80, 80
  };
  c_strings_elem query_ref_mysql =
    {c_strings_elem_type, "INSERT INTO test_table (column_1, column_2, column_3) VALUES (?, ?, ?);", 72, 72 };

  c_columns_ptr columns = NULL;

  columns = c_columns_init (3);
  c_columns_add_last_copy (columns, &column1);
  c_columns_add_last_copy (columns, &column2);
  c_columns_add_last_copy (columns, &column3);


  query_result = sqlsync_create_INSERT_query (sqlsync_server_postgres, &table, columns, NULL);
  TEST ("SELECT query for PostgreSQL", c_strings_elem_cmp (query_result, &query_ref_postgres) == 0);
  c_strings_elem_free (query_result);

  query_result = sqlsync_create_INSERT_query (sqlsync_server_mysql, &table, columns, NULL);
  TEST ("SELECT query for MySQL", c_strings_elem_cmp (query_result, &query_ref_mysql) == 0);
  c_strings_elem_free (query_result);

  c_columns_destroy (&columns);

}


void
test_sqlsync_query_data_create_SELECT ()
{
  c_strings_elem table = {c_strings_elem_type, "test_table", 10, 10 };

  c_strings_elem column_name1 = {c_strings_elem_type, "column_1", 8, 8 };
  c_strings_elem column_name2 = {c_strings_elem_type, "column_2", 8, 8 };
  c_strings_elem column_name3 = {c_strings_elem_type, "column_3", 8, 8 };

  c_columns_elem column1 = {c_columns_elem_type, &column_name1, 10, 0 };
  c_columns_elem column2 = {c_columns_elem_type,  &column_name2, 10, 0 };
  c_columns_elem column3 = {c_columns_elem_type,  &column_name3, 10, 0 };
  c_strings_elem_ptr query_result = NULL;

  c_strings_elem query_ref_postgres =
    {c_strings_elem_type, "SELECT \"column_1\", \"column_2\", \"column_3\" FROM \"test_table\";", 61, 61 };
  c_strings_elem query_ref_mysql =
    {c_strings_elem_type,  "SELECT column_1, column_2, column_3 FROM test_table;", 53, 53 };

  c_columns_ptr columns = NULL;

  columns = c_columns_init (3);
  c_columns_add_last_copy (columns, &column1);
  c_columns_add_last_copy (columns, &column2);
  c_columns_add_last_copy (columns, &column3);


  query_result = sqlsync_create_SELECT_query (sqlsync_server_postgres, &table, columns);
  TEST ("SELECT query for PostgreSQL", c_strings_elem_cmp (query_result, &query_ref_postgres) == 0);
  c_strings_elem_free (query_result);


  query_result = sqlsync_create_SELECT_query (sqlsync_server_mysql, &table, columns);
  TEST ("SELECT query for MySQL", c_strings_elem_cmp (query_result, &query_ref_mysql) == 0);
  c_strings_elem_free (query_result);


  c_columns_destroy (&columns);
}

void
test_sqlsync_query_data ()
{
  test_sqlsync_query_data_create_SELECT ();
  test_sqlsync_query_data_create_INSERT ();
}
